@@ -9,6 +9,7 @@
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 $SIG{__DIE__} = sub { Carp::confess @_ };
 
@@ -16,10 +17,12 @@ use Cwd qw(abs_path);
 use File::Basename qw(dirname);
 use Getopt::Long qw(GetOptions);
 use Pod::Usage qw(pod2usage);
-use Scalar::Util qw(blessed);
 use Storable;
 
 use lib dirname($0) . '/lib';
+use lib dirname($0) . '/../lib';
+use lib dirname($0) . '/../test/lib';
+
 use BackRestDoc::Common::Doc;
 use BackRestDoc::Common::DocConfig;
 use BackRestDoc::Common::DocManifest;
@@ -28,7 +31,7 @@ use BackRestDoc::Html::DocHtmlSite;
 use BackRestDoc::Latex::DocLatex;
 use BackRestDoc::Markdown::DocMarkdown;
 
-use lib dirname($0) . '/../lib';
+use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
@@ -59,9 +62,16 @@ doc.pl [options]
    --var            Override variables defined in the XML
    --doc-path       Document path to render (manifest.xml should be located here)
    --out            Output types (html, pdf, markdown)
-   --keyword        Keyword used to filter output
    --require        Require only certain sections of the document (to speed testing)
+   --include        Include source in generation (links will reference website)
    --exclude        Exclude source from generation (links will reference website)
+
+Keyword Options:
+   --keyword        Keyword used to filter output
+   --keyword-add    Add keyword without overriding 'default' keyword
+   --dev            Add 'dev' keyword
+   --debug          Add 'debug' keyword
+   --pre            Add 'pre' keyword
 =cut
 
 ####################################################################################################################################
@@ -78,9 +88,14 @@ my $oVariableOverride = {};
 my $strDocPath;
 my @stryOutput;
 my @stryKeyword;
+my @stryKeywordAdd;
 my @stryRequire;
+my @stryInclude;
 my @stryExclude;
 my $bDeploy = false;
+my $bDev = false;
+my $bDebug = false;
+my $bPre = false;
 
 GetOptions ('help' => \$bHelp,
             'version' => \$bVersion,
@@ -88,11 +103,16 @@ GetOptions ('help' => \$bHelp,
             'log-level=s' => \$strLogLevel,
             'out=s@' => \@stryOutput,
             'keyword=s@' => \@stryKeyword,
+            'keyword-add=s@' => \@stryKeywordAdd,
             'require=s@' => \@stryRequire,
+            'include=s@' => \@stryInclude,
             'exclude=s@' => \@stryExclude,
             'no-exe', \$bNoExe,
             'deploy', \$bDeploy,
             'no-cache', \$bNoCache,
+            'dev', \$bDev,
+            'debug', \$bDebug,
+            'pre', \$bPre,
             'cache-only', \$bCacheOnly,
             'var=s%', $oVariableOverride,
             'doc-path=s', \$strDocPath)
@@ -147,7 +167,34 @@ eval
         @stryKeyword = ('default');
     }
 
-    logLevelSet(undef, uc($strLogLevel));
+    # Push added keywords
+    push(@stryKeyword, @stryKeywordAdd);
+
+    # If --dev passed then add the dev keyword
+    if ($bDev)
+    {
+        push(@stryKeyword, 'dev');
+    }
+
+    # If --debug passed then add the debug keyword
+    if ($bDebug)
+    {
+        push(@stryKeyword, 'debug');
+    }
+
+    # If --pre passed then add the pre keyword
+    if ($bPre)
+    {
+        push(@stryKeyword, 'pre');
+    }
+
+    # Doesn't make sense to pass include and exclude
+    if (@stryInclude > 0 && @stryExclude > 0)
+    {
+        confess "cannot specify both --include and --exclude";
+    }
+
+    logLevelSet(undef, uc($strLogLevel), OFF);
 
     # Get the base path
     my $strBasePath = abs_path(dirname($0));
@@ -168,7 +215,7 @@ eval
 
     # Load the manifest
     my $oManifest = new BackRestDoc::Common::DocManifest(
-        \@stryKeyword, \@stryRequire, \@stryExclude, $oVariableOverride, $strDocPath, $bDeploy, $bCacheOnly);
+        \@stryKeyword, \@stryRequire, \@stryInclude, \@stryExclude, $oVariableOverride, $strDocPath, $bDeploy, $bCacheOnly);
 
     if (!$bNoCache)
     {
@@ -266,20 +313,24 @@ eval
     {
         $oManifest->cacheWrite();
     }
-};
+
+    # Exit with success
+    exit 0;
+}
 
 ####################################################################################################################################
 # Check for errors
 ####################################################################################################################################
-if ($@)
+or do
 {
-    my $oMessage = $@;
+    # If a backrest exception then return the code
+    exit $EVAL_ERROR->code() if (isException($EVAL_ERROR));
 
-    # If a backrest exception then return the code - don't confess
-    if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
-    {
-        exit $oMessage->code();
-    }
+    # Else output the unhandled error
+    print $EVAL_ERROR;
+    exit ERROR_UNHANDLED;
+};
 
-    confess $oMessage;
-}
+# It shouldn't be possible to get here
+&log(ASSERT, 'execution reached invalid location in ' . __FILE__ . ', line ' . __LINE__);
+exit ERROR_ASSERT;
