@@ -6,13 +6,12 @@ package BackRestDoc::Common::Doc;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use File::Basename qw(dirname);
 use Scalar::Util qw(blessed);
-
 use XML::Checker::Parser;
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::FileCommon;
@@ -67,14 +66,16 @@ sub new
                 };
 
                 $oTree = $oParser->parsefile($self->{strFileName});
-            };
 
-            # Report any error that stopped parsing
-            if ($@)
-            {
-                $@ =~ s/at \/.*?$//s;               # remove module line number
-                die "malformed xml in '$self->{strFileName}':\n" . trim($@);
+                return true;
             }
+            # Report any error that stopped parsing
+            or do
+            {
+                my $strException = $EVAL_ERROR;
+                $strException =~ s/at \/.*?$//s;               # remove module line number
+                die "malformed xml in '$self->{strFileName}':\n" . trim($strException);
+            };
 
             # Parse and build the doc
             $self->{oDoc} = $self->build($self->parse(${$oTree}[0], ${$oTree}[1]));
@@ -252,7 +253,7 @@ sub build
             {
                 $$oOut{field}{text} = $oSub;
             }
-            elsif (defined($$oSub{value}) && !defined($$oSub{param}))
+            elsif ((defined($$oSub{value}) && !defined($$oSub{param})) && $strName ne 'code-block')
             {
                 $$oOut{field}{$strName} = $$oSub{value};
             }
