@@ -11,7 +11,6 @@ use Exporter qw(import);
     our @EXPORT = qw();
 use File::Basename qw(dirname);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
@@ -77,7 +76,7 @@ sub process
 
     foreach my $strCommand (sort(keys(%{commandHashGet()})))
     {
-        if ($strCommand eq CMD_REMOTE)
+        if ($strCommand eq CMD_REMOTE || $strCommand eq CMD_LOCAL)
         {
             next;
         }
@@ -96,7 +95,7 @@ sub process
 
     foreach my $strOption (sort(keys(%{$oOptionRule})))
     {
-        if ($strOption =~ /^test/ || $strOption eq 'no-fork')
+        if ($strOption =~ /^test/ || $strOption eq 'fork')
         {
             next;
         }
@@ -605,7 +604,11 @@ sub manGet
 
             if (defined($strDefault))
             {
-                if ($$hOptionRule{$strOption}{&OPTION_RULE_TYPE} eq &OPTION_TYPE_BOOLEAN)
+                if ($strOption eq OPTION_BACKUP_CMD || $strOption eq OPTION_DB_CMD)
+                {
+                    $strDefault = BACKREST_EXE;
+                }
+                elsif ($$hOptionRule{$strOption}{&OPTION_RULE_TYPE} eq &OPTION_TYPE_BOOLEAN)
                 {
                     $strDefault = $strDefault ? 'y' : 'n';
                 }
@@ -917,13 +920,9 @@ sub helpOptionGet
     {
         my $strDefault;
 
-        if ($strOption eq OPTION_CONFIG || $strOption eq OPTION_CONFIG_REMOTE)
+        if ($strOption eq OPTION_BACKUP_CMD || $strOption eq OPTION_DB_CMD)
         {
-            $strDefault = '/etc/{[backrest-exe]}.conf';
-        }
-        elsif ($strOption eq OPTION_COMMAND_REMOTE)
-        {
-            $strDefault = '[INSTALL-PATH]/{[backrest-exe]}';
+            $strDefault = '[INSTALL-PATH]/' . BACKREST_EXE;
         }
         else
         {
