@@ -10,10 +10,9 @@ use Carp qw(confess);
 
 use Exporter qw(import);
     our @EXPORT = qw();
-use File::Basename qw(dirname);
 use Storable qw(dclone);
 
-use lib dirname($0) . '/../lib';
+use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
@@ -21,7 +20,6 @@ use pgBackRest::Config::Config;
 use pgBackRest::FileCommon;
 use pgBackRest::Version;
 
-use lib dirname($0) . '/../test/lib';
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::HostTest;
 use pgBackRestTest::Common::HostGroupTest;
@@ -69,6 +67,8 @@ sub new
 
     $self->{bExe} = $bExe;
 
+    $self->{iCmdLineLen} = $self->{oDoc}->paramGet('cmd-line-len', false, 80);
+
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -107,6 +107,7 @@ sub executeKey
 
     # Format and split command
     $strCommand =~ s/[ ]*\n[ ]*/ \\\n    /smg;
+    $strCommand =~ s/ \\\@ \\//smg;
     my @stryCommand = split("\n", $strCommand);
 
     my $hCacheKey =
@@ -188,16 +189,17 @@ sub execute
         # Make sure that no lines are greater than 80 chars
         foreach my $strLine (split("\n", $strCommand))
         {
-            if (length(trim($strLine)) > 80)
+            if (length(trim($strLine)) > $self->{iCmdLineLen})
             {
-                confess &log(ERROR, "command has a line > 80 characters:\n${strCommand}\noffending line: ${strLine}");
+                confess &log(ERROR,
+                    "command has a line > $self->{iCmdLineLen} characters:\n${strCommand}\noffending line: ${strLine}");
             }
         }
     }
 
     &log(DEBUG, ('    ' x $iIndent) . "execute: $strCommand");
 
-    if (!$oCommand->paramTest('skip', 'y'))
+    if ($self->{oManifest}->variableReplace($oCommand->paramGet('skip', false, 'n')) ne 'y')
     {
         if ($self->{bExe} && $self->isRequired($oSection))
         {
@@ -235,7 +237,7 @@ sub execute
                     if ($strCommand =~ / pgbackrest /)
                     {
                         $strOutput =~ s/^                             //smg;
-                        $strOutput =~ s/^[0-9]{4}-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-6][0-9]:[0-6][0-9]\.[0-9]{3} T[0-9]{2} //smg;
+                        $strOutput =~ s/^[0-9]{4}-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-6][0-9]:[0-6][0-9]\.[0-9]{3} //smg;
                     }
 
                     # my @stryOutput = split("\n", $strOutput);
@@ -505,8 +507,8 @@ sub backrestConfig
 
             foreach my $oOption ($oConfig->nodeList('backrest-config-option'))
             {
-                my $strSection = $oOption->paramGet('section');
-                my $strKey = $oOption->paramGet('key');
+                my $strSection = $self->{oManifest}->variableReplace($oOption->paramGet('section'));
+                my $strKey = $self->{oManifest}->variableReplace($oOption->paramGet('key'));
                 my $strValue;
 
                 if (!$oOption->paramTest('remove', 'y'))
@@ -531,13 +533,16 @@ sub backrestConfig
                     my $oOption = optionRuleGet();
 
                     # Make sure the specified option exists
-                    if (!defined($$oOption{$strKey}))
-                    {
-                        confess &log(ERROR, "option ${strKey} does not exist");
-                    }
+                    # ??? This is too simplistic to handle new indexed options.  The check below works for now but it would be good
+                    # ??? to bring back more sophisticated checking in the future.
+                    # if (!defined($$oOption{$strKey}))
+                    # {
+                    #     confess &log(ERROR, "option ${strKey} does not exist");
+                    # }
 
                     # If this option is a hash and the value is already set then append to the array
-                    if ($$oOption{$strKey}{&OPTION_RULE_TYPE} eq OPTION_TYPE_HASH &&
+                    if (defined($$oOption{$strKey}) &&
+                        $$oOption{$strKey}{&OPTION_RULE_TYPE} eq OPTION_TYPE_HASH &&
                         defined(${$self->{config}}{$strHostName}{$$hCacheKey{file}}{$strSection}{$strKey}))
                     {
                         my @oValue = ();
@@ -567,14 +572,28 @@ sub backrestConfig
                 }
             }
 
-            my $strLocalFile = "/home/vagrant/data/db-master/etc/pgbackrest.conf";
+            my $strLocalFile = "/home/vagrant/data/pgbackrest.conf";
 
             # Save the ini file
             iniSave($strLocalFile, $self->{config}{$strHostName}{$$hCacheKey{file}}, true);
 
             $oHost->copyTo($strLocalFile, $$hCacheKey{file}, $oConfig->paramGet('owner', false, 'postgres:postgres'), '640');
 
-            $strConfig = fileStringRead($strLocalFile);
+            # Remove the log-console-stderr option before pushing into the cache
+            # ??? This is not very pretty and should be replaced with a general way to hide config options
+            my $oConfigClean = dclone($self->{config}{$strHostName}{$$hCacheKey{file}});
+            delete($$oConfigClean{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_LEVEL_STDERR});
+
+            if (keys(%{$$oConfigClean{&CONFIG_SECTION_GLOBAL}}) == 0)
+            {
+                delete($$oConfigClean{&CONFIG_SECTION_GLOBAL});
+            }
+
+            iniSave("${strLocalFile}.clean", $oConfigClean, true);
+
+            # Push config file into the cache
+            $strConfig = fileStringRead("${strLocalFile}.clean");
+
             my @stryConfig = undef;
 
             if (trim($strConfig) ne '')
@@ -648,7 +667,7 @@ sub postgresConfig
                 confess &log(ERROR, "cannot configure postgres on host ${strHostName} because the host does not exist");
             }
 
-            my $strLocalFile = '/home/vagrant/data/db-master/etc/postgresql.conf';
+            my $strLocalFile = '/home/vagrant/data/postgresql.conf';
             $oHost->copyFrom($$hCacheKey{file}, $strLocalFile);
 
             if (!defined(${$self->{'pg-config'}}{$strHostName}{$$hCacheKey{file}}{base}) && $self->{bExe})
@@ -814,22 +833,22 @@ sub cachePop
 
         if (!defined($hCache))
         {
-            confess &log(ERROR, 'unable to get index from cache', -1);
+            confess &log(ERROR, 'unable to get index from cache', ERROR_FILE_INVALID);
         }
 
         if (!defined($$hCache{key}))
         {
-            confess &log(ERROR, 'unable to get key from cache', -1);
+            confess &log(ERROR, 'unable to get key from cache', ERROR_FILE_INVALID);
         }
 
         if (!defined($$hCache{type}))
         {
-            confess &log(ERROR, 'unable to get type from cache', -1);
+            confess &log(ERROR, 'unable to get type from cache', ERROR_FILE_INVALID);
         }
 
         if ($$hCache{type} ne $strCacheType)
         {
-            confess &log(ERROR, 'types do not match, cache is invalid', -1);
+            confess &log(ERROR, 'types do not match, cache is invalid', ERROR_FILE_INVALID);
         }
 
         if ($oJSON->encode($$hCache{key}) ne $oJSON->encode($hCacheKey))
@@ -837,7 +856,7 @@ sub cachePop
             confess &log(ERROR,
                 "keys at index $self->{iCacheIdx} do not match, cache is invalid." .
                 "\ncache key: " . $oJSON->encode($$hCache{key}) .
-                "\ncurrent key: " . $oJSON->encode($hCacheKey), -1);
+                "\ncurrent key: " . $oJSON->encode($hCacheKey), ERROR_FILE_INVALID);
         }
 
         $bCacheHit = true;
@@ -957,14 +976,14 @@ sub sectionChildProcess
 
                 my $oHost = new pgBackRestTest::Common::HostTest(
                     $$hCacheKey{name}, "doc-$$hCacheKey{name}", $$hCacheKey{image}, $$hCacheKey{user}, $$hCacheKey{os},
-                    [$$hCacheKey{mount}]);
+                    defined($$hCacheKey{mount}) ? [$$hCacheKey{mount}] : undef);
 
                 $self->{host}{$$hCacheKey{name}} = $oHost;
                 $self->{oManifest}->variableSet("host-$$hCacheKey{name}-ip", $oHost->{strIP}, true);
                 $$hCacheValue{ip} = $oHost->{strIP};
 
                 # Execute cleanup commands
-                foreach my $oExecute ($oChild->nodeList('execute'))
+                foreach my $oExecute ($oChild->nodeList('execute', false))
                 {
                     $self->execute($oSection, $$hCacheKey{name}, $oExecute, $iDepth + 1, false);
                 }
