@@ -12,9 +12,7 @@ use Exporter qw(import);
     our @EXPORT = qw();
 use File::Basename qw(dirname);
 use JSON::PP;
-use Scalar::Util qw(blessed);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::FileCommon;
@@ -28,7 +26,13 @@ use constant FILE_MANIFEST                                          => 'manifest
 # Render constants
 ####################################################################################################################################
 use constant RENDER                                                 => 'render';
+use constant RENDER_COMPACT                                         => 'compact';
+    push @EXPORT, qw(RENDER_COMPACT);
 use constant RENDER_FILE                                            => 'file';
+use constant RENDER_MENU                                            => 'menu';
+    push @EXPORT, qw(RENDER_MENU);
+use constant RENDER_PRETTY                                          => 'pretty';
+    push @EXPORT, qw(RENDER_PRETTY);
 
 use constant RENDER_TYPE                                            => 'type';
 use constant RENDER_TYPE_HTML                                       => 'html';
@@ -54,6 +58,7 @@ sub new
         my $strOperation,
         $self->{stryKeyword},
         $self->{stryRequire},
+        $self->{stryInclude},
         $self->{stryExclude},
         my $oVariableOverride,
         $self->{strDocPath},
@@ -65,6 +70,7 @@ sub new
             __PACKAGE__ . '->new', \@_,
             {name => 'stryKeyword'},
             {name => 'stryRequire'},
+            {name => 'stryInclude'},
             {name => 'stryExclude'},
             {name => 'oVariableOverride', required => false},
             {name => 'strDocPath', required => false},
@@ -130,6 +136,9 @@ sub new
 
         # Get the file param
         $${oRenderHash}{file} = $oRender->paramGet(RENDER_FILE, false);
+        $${oRenderHash}{&RENDER_COMPACT} = $oRender->paramGet(RENDER_COMPACT, false, 'n') eq 'y' ? true : false;
+        $${oRenderHash}{&RENDER_PRETTY} = $oRender->paramGet(RENDER_PRETTY, false, 'n') eq 'y' ? true : false;
+        $${oRenderHash}{&RENDER_MENU} = false;
 
         logDebugMisc
         (
@@ -157,14 +166,14 @@ sub new
                 next;
             }
 
-            $$oRenderOutHash{source} = $strSource;
-
-            # Get the menu caption
-            if (defined($oRenderOut->paramGet('menu', false)) && $strType ne RENDER_TYPE_HTML)
+            # Skip sources not in include list
+            if (@{$self->{stryInclude}} > 0 && !grep(/^$strSource$/, @{$self->{stryInclude}}))
             {
-                confess &log(ERROR, "menu is only valid with html render type");
+                next;
             }
 
+            $$oRenderOutHash{source} = $strSource;
+
             # Get the filename
             if (defined($oRenderOut->paramGet('file', false)))
             {
@@ -178,8 +187,16 @@ sub new
                 }
             }
 
+            # Get the menu caption
+            if (defined($oRenderOut->paramGet('menu', false)) && $strType ne RENDER_TYPE_HTML)
+            {
+                confess &log(ERROR, "menu is only valid with html render type");
+            }
+
             if (defined($oRenderOut->paramGet('menu', false)))
             {
+                $${oRenderHash}{&RENDER_MENU} = true;
+
                 if ($strType eq RENDER_TYPE_HTML)
                 {
                     $$oRenderOutHash{menu} = $oRenderOut->paramGet('menu', false);
