@@ -9,9 +9,7 @@ use Carp qw(confess);
 
 use Exporter qw(import);
     our @EXPORT = qw();
-use File::Basename qw(dirname);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
@@ -261,6 +259,7 @@ sub docGet
     # Create the doc
     my $oDoc = new BackRestDoc::Common::Doc();
     $oDoc->paramSet('title', $self->{oDoc}->paramGet('title'));
+    $oDoc->paramSet('toc-number', $self->{oDoc}->paramGet('toc-number'));
 
     # Set the description for use as a meta tag
     $oDoc->fieldSet('description', $self->{oDoc}->fieldGet('description'));
@@ -282,6 +281,9 @@ sub docGet
         # Get the release version
         my $strVersion = $oRelease->paramGet('version');
 
+        # Display versions in TOC?
+        my $bTOC = true;
+
         # Create a release section
         if ($strVersion =~ /dev$/)
         {
@@ -320,6 +322,7 @@ sub docGet
             }
 
             $iUnsupportedReleaseTotal++;
+            $bTOC = false;
         }
 
         # Format the date
@@ -345,7 +348,7 @@ sub docGet
         }
 
         # Add section and titles
-        my $oReleaseSection = $oSection->nodeAdd('section', undef, {id => $strVersion});
+        my $oReleaseSection = $oSection->nodeAdd('section', undef, {id => $strVersion, toc => !$bTOC ? 'n' : undef});
         $oReleaseSection->paramSet(XML_SECTION_PARAM_ANCHOR, XML_SECTION_PARAM_ANCHOR_VALUE_NOINHERIT);
 
         $oReleaseSection->nodeAdd('title')->textSet(
