@@ -8,14 +8,9 @@ use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
 
-use Data::Dumper;
 use Exporter qw(import);
     our @EXPORT = qw();
-use File::Basename qw(dirname);
-use File::Copy;
-use Storable qw(dclone);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
@@ -76,8 +71,7 @@ sub process
     my $strLatex;
 
     # Initialize page
-    my $strTitle = "{[project]}" .
-                   (defined($oPage->paramGet('title', false)) ? ' ' . $oPage->paramGet('title') : '');
+    my $strTitle = $oPage->paramGet('title');
     my $strSubTitle = $oPage->paramGet('subtitle', false);
 
     # Render sections
@@ -330,6 +324,18 @@ sub sectionProcess
 
             $strLatex .= "\n" . $self->processText($oDescription) . "\n";
         }
+        # Add a list
+        elsif ($oChild->nameGet() eq 'list')
+        {
+            $strLatex .= "\n\\begin{itemize}";
+
+            foreach my $oListItem ($oChild->nodeList())
+            {
+                $strLatex .= "\n    \\item " . $self->processText($oListItem->textGet());
+            }
+
+            $strLatex .= "\n\\end{itemize}";
+        }
         # Add/remove config options
         elsif ($oChild->nameGet() eq 'backrest-config' || $oChild->nameGet() eq 'postgres-config')
         {
