@@ -15,7 +15,6 @@ use File::Basename qw(dirname);
 use File::Copy;
 use Storable qw(dclone);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::ConfigHelp;
@@ -73,8 +72,7 @@ sub process
     my $oPage = $self->{oDoc};
 
     # Initialize page
-    my $strMarkdown = "# {[project]}" .
-                   (defined($oPage->paramGet('title', false)) ? ' ' . $oPage->paramGet('title') : '');
+    my $strMarkdown = "# " . $oPage->paramGet('title');
 
     if (defined($oPage->paramGet('subtitle', false)))
     {
@@ -281,7 +279,12 @@ sub sectionProcess
         {
             if ($oChild->paramTest('title'))
             {
-                $strMarkdown .= "\n\n_" . $oChild->paramGet('title') . "_:";
+                if (defined($strLastChild) && $strLastChild ne 'code-block')
+                {
+                    $strMarkdown .= "\n";
+                }
+
+                $strMarkdown .= "\n_" . $oChild->paramGet('title') . "_:";
             }
 
             $strMarkdown .= "\n```\n" . trim($oChild->valueGet()) . "\n```";
@@ -334,11 +337,9 @@ sub sectionProcess
         # Add a list
         elsif ($oChild->nameGet() eq 'list')
         {
-            $strMarkdown .= "\n";
-
             foreach my $oListItem ($oChild->nodeList())
             {
-                $strMarkdown .= "\n- " . $self->processText($oListItem->textGet());
+                $strMarkdown .= "\n\n- " . $self->processText($oListItem->textGet());
             }
         }
         # Add a subsection
