@@ -9,6 +9,7 @@
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 $SIG{__DIE__} = sub { Carp::confess @_ };
 
@@ -16,10 +17,12 @@ use Cwd qw(abs_path);
 use File::Basename qw(dirname);
 use Getopt::Long qw(GetOptions);
 use Pod::Usage qw(pod2usage);
-use Scalar::Util qw(blessed);
 use Storable;
 
 use lib dirname($0) . '/lib';
+use lib dirname($0) . '/../lib';
+use lib dirname($0) . '/../test/lib';
+
 use BackRestDoc::Common::Doc;
 use BackRestDoc::Common::DocConfig;
 use BackRestDoc::Common::DocManifest;
@@ -28,14 +31,13 @@ use BackRestDoc::Html::DocHtmlSite;
 use BackRestDoc::Latex::DocLatex;
 use BackRestDoc::Markdown::DocMarkdown;
 
-use lib dirname($0) . '/../lib';
+use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
 use pgBackRest::FileCommon;
 use pgBackRest::Version;
 
-use lib dirname($0) . '/../test/lib';
 use pgBackRestTest::Common::ExecuteTest;
 
 ####################################################################################################################################
@@ -114,7 +116,7 @@ eval
         $strLogLevel = 'error';
     }
 
-    logLevelSet(undef, uc($strLogLevel));
+    logLevelSet(undef, uc($strLogLevel), OFF);
 
     # Set the paths
     my $strDocPath = dirname(abs_path($0));
@@ -130,6 +132,9 @@ eval
         # Remove permanent cache file
         fileRemove("${strDocPath}/resource/exe.cache", true);
 
+        # Remove all docker containers to get consistent IP address assignments
+        executeTest('docker rm -f $(docker ps -a -q)');
+
         # Generate deployment docs for RHEL/Centos 6
         &log(INFO, "Generate RHEL/CentOS 6 documentation");
 
@@ -140,7 +145,7 @@ eval
         &log(INFO, "Generate Debian/Ubuntu documentation");
 
         executeTest("${strDocExe} --deploy");
-        executeTest("${strDocExe} --deploy --cache-only --out=man --out=html --var=project-url-root=index.html --exclude=backlog");
+        executeTest("${strDocExe} --deploy --cache-only --out=man --out=html --var=project-url-root=index.html");
     }
 
     if ($bDeploy)
@@ -150,7 +155,7 @@ eval
 
         executeTest(
             $strDocExe . ($bDev ? '' : ' --deploy --cache-only') . ' --out=html --var=project-url-root=index.html' .
-            ($bDev ? ' --keyword=default --keyword=dev' :  ' --exclude=backlog --exclude=release'));
+            ($bDev ? ' --keyword=default --keyword=dev' :  ' --exclude=release'));
 
         # Deploy to server
         &log(INFO, '...Deploy to server');
@@ -176,20 +181,24 @@ eval
         executeTest("ssh ${strHost} find ${strPath} -type d -exec chmod 550 {} +");
         executeTest("ssh ${strHost} find ${strPath} -type f -exec chmod 440 {} +");
     }
-};
+
+    # Exit with success
+    exit 0;
+}
 
 ####################################################################################################################################
 # Check for errors
 ####################################################################################################################################
-if ($@)
+or do
 {
-    my $oMessage = $@;
+    # If a backrest exception then return the code
+    exit $EVAL_ERROR->code() if (isException($EVAL_ERROR));
 
-    # If a backrest exception then return the code - don't confess
-    if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
-    {
-        exit $oMessage->code();
-    }
+    # Else output the unhandled error
+    print $EVAL_ERROR;
+    exit ERROR_UNHANDLED;
+};
 
-    confess $oMessage;
-}
+# It shouldn't be possible to get here
+&log(ASSERT, 'execution reached invalid location in ' . __FILE__ . ', line ' . __LINE__);
+exit ERROR_ASSERT;
