@@ -6,6 +6,7 @@ package pgBackRest::Archive;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use Exporter qw(import);
     our @EXPORT = qw();
@@ -13,39 +14,20 @@ use Fcntl qw(SEEK_CUR O_RDONLY O_WRONLY O_CREAT);
 use File::Basename qw(dirname basename);
 use Scalar::Util qw(blessed);
 
-use lib dirname($0);
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Lock;
 use pgBackRest::Common::Log;
+use pgBackRest::ArchiveCommon;
 use pgBackRest::ArchiveInfo;
 use pgBackRest::Common::String;
 use pgBackRest::Common::Wait;
 use pgBackRest::Config::Config;
 use pgBackRest::Db;
+use pgBackRest::DbVersion;
 use pgBackRest::File;
 use pgBackRest::FileCommon;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_ARCHIVE                                             => 'Archive';
-
-use constant OP_ARCHIVE_GET                                         => OP_ARCHIVE . '->get';
-use constant OP_ARCHIVE_GET_ARCHIVE_ID                              => OP_ARCHIVE . '->getArchiveId';
-    push @EXPORT, qw(OP_ARCHIVE_GET_ARCHIVE_ID);
-use constant OP_ARCHIVE_GET_CHECK                                   => OP_ARCHIVE . '->getCheck';
-    push @EXPORT, qw(OP_ARCHIVE_GET_CHECK);
-use constant OP_ARCHIVE_GET_PROCESS                                 => OP_ARCHIVE . '->getProcess';
-use constant OP_ARCHIVE_NEW                                         => OP_ARCHIVE . '->new';
-use constant OP_ARCHIVE_PROCESS                                     => OP_ARCHIVE . '->process';
-use constant OP_ARCHIVE_PUSH                                        => OP_ARCHIVE . '->pushProcess';
-use constant OP_ARCHIVE_PUSH_CHECK                                  => OP_ARCHIVE . '->pushCheck';
-    push @EXPORT, qw(OP_ARCHIVE_PUSH_CHECK);
-use constant OP_ARCHIVE_PUSH_PROCESS                                => OP_ARCHIVE . '->pushProcess';
-use constant OP_ARCHIVE_RANGE                                       => OP_ARCHIVE . '->range';
-use constant OP_ARCHIVE_WAL_FILE_NAME                               => OP_ARCHIVE . '->walFileName';
-use constant OP_ARCHIVE_WAL_INFO                                    => OP_ARCHIVE . '->walInfo';
-use constant OP_ARCHIVE_XFER                                        => OP_ARCHIVE . '->xfer';
+use pgBackRest::Protocol::Common;
+use pgBackRest::Protocol::Protocol;
 
 ####################################################################################################################################
 # PostgreSQL WAL magic
@@ -63,12 +45,6 @@ my $oWalMagicHash =
     hex('0xD093') => PG_VERSION_96,
 };
 
-####################################################################################################################################
-# PostgreSQL WAL system id offset
-####################################################################################################################################
-use constant PG_WAL_SYSTEM_ID_OFFSET_GTE_93                         => 20;
-use constant PG_WAL_SYSTEM_ID_OFFSET_LT_93                          => 12;
-
 ####################################################################################################################################
 # constructor
 ####################################################################################################################################
@@ -77,14 +53,7 @@ sub new
     my $class = shift;          # Class name
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_ARCHIVE_NEW
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->new');
 
     # Create the class hash
     my $self = {};
@@ -108,14 +77,7 @@ sub process
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_ARCHIVE_PROCESS
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->process');
 
     my $iResult;
 
@@ -151,14 +113,7 @@ sub getProcess
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_ARCHIVE_GET_PROCESS
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->getProcess');
 
     # Make sure the archive file is defined
     if (!defined($ARGV[1]))
@@ -169,7 +124,7 @@ sub getProcess
     # Make sure the destination file is defined
     if (!defined($ARGV[2]))
     {
-        confess &log(ERROR, 'WAL segment destination no provided', ERROR_PARAM_REQUIRED);
+        confess &log(ERROR, 'WAL segment destination not provided', ERROR_PARAM_REQUIRED);
     }
 
     # Info for the Postgres log
@@ -205,7 +160,7 @@ sub walFileName
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_WAL_FILE_NAME, \@_,
+            __PACKAGE__ . '->walFileName', \@_,
             {name => 'oFile'},
             {name => 'strArchiveId'},
             {name => 'strWalSegment'},
@@ -284,7 +239,7 @@ sub walInfo
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_WAL_INFO, \@_,
+            __PACKAGE__ . '->walInfo', \@_,
             {name => 'strWalFile'}
         );
 
@@ -372,7 +327,7 @@ sub get
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_GET, \@_,
+            __PACKAGE__ . '->get', \@_,
             {name => 'strSourceArchive'},
             {name => 'strDestinationFile'}
         );
@@ -384,18 +339,19 @@ sub get
     (
         optionGet(OPTION_STANZA),
         optionGet(OPTION_REPO_PATH),
-        optionRemoteType(),
-        protocolGet()
+        protocolGet(BACKUP)
     );
 
     # If the destination file path is not absolute then it is relative to the db data path
     if (index($strDestinationFile, '/',) != 0)
     {
-        # ??? If db-path is required this is can be removed.
-        # ??? db-path should be added as a requirement for the remote settings work.
         if (!optionTest(OPTION_DB_PATH))
         {
-            confess &log(ERROR, 'option db-path must be set when relative xlog paths are used');
+            confess &log(
+                ERROR, "option db-path must be set when relative xlog paths are used\n" .
+                "HINT: Are \%f and \%p swapped when passed to archive-get?\n" .
+                'HINT: PostgreSQL generally passes an absolute path to archive-push but in some environments does not, in which' .
+                " case the 'db-path' option must be specified (this is rare).");
         }
 
         $strDestinationFile = optionGet(OPTION_DB_PATH) . "/${strDestinationFile}";
@@ -407,9 +363,8 @@ sub get
 
     # If there are no matching archive files then there are two possibilities:
     # 1) The end of the archive stream has been reached, this is normal and a 1 will be returned
-    # 2) There is a hole in the archive stream and a hard error should be returned.  However, holes are possible due to
-    #    async archiving and threading - so when to report a hole?  Since a hard error will cause PG to terminate, for now
-    #    treat as case #1.
+    # 2) There is a hole in the archive stream and a hard error should be returned.  However, holes are possible due to async
+    #    archiving - so when to report a hole?  Since a hard error will cause PG to terminate, for now treat as case #1.
     my $iResult = 0;
 
     if (!defined($strArchiveFile))
@@ -455,7 +410,7 @@ sub getCheck
     ) =
         logDebugParam
     (
-        OP_ARCHIVE_GET_CHECK, \@_,
+        __PACKAGE__ . '->getCheck', \@_,
         {name => 'oFile'},
         {name => 'strDbVersion', required => false},
         {name => 'ullDbSysId', required => false}
@@ -467,20 +422,12 @@ sub getCheck
     if (!defined($strDbVersion) || !defined($ullDbSysId) )
     {
         # get DB info for comparison
-        ($strDbVersion, my $iControlVersion, my $iCatalogVersion, $ullDbSysId) =
-            (new pgBackRest::Db())->info($oFile, optionGet(OPTION_DB_PATH));
+        ($strDbVersion, my $iControlVersion, my $iCatalogVersion, $ullDbSysId) = dbMasterGet()->info();
     }
 
     if ($oFile->isRemote(PATH_BACKUP_ARCHIVE))
     {
-        # Build param hash
-        my %oParamHash;
-
-        # Pass the database information to the remote server
-        $oParamHash{'db-version'} = $strDbVersion;
-        $oParamHash{'db-sys-id'} = $ullDbSysId;
-
-        $strArchiveId = $oFile->{oProtocol}->cmdExecute(OP_ARCHIVE_GET_CHECK, \%oParamHash, true);
+        $strArchiveId = $oFile->{oProtocol}->cmdExecute(OP_ARCHIVE_GET_CHECK, [$strDbVersion, $ullDbSysId], true);
     }
     else
     {
@@ -497,6 +444,63 @@ sub getCheck
     );
 }
 
+####################################################################################################################################
+# getBackupInfoCheck
+#
+# Check the backup.info file, if it exists, to confirm the DB version, system-id, control and catalog numbers match the database.
+####################################################################################################################################
+sub getBackupInfoCheck
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $oFile,
+        $strDbVersion,
+        $iControlVersion,
+        $iCatalogVersion,
+        $ullDbSysId
+    ) =
+        logDebugParam
+    (
+        __PACKAGE__ . '->getBackupInfoCheck', \@_,
+        {name => 'oFile'},
+        {name => 'strDbVersion', required => false},
+        {name => 'iControlVersion', required => false},
+        {name => 'iCatalogVersion', required => false},
+        {name => 'ullDbSysId', required => false}
+    );
+
+    # If the db info are not passed, then we need to retrieve the database information
+    my $iDbHistoryId;
+
+    if (!defined($strDbVersion) || !defined($iControlVersion) || !defined($iCatalogVersion) || !defined($ullDbSysId))
+    {
+        # get DB info for comparison
+        ($strDbVersion, $iControlVersion, $iCatalogVersion, $ullDbSysId) = dbMasterGet()->info();
+    }
+
+    if ($oFile->isRemote(PATH_BACKUP))
+    {
+        $iDbHistoryId = $oFile->{oProtocol}->cmdExecute(
+            OP_ARCHIVE_GET_BACKUP_INFO_CHECK, [$strDbVersion, $iControlVersion, $iCatalogVersion, $ullDbSysId]);
+    }
+    else
+    {
+        $iDbHistoryId = (new pgBackRest::BackupInfo($oFile->pathGet(PATH_BACKUP_CLUSTER)))->check(
+            $strDbVersion, $iControlVersion, $iCatalogVersion, $ullDbSysId);
+    }
+
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'iDbHistoryId', value => $iDbHistoryId, trace => true}
+    );
+}
+
 ####################################################################################################################################
 # getArchiveId
 #
@@ -510,14 +514,7 @@ sub getArchiveId
     my $oFile = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_ARCHIVE_GET_ARCHIVE_ID
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->getArchiveId');
 
     my $strArchiveId;
 
@@ -546,23 +543,16 @@ sub pushProcess
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_ARCHIVE_PUSH_PROCESS
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->pushProcess');
 
     # Make sure the archive push command happens on the db side
-    if (optionRemoteTypeTest(DB))
+    if (!isDbLocal())
     {
         confess &log(ERROR, CMD_ARCHIVE_PUSH . ' operation must run on the db host');
     }
 
-    # Load the archive object
-    use pgBackRest::Archive;
+    # Batch flag indicates that the archive-push command was called without a WAL segment
+    my $bBatch = false;
 
     # If an archive section has been defined, use that instead of the backup section when command is CMD_ARCHIVE_PUSH
     my $bArchiveAsync = optionGet(OPTION_ARCHIVE_ASYNC);
@@ -576,6 +566,8 @@ sub pushProcess
     }
 
     # If an archive file is defined, then push it
+    my $oException = undef;
+
     if (defined($ARGV[1]))
     {
         # If the stop file exists then discard the archive log
@@ -593,62 +585,92 @@ sub pushProcess
 
         &log(INFO, 'push WAL segment ' . $ARGV[1] . ($bArchiveAsync ? ' asynchronously' : ''));
 
-        $self->push($ARGV[1], $bArchiveAsync);
-
-        # Fork if async archiving is enabled
-        if ($bArchiveAsync)
+        # Push WAL segment - when async, any error will be deferred until after the async process has been started
+        eval
         {
-            # Fork and disable the async archive flag if this is the parent process
-            if (!optionTest(OPTION_TEST_NO_FORK) || !optionGet(OPTION_TEST_NO_FORK))
-            {
-                $bArchiveAsync = fork() == 0 ? true : false;
-            }
-            # Else the no-fork flag has been specified for testing
-            else
-            {
-                logDebugMisc($strOperation, 'no fork on archive local for TESTING');
-            }
+            $self->push($ARGV[1], $bArchiveAsync);
+                return true;
         }
+        or do
+        {
+            $oException = $EVAL_ERROR;
+
+            if (!$bArchiveAsync || !optionGet(OPTION_TEST_FORK))
+            {
+                confess $oException;
+            }
+        };
+    }
+    # If archive-push is called without a WAL segment then still run in batch mode to clear out the async queue
+    else
+    {
+        $bBatch = true;
     }
 
-    if ($bArchiveAsync)
+    # If async or batch mode then acquire a lock.  Also fork if in async mode so that the parent process can return to Postres.
+    if ($bArchiveAsync || $bBatch)
     {
-        # Start the async archive push
-        logDebugMisc($strOperation, 'start async archive-push');
-
         # Create a lock file to make sure async archive-push does not run more than once
         if (!lockAcquire(commandGet(), false))
         {
             logDebugMisc($strOperation, 'async archive-push process is already running - exiting');
+            $bBatch = false;
         }
         else
         {
-            # Open the log file
-            logFileSet(optionGet(OPTION_LOG_PATH) . '/' . optionGet(OPTION_STANZA) . '-archive-async');
-
-            # Call the archive_xfer function and continue to loop as long as there are files to process
-            my $iLogTotal;
-
-            while (!defined($iLogTotal) || $iLogTotal > 0)
+            # Only fork if a WAL segment was specified, otherwise jut run
+            if (!$bBatch)
             {
-                $iLogTotal = $self->xfer(optionGet(OPTION_SPOOL_PATH) . "/archive/" .
-                                         optionGet(OPTION_STANZA) . "/out", $strStopFile);
-
-                if ($iLogTotal > 0)
+                # Fork and disable the async archive flag if this is the parent process
+                if (optionGet(OPTION_TEST_FORK))
                 {
-                    logDebugMisc($strOperation, "transferred ${iLogTotal} WAL segment" .
-                                 ($iLogTotal > 1 ? 's' : '') . ', calling Archive->xfer() again');
+                    $bBatch = fork() == 0 ? true : false;
                 }
+                # Else the no-fork flag has been specified for testing
                 else
                 {
-                    logDebugMisc($strOperation, 'transfer found 0 WAL segments - exiting');
+                    logDebugMisc($strOperation, 'no fork on archive local for TESTING');
+                    $bBatch = true;
                 }
             }
-
-            lockRelease();
         }
     }
 
+    # Continue with batch processing
+    if ($bBatch)
+    {
+        # Start the async archive push
+        logDebugMisc($strOperation, 'start async archive-push');
+
+        # Open the log file
+        logFileSet(optionGet(OPTION_LOG_PATH) . '/' . optionGet(OPTION_STANZA) . '-archive-async');
+
+        # Call the archive_xfer function and continue to loop as long as there are files to process
+        my $iLogTotal;
+
+        while (!defined($iLogTotal) || $iLogTotal > 0)
+        {
+            $iLogTotal = $self->xfer(optionGet(OPTION_SPOOL_PATH) . "/archive/" .
+                                     optionGet(OPTION_STANZA) . "/out", $strStopFile);
+
+            if ($iLogTotal > 0)
+            {
+                logDebugMisc($strOperation, "transferred ${iLogTotal} WAL segment" .
+                             ($iLogTotal > 1 ? 's' : '') . ', calling Archive->xfer() again');
+            }
+            else
+            {
+                logDebugMisc($strOperation, 'transfer found 0 WAL segments - exiting');
+            }
+        }
+
+        lockRelease();
+    }
+    elsif (defined($oException))
+    {
+        confess $oException;
+    }
+
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -673,7 +695,7 @@ sub push
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_PUSH, \@_,
+            __PACKAGE__ . '->push', \@_,
             {name => 'strSourceFile'},
             {name => 'bAsync'}
         );
@@ -683,18 +705,21 @@ sub push
     (
         optionGet(OPTION_STANZA),
         $bAsync ? optionGet(OPTION_SPOOL_PATH) : optionGet(OPTION_REPO_PATH),
-        $bAsync ? NONE : optionRemoteType(),
-        protocolGet($bAsync)
+        protocolGet($bAsync ? NONE : BACKUP)
     );
 
     lockStopTest();
 
     # If the source file path is not absolute then it is relative to the data path
-    if (index($strSourceFile, '/',) != 0)
+    if (index($strSourceFile, '/') != 0)
     {
         if (!optionTest(OPTION_DB_PATH))
         {
-            confess &log(ERROR, 'option db-path must be set when relative xlog paths are used');
+            confess &log(
+                ERROR, "option 'db-path' must be set when relative xlog paths are used\n" .
+                "HINT: Is \%f passed to archive-push instead of \%p?\n" .
+                'HINT: PostgreSQL generally passes an absolute path to archive-push but in some environments does not, in which' .
+                " case the 'db-path' option must be specified (this is rare).");
         }
 
         $strSourceFile = optionGet(OPTION_DB_PATH) . "/${strSourceFile}";
@@ -721,8 +746,9 @@ sub push
         if ($bArchiveFile)
         {
             my ($strDbVersion, $ullDbSysId) = $self->walInfo($strSourceFile);
-            ($strArchiveId, $strChecksum) = $self->pushCheck($oFile, substr(basename($strSourceFile), 0, 24), $bPartial,
-                                                             $strSourceFile, $strDbVersion, $ullDbSysId);
+            ($strArchiveId, $strChecksum) = $self->pushCheck(
+                $oFile, substr(basename($strSourceFile), 0, 24), $bPartial, $strSourceFile, $strDbVersion, $ullDbSysId,
+                optionGet(OPTION_REPO_SYNC));
         }
         else
         {
@@ -749,7 +775,8 @@ sub push
                      undef, undef, undef,                                       # Unused params
                      true,                                                      # Create path if it does not exist
                      undef, undef,                                              # User and group
-                     $bArchiveFile);                                            # Append checksum if archive file
+                     $bArchiveFile,                                             # Append checksum if archive file
+                     $bAsync ? true : optionGet(OPTION_REPO_SYNC));             # Sync if spool, else check repo sync option
     }
 
     # Return from function and log return values if any
@@ -775,51 +802,39 @@ sub pushCheck
         $bPartial,
         $strWalFile,
         $strDbVersion,
-        $ullDbSysId
+        $ullDbSysId,
+        $bPathSync,
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_PUSH_CHECK, \@_,
+            __PACKAGE__ . '->pushCheck', \@_,
             {name => 'oFile'},
             {name => 'strWalSegment'},
             {name => 'bPartial'},
             {name => 'strWalFile', required => false},
             {name => 'strDbVersion'},
-            {name => 'ullDbSysId'}
+            {name => 'ullDbSysId'},
+            {name => 'bPathSync'},
         );
 
     # Set operation and debug strings
-    my $strChecksum;
     my $strArchiveId;
+    my $strChecksum;
 
     if ($oFile->isRemote(PATH_BACKUP_ARCHIVE))
     {
-        # Build param hash
-        my %oParamHash;
-
-        $oParamHash{'wal-segment'} = $strWalSegment;
-        $oParamHash{'partial'} = $bPartial;
-        $oParamHash{'db-version'} = $strDbVersion;
-        $oParamHash{'db-sys-id'} = $ullDbSysId;
-
         # Execute the command
-        my $strResult = $oFile->{oProtocol}->cmdExecute(OP_ARCHIVE_PUSH_CHECK, \%oParamHash, true);
-
-        $strArchiveId = (split("\t", $strResult))[0];
-        $strChecksum = (split("\t", $strResult))[1];
-
-        if ($strChecksum eq 'Y')
-        {
-            undef($strChecksum);
-        }
+        ($strArchiveId, $strChecksum) = $oFile->{oProtocol}->cmdExecute(
+            OP_ARCHIVE_PUSH_CHECK, [$strWalSegment, $bPartial, undef, $strDbVersion, $ullDbSysId, $bPathSync], true);
     }
     else
     {
         # Create the archive path if it does not exist
         $oFile->pathCreate(PATH_BACKUP_ARCHIVE, undef, undef, true, true);
 
-        # If the info file exists check db version and system-id, else create it with the db version and system-id passed and the history
-        $strArchiveId = (new pgBackRest::ArchiveInfo($oFile->pathGet(PATH_BACKUP_ARCHIVE)))->check($strDbVersion, $ullDbSysId);
+        # If the info file exists check db version and system-id, else create it
+        $strArchiveId = (new pgBackRest::ArchiveInfo($oFile->pathGet(PATH_BACKUP_ARCHIVE)))->check(
+            $strDbVersion, $ullDbSysId, $bPathSync);
 
         # Check if the WAL segment already exists in the archive
         $strChecksum = $self->walFileName($oFile, $strArchiveId, $strWalSegment, $bPartial);
@@ -870,7 +885,7 @@ sub xfer
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_XFER, \@_,
+            __PACKAGE__ . '->xfer', \@_,
             {name => 'strArchivePath'},
             {name => 'strStopFile'}
         );
@@ -880,27 +895,25 @@ sub xfer
     (
         optionGet(OPTION_STANZA),
         optionGet(OPTION_REPO_PATH),
-        NONE,
-        protocolGet(true)
+        protocolGet(NONE)
     );
 
     # Load the archive manifest - all the files that need to be pushed
-    my %oManifestHash;
-    $oFile->manifest(PATH_DB_ABSOLUTE, $strArchivePath, \%oManifestHash);
+    my $hManifest = $oFile->manifest(PATH_DB_ABSOLUTE, $strArchivePath);
 
     # Get all the files to be transferred and calculate the total size
     my @stryFile;
     my $lFileSize = 0;
     my $lFileTotal = 0;
 
-    foreach my $strFile (sort(keys(%{$oManifestHash{name}})))
+    foreach my $strFile (sort(keys(%{$hManifest})))
     {
         if ($strFile =~ "^[0-F]{24}(\\.partial){0,1}(-[0-f]{40})(\\.$oFile->{strCompressExtension}){0,1}\$" ||
             $strFile =~ /^[0-F]{8}\.history$/ || $strFile =~ /^[0-F]{24}\.[0-F]{8}\.backup$/)
         {
             CORE::push(@stryFile, $strFile);
 
-            $lFileSize += $oManifestHash{name}{$strFile}{size};
+            $lFileSize += $hManifest->{$strFile}{size};
             $lFileTotal++;
         }
     }
@@ -913,20 +926,21 @@ sub xfer
     }
     else
     {
+        my $oException = undef;
+
         eval
         {
             # Start backup test point
             &log(TEST, TEST_ARCHIVE_PUSH_ASYNC_START);
 
             # If the archive repo is remote create a new file object to do the copies
-            if (!optionRemoteTypeTest(NONE))
+            if (!isRepoLocal())
             {
                 $oFile = new pgBackRest::File
                 (
                     optionGet(OPTION_STANZA),
                     optionGet(OPTION_REPO_PATH),
-                    optionRemoteType(),
-                    protocolGet()
+                    protocolGet(BACKUP)
                 );
             }
 
@@ -957,14 +971,19 @@ sub xfer
                 my $bDestinationCompress = $bArchiveFile && optionGet(OPTION_COMPRESS);
                 my $strDestinationFile = basename($strFile);
 
-                if (!$bSourceCompressed && $bDestinationCompress)
+                # Strip off existing checksum
+                my $strAppendedChecksum = undef;
+
+                if ($bArchiveFile)
+                {
+                    $strAppendedChecksum = substr($strDestinationFile, $bPartial ? 33 : 25, 40);
+                    $strDestinationFile = substr($strDestinationFile, 0, $bPartial ? 32 : 24);
+                }
+
+                if ($bDestinationCompress)
                 {
                     $strDestinationFile .= ".$oFile->{strCompressExtension}";
                 }
-                elsif ($bSourceCompressed && !$bDestinationCompress)
-                {
-                    $strDestinationFile = substr($strDestinationFile, 0, length($strDestinationFile) - 3);
-                }
 
                 logDebugMisc
                 (
@@ -982,26 +1001,41 @@ sub xfer
                 if ($bArchiveFile)
                 {
                     my ($strDbVersion, $ullDbSysId) = $self->walInfo($strArchiveFile);
-                    ($strArchiveId, $strChecksum) = $self->pushCheck($oFile, substr(basename($strArchiveFile), 0, 24), $bPartial,
-                                                                     $strArchiveFile, $strDbVersion, $ullDbSysId);
+                    ($strArchiveId, $strChecksum) = $self->pushCheck(
+                        $oFile, substr(basename($strArchiveFile), 0, 24), $bPartial, $strArchiveFile, $strDbVersion, $ullDbSysId,
+                        optionGet(OPTION_REPO_SYNC));
                 }
                 else
                 {
                     $strArchiveId = $self->getCheck($oFile);
                 }
 
-                # Only copy the WAL segment if checksum is not defined.  If checksum is defined it means that the WAL segment already
-                # exists in the repository with the same checksum (else there would have been an error on checksum mismatch).
+                # Only copy the WAL segment if checksum is not defined.  If checksum is defined it means that the WAL segment
+                # already exists in the repository with the same checksum (else there would have been an error on checksum
+                # mismatch).
                 if (!defined($strChecksum))
                 {
                     # Copy the archive file
-                    $oFile->copy(PATH_DB_ABSOLUTE, $strArchiveFile,         # Source path/file
-                                 PATH_BACKUP_ARCHIVE,                       # Destination path
-                                 "${strArchiveId}/${strDestinationFile}",   # Destination file
-                                 $bSourceCompressed,                        # Source is not compressed
-                                 $bDestinationCompress,                     # Destination compress is configurable
-                                 undef, undef, undef,                       # Unused params
-                                 true);                                     # Create path if it does not exist
+                    my ($bResult, $strCopyChecksum) = $oFile->copy(
+                        PATH_DB_ABSOLUTE, $strArchiveFile,          # Source path/file
+                        PATH_BACKUP_ARCHIVE,                        # Destination path
+                        "${strArchiveId}/${strDestinationFile}",    # Destination file
+                        $bSourceCompressed,                         # Source is not compressed
+                        $bDestinationCompress,                      # Destination compress is configurable
+                        undef, undef, undef,                        # Unused params
+                        true,                                       # Create path if it does not exist
+                        undef, undef,                               # Unused params
+                        true,                                       # Append checksum
+                        optionGet(OPTION_REPO_SYNC));               # Sync path if set
+
+                    # If appended checksum does not equal copy checksum
+                    if (defined($strAppendedChecksum) && $strAppendedChecksum ne $strCopyChecksum)
+                    {
+                        confess &log(
+                            ERROR,
+                            "archive ${strArchiveFile} appended checksum ${strAppendedChecksum} does not match" .
+                                " copy checksum ${strCopyChecksum}", ERROR_ARCHIVE_MISMATCH);
+                    }
                 }
 
                 #  Remove the source archive file
@@ -1011,11 +1045,15 @@ sub xfer
                                            CMD_ARCHIVE_PUSH . ' is being run with different permissions in different contexts.');
 
                 # Remove the copied segment from the total size
-                $lFileSize -= $oManifestHash{name}{$strFile}{size};
+                $lFileSize -= $hManifest->{$strFile}{size};
             }
-        };
 
-        my $oException = $@;
+            return true;
+        }
+        or do
+        {
+            $oException = $EVAL_ERROR;
+        };
 
         # Create a stop file if the archive store exceeds the max even after xfer
         if (optionTest(OPTION_ARCHIVE_MAX_MB))
@@ -1051,70 +1089,6 @@ sub xfer
     );
 }
 
-####################################################################################################################################
-# range
-#
-# Generates a range of archive log file names given the start and end log file name.  For pre-9.3 databases, use bSkipFF to exclude
-# the FF that prior versions did not generate.
-####################################################################################################################################
-sub range
-{
-    my $self = shift;
-
-    # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation,
-        $strArchiveStart,
-        $strArchiveStop,
-        $bSkipFF
-    ) =
-        logDebugParam
-        (
-            OP_ARCHIVE_RANGE, \@_,
-            {name => 'strArchiveStart'},
-            {name => 'strArchiveStop'},
-            {name => 'bSkipFF', default => false}
-        );
-
-    # Working variables
-    my @stryArchive;
-    my $iArchiveIdx = 0;
-
-    # Iterate through all archive logs between start and stop
-    my @stryArchiveSplit = split('/', $strArchiveStart);
-    my $iStartMajor = hex($stryArchiveSplit[0]);
-    my $iStartMinor = hex(substr(sprintf("%08s", $stryArchiveSplit[1]), 0, 2));
-
-    @stryArchiveSplit = split('/', $strArchiveStop);
-    my $iStopMajor = hex($stryArchiveSplit[0]);
-    my $iStopMinor = hex(substr(sprintf("%08s", $stryArchiveSplit[1]), 0, 2));
-
-    $stryArchive[$iArchiveIdx] = uc(sprintf("%08x%08x", $iStartMajor, $iStartMinor));
-    $iArchiveIdx += 1;
-
-    while (!($iStartMajor == $iStopMajor && $iStartMinor == $iStopMinor))
-    {
-        $iStartMinor += 1;
-
-        if ($bSkipFF && $iStartMinor == 255 || !$bSkipFF && $iStartMinor == 256)
-        {
-            $iStartMajor += 1;
-            $iStartMinor = 0;
-        }
-
-        $stryArchive[$iArchiveIdx] = uc(sprintf("%08x%08x", $iStartMajor, $iStartMinor));
-        $iArchiveIdx += 1;
-    }
-
-    # Return from function and log return values if any
-    return logDebugReturn
-    (
-        $strOperation,
-        {name => 'stryArchive', value => \@stryArchive}
-    );
-}
-
 ####################################################################################################################################
 # check
 #
@@ -1134,15 +1108,14 @@ sub check
     (
         optionGet(OPTION_STANZA),
         optionGet(OPTION_REPO_PATH),
-        optionRemoteType(),
-        protocolGet()
+        protocolGet(isRepoLocal() ? DB : BACKUP)
     );
 
     # Initialize the database object
-    my $oDb = new pgBackRest::Db();
+    my $oDb = dbMasterGet();
 
     # Validate the database configuration
-    $oDb->configValidate($oFile, optionGet(OPTION_DB_PATH));
+    $oDb->configValidate();
 
     # Force archiving
     my $strWalSegment = $oDb->xlogSwitch();
@@ -1159,7 +1132,6 @@ sub check
 
     my $strArchiveId = undef;
     my $strArchiveFile = undef;
-    my $oBackupInfo = undef;
 
     # Turn off console logging to control when to display the error
     logLevelSet(undef, OFF);
@@ -1174,23 +1146,18 @@ sub check
 
             # Clear any previous errors if we've found the archive.info
             $iResult = 0;
-        };
 
-        if ($@)
-        {
-            my $oMessage = $@;
-
-            # If this is a backrest error then capture the last code and message else confess
-            if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
-            {
-                $iResult = $oMessage->code();
-                $strResultMessage = $oMessage->message();
-            }
-            else
-            {
-                confess $oMessage;
-            }
+            return true;
         }
+        or do
+        {
+            # Confess unhandled errors
+            confess $EVAL_ERROR if (!isException($EVAL_ERROR));
+
+            # If this is a backrest error then capture the last code and message
+            $iResult = $EVAL_ERROR->code();
+            $strResultMessage = $EVAL_ERROR->message();
+        };
     } while (!defined($strArchiveId) && waitMore($oWait));
 
     # If able to get the archive id then check the archived WAL file with the time remaining
@@ -1199,67 +1166,49 @@ sub check
         eval
         {
             $strArchiveFile = $self->walFileName($oFile, $strArchiveId, $strWalSegment, false, $iArchiveTimeout);
-        };
-
-        # If this is a backrest error then capture the code and message else confess
-        if ($@)
-        {
-            my $oMessage = $@;
-
-            # If a backrest exception then return the code else confess
-            if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
-            {
-                $iResult = $oMessage->code();
-                $strResultMessage = $oMessage->message();
-            }
-            else
-            {
-                confess $oMessage;
-            }
+            return true;
         }
+        # If this is a backrest error then capture the code and message else confess
+        or do
+        {
+            # Confess unhandled errors
+            confess $EVAL_ERROR if (!isException($EVAL_ERROR));
+
+            # If this is a backrest error then capture the last code and message
+            $iResult = $EVAL_ERROR->code();
+            $strResultMessage = $EVAL_ERROR->message();
+        };
+    }
+
+    # If the archive info was successful, then check the backup info
+    if ($iResult == 0)
+    {
+        # Check the backup info
+        eval
+        {
+            $self->getBackupInfoCheck($oFile);
+            return true;
+        }
+        or do
+        {
+            # If there is an error but it is not that the file is missing then confess
+            if (!isException($EVAL_ERROR) || $EVAL_ERROR->code() != ERROR_PATH_MISSING)
+            {
+                confess $EVAL_ERROR;
+            }
+        };
     }
 
     # Reset the console logging
     logLevelSet(undef, optionGet(OPTION_LOG_LEVEL_CONSOLE));
 
-    # If the archive info was successful, then check the backup info
+    # If the archive info was successful and backup.info check did not error in an unexpected way, then indicate success
+    # Else, log the error.
     if ($iResult == 0)
     {
-        # If backup repo is local, then check the backup info
-        if (!optionRemoteTypeTest(BACKUP))
-        {
-            # Load or build backup.info
-            eval
-            {
-                $oBackupInfo = new pgBackRest::BackupInfo($oFile->pathGet(PATH_BACKUP_CLUSTER));
-            };
-            if ($@)
-            {
-                my $oMessage = $@;
-
-                # If this is a backrest error but it is not that the file is missing then confess
-                if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception')
-                    && ($oMessage->code() != ERROR_PATH_MISSING))
-                {
-                    confess $oMessage;
-                }
-            }
-            else
-            {
-                # Get the current database info
-                my ($strDbVersion, $iControlVersion, $iCatalogVersion, $ullDbSysId) =
-                    $oDb->info($oFile, optionGet(OPTION_DB_PATH));
-
-                # Check that the stanza backup info is compatible with the current version of the database
-                # If not, an error will be thrown
-                my $iDbHistoryId = $oBackupInfo->check($strDbVersion, $iControlVersion, $iCatalogVersion, $ullDbSysId);
-            }
-        }
-
-        # If the backup and archive checks were successful, then indicate success
         &log(INFO,
-            "WAL segment ${strWalSegment} successfully stored in the archive at '" .
-            $oFile->pathGet(PATH_BACKUP_ARCHIVE, "$strArchiveId/${strArchiveFile}") . "'");
+        "WAL segment ${strWalSegment} successfully stored in the archive at '" .
+        $oFile->pathGet(PATH_BACKUP_ARCHIVE, "$strArchiveId/${strArchiveFile}") . "'");
     }
     else
     {
