@@ -16,24 +16,15 @@ use Exporter qw(import);
 use File::Basename qw(dirname basename);
 use File::stat;
 
-use lib dirname($0);
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::BackupInfo;
 use pgBackRest::Config::Config;
 use pgBackRest::File;
+use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_ARCHIVE_INFO                                        => 'ArchiveInfo';
-
-use constant OP_ARCHIVE_INFO_ARCHIVE_ID                             => OP_ARCHIVE_INFO . "->archiveId";
-use constant OP_ARCHIVE_INFO_CHECK                                  => OP_ARCHIVE_INFO . "->check";
-use constant OP_ARCHIVE_INFO_NEW                                    => OP_ARCHIVE_INFO . "->new";
-
 ####################################################################################################################################
 # File/path constants
 ####################################################################################################################################
@@ -41,7 +32,7 @@ use constant ARCHIVE_INFO_FILE                                      => 'archive.
     our @EXPORT = qw(ARCHIVE_INFO_FILE);
 
 ####################################################################################################################################
-# Backup info Constants
+# Archive info constants
 ####################################################################################################################################
 use constant INFO_ARCHIVE_SECTION_DB                                => INFO_BACKUP_SECTION_DB;
     push @EXPORT, qw(INFO_ARCHIVE_SECTION_DB);
@@ -71,14 +62,14 @@ sub new
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_INFO_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'strArchiveClusterPath'},
             {name => 'bRequired', default => false}
         );
 
     # Build the archive info path/file name
     my $strArchiveInfoFile = "${strArchiveClusterPath}/" . ARCHIVE_INFO_FILE;
-    my $bExists = -e $strArchiveInfoFile ? true : false;
+    my $bExists = fileExists($strArchiveInfoFile);
 
     if (!$bExists && $bRequired)
     {
@@ -117,13 +108,15 @@ sub check
     (
         $strOperation,
         $strDbVersion,
-        $ullDbSysId
+        $ullDbSysId,
+        $bPathSync,
     ) =
         logDebugParam
         (
-            OP_ARCHIVE_INFO_CHECK, \@_,
+            __PACKAGE__ . '->check', \@_,
             {name => 'strDbVersion'},
-            {name => 'ullDbSysId'}
+            {name => 'ullDbSysId'},
+            {name => 'bPathSync', default => false},
         );
 
     my $bSave = false;
@@ -171,6 +164,12 @@ sub check
     if ($bSave)
     {
         $self->save();
+
+        # Sync path if requested
+        if ($bPathSync)
+        {
+            filePathSync($self->{strArchiveClusterPath});
+        }
     }
 
     # Return from function and log return values if any
@@ -192,14 +191,7 @@ sub archiveId
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_ARCHIVE_INFO_ARCHIVE_ID
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->archiveId');
 
     # Return from function and log return values if any
     return logDebugReturn
