@@ -13,7 +13,6 @@ use Exporter qw(import);
 use File::Basename qw(dirname basename);
 use File::stat;
 
-use lib dirname($0);
 use pgBackRest::BackupCommon;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
@@ -23,19 +22,6 @@ use pgBackRest::File;
 use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_BACKUP_INFO                                         => 'BackupInfo';
-
-use constant OP_INFO_BACKUP_ADD                                     => OP_BACKUP_INFO . "->add";
-use constant OP_INFO_BACKUP_CHECK                                   => OP_BACKUP_INFO . "->check";
-use constant OP_INFO_BACKUP_CURRENT                                 => OP_BACKUP_INFO . "->current";
-use constant OP_INFO_BACKUP_DELETE                                  => OP_BACKUP_INFO . "->delete";
-use constant OP_INFO_BACKUP_LAST                                    => OP_BACKUP_INFO . "->last";
-use constant OP_INFO_BACKUP_LIST                                    => OP_BACKUP_INFO . "->list";
-use constant OP_INFO_BACKUP_NEW                                     => OP_BACKUP_INFO . "->new";
-
 ####################################################################################################################################
 # File/path constants
 ####################################################################################################################################
@@ -62,22 +48,24 @@ use constant INFO_BACKUP_KEY_ARCHIVE_START                          => MANIFEST_
     push @EXPORT, qw(INFO_BACKUP_KEY_ARCHIVE_START);
 use constant INFO_BACKUP_KEY_ARCHIVE_STOP                           => MANIFEST_KEY_ARCHIVE_STOP;
     push @EXPORT, qw(INFO_BACKUP_KEY_ARCHIVE_STOP);
+use constant INFO_BACKUP_KEY_BACKUP_STANDBY                         => MANIFEST_KEY_BACKUP_STANDBY;
+    push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_STANDBY);
 use constant INFO_BACKUP_KEY_BACKUP_REPO_SIZE                       => 'backup-info-repo-size';
-     push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_REPO_SIZE);
+    push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_REPO_SIZE);
 use constant INFO_BACKUP_KEY_BACKUP_REPO_SIZE_DELTA                 => 'backup-info-repo-size-delta';
-     push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_REPO_SIZE_DELTA);
+    push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_REPO_SIZE_DELTA);
 use constant INFO_BACKUP_KEY_BACKUP_SIZE                            => 'backup-info-size';
-     push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_SIZE);
+    push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_SIZE);
 use constant INFO_BACKUP_KEY_BACKUP_SIZE_DELTA                      => 'backup-info-size-delta';
-     push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_SIZE_DELTA);
+    push @EXPORT, qw(INFO_BACKUP_KEY_BACKUP_SIZE_DELTA);
 use constant INFO_BACKUP_KEY_CATALOG                                => MANIFEST_KEY_CATALOG;
     push @EXPORT, qw(INFO_BACKUP_KEY_CATALOG);
 use constant INFO_BACKUP_KEY_CONTROL                                => MANIFEST_KEY_CONTROL;
     push @EXPORT, qw(INFO_BACKUP_KEY_CONTROL);
 use constant INFO_BACKUP_KEY_COMPRESS                               => MANIFEST_KEY_COMPRESS;
     push @EXPORT, qw(INFO_BACKUP_KEY_COMPRESS);
-use constant INFO_BACKUP_KEY_CHECKSUM                               => INI_KEY_CHECKSUM;
-    push @EXPORT, qw(INFO_BACKUP_KEY_CHECKSUM);
+use constant INFO_BACKUP_KEY_CHECKSUM_PAGE                          => MANIFEST_KEY_CHECKSUM_PAGE;
+    push @EXPORT, qw(INFO_BACKUP_KEY_CHECKSUM_PAGE);
 use constant INFO_BACKUP_KEY_DB_VERSION                             => MANIFEST_KEY_DB_VERSION;
     push @EXPORT, qw(INFO_BACKUP_KEY_DB_VERSION);
 use constant INFO_BACKUP_KEY_FORMAT                                 => INI_KEY_FORMAT;
@@ -121,7 +109,7 @@ sub new
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'strBackupClusterPath'},
             {name => 'bValidate', default => true}
         );
@@ -160,14 +148,7 @@ sub validate
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation,
-    ) =
-        logDebugParam
-        (
-            OP_INFO_BACKUP_NEW
-        );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->validate');
 
     # Check for backups that are not in FILE_BACKUP_INFO
     my $strPattern = backupRegExpGet(true, true, true);
@@ -177,7 +158,7 @@ sub validate
         my $strManifestFile = "$self->{strBackupClusterPath}/${strBackup}/" . FILE_MANIFEST;
 
         # ??? Check for and move history files that were not moved before and maybe don't consider it to be an error when they
-        # can't be moved.  This would also be true for the first move attempty in Backup->process();
+        # can't be moved.  This would also be true for the first move attempt in Backup->process();
 
         if (!$self->current($strBackup) && fileExists($strManifestFile))
         {
@@ -231,7 +212,7 @@ sub check
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_CHECK, \@_,
+            __PACKAGE__ . '->check', \@_,
             {name => 'strDbVersion', trace => true},
             {name => 'iControlVersion', trace => true},
             {name => 'iCatalogVersion', trace => true},
@@ -305,7 +286,7 @@ sub add
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_ADD, \@_,
+            __PACKAGE__ . '->add', \@_,
             {name => 'oBackupManifest', trace => true}
         );
 
@@ -367,6 +348,10 @@ sub add
         $oBackupManifest->get(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_ARCHIVE_START, undef, false));
     $self->set(INFO_BACKUP_SECTION_BACKUP_CURRENT, $strBackupLabel, INFO_BACKUP_KEY_ARCHIVE_STOP,
         $oBackupManifest->get(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_ARCHIVE_STOP, undef, false));
+    $self->boolSet(INFO_BACKUP_SECTION_BACKUP_CURRENT, $strBackupLabel, INFO_BACKUP_KEY_BACKUP_STANDBY,
+        $oBackupManifest->boolGet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BACKUP_STANDBY));
+    $self->boolSet(INFO_BACKUP_SECTION_BACKUP_CURRENT, $strBackupLabel, INFO_BACKUP_KEY_CHECKSUM_PAGE,
+        $oBackupManifest->boolGet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_CHECKSUM_PAGE));
     $self->boolSet(INFO_BACKUP_SECTION_BACKUP_CURRENT, $strBackupLabel, INFO_BACKUP_KEY_COMPRESS,
         $oBackupManifest->boolGet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS));
     $self->numericSet(INFO_BACKUP_SECTION_BACKUP_CURRENT, $strBackupLabel, INFO_BACKUP_KEY_FORMAT,
@@ -419,7 +404,7 @@ sub current
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_CURRENT, \@_,
+            __PACKAGE__ . '->current', \@_,
             {name => 'strBackup'}
         );
 
@@ -449,7 +434,7 @@ sub list
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_LIST, \@_,
+            __PACKAGE__ . '->list', \@_,
             {name => 'strFilter', required => false},
             {name => 'strOrder', default => 'forward'}
         );
@@ -499,7 +484,7 @@ sub last
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_LAST, \@_,
+            __PACKAGE__ . '->last', \@_,
             {name => 'strType'}
         );
 
@@ -531,7 +516,7 @@ sub delete
     ) =
         logDebugParam
         (
-            OP_INFO_BACKUP_DELETE, \@_,
+            __PACKAGE__ . '->delete', \@_,
             {name => 'strBackupLabel'}
         );
 
