@@ -7,6 +7,8 @@ use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess longmess);
 
+use Scalar::Util qw(blessed);
+
 use Exporter qw(import);
     our @EXPORT = qw();
 
@@ -98,8 +100,6 @@ use constant ERROR_TERM                                             => ERROR_MIN
     push @EXPORT, qw(ERROR_TERM);
 use constant ERROR_FILE_WRITE                                       => ERROR_MINIMUM + 39;
     push @EXPORT, qw(ERROR_FILE_WRITE);
-use constant ERROR_UNHANDLED_EXCEPTION                              => ERROR_MINIMUM + 40;
-    push @EXPORT, qw(ERROR_UNHANDLED_EXCEPTION);
 use constant ERROR_PROTOCOL_TIMEOUT                                 => ERROR_MINIMUM + 41;
     push @EXPORT, qw(ERROR_PROTOCOL_TIMEOUT);
 use constant ERROR_FEATURE_NOT_SUPPORTED                            => ERROR_MINIMUM + 42;
@@ -136,9 +136,25 @@ use constant ERROR_ARCHIVE_TIMEOUT                                  => ERROR_MIN
     push @EXPORT, qw(ERROR_ARCHIVE_TIMEOUT);
 use constant ERROR_FILE_MODE                                        => ERROR_MINIMUM + 58;
     push @EXPORT, qw(ERROR_FILE_MODE);
+use constant ERROR_OPTION_MULTIPLE_VALUE                            => ERROR_MINIMUM + 59;
+    push @EXPORT, qw(ERROR_OPTION_MULTIPLE_VALUE);
+use constant ERROR_PROTOCOL_OUTPUT_REQUIRED                         => ERROR_MINIMUM + 60;
+    push @EXPORT, qw(ERROR_PROTOCOL_OUTPUT_REQUIRED);
+use constant ERROR_LINK_OPEN                                        => ERROR_MINIMUM + 61;
+    push @EXPORT, qw(ERROR_LINK_OPEN);
+use constant ERROR_ARCHIVE_DISABLED                                 => ERROR_MINIMUM + 62;
+    push @EXPORT, qw(ERROR_ARCHIVE_DISABLED);
+use constant ERROR_ARCHIVE_DIR_INVALID                              => ERROR_MINIMUM + 63;
+    push @EXPORT, qw(ERROR_ARCHIVE_DIR_INVALID);
+use constant ERROR_BACKUP_DIR_INVALID                               => ERROR_MINIMUM + 64;
+    push @EXPORT, qw(ERROR_BACKUP_DIR_INVALID);
+use constant ERROR_OPTION_COMMAND                                   => ERROR_MINIMUM + 65;
+    push @EXPORT, qw(ERROR_OPTION_COMMAND);
 
-use constant ERROR_INVALID_VALUE                                    => ERROR_MAXIMUM - 1;
+use constant ERROR_INVALID_VALUE                                    => ERROR_MAXIMUM - 2;
     push @EXPORT, qw(ERROR_INVALID_VALUE);
+use constant ERROR_UNHANDLED                                        => ERROR_MAXIMUM - 1;
+    push @EXPORT, qw(ERROR_UNHANDLED);
 use constant ERROR_UNKNOWN                                          => ERROR_MAXIMUM;
     push @EXPORT, qw(ERROR_UNKNOWN);
 
@@ -148,27 +164,41 @@ use constant ERROR_UNKNOWN                                          => ERROR_MAX
 sub new
 {
     my $class = shift;       # Class name
+    my $strLevel = shift;    # Log level
     my $iCode = shift;       # Error code
     my $strMessage = shift;  # ErrorMessage
     my $strTrace = shift;    # Stack trace
+    my $rExtra = shift;      # Extra info used exclusively by the logging system
 
-    # if ($iCode < ERROR_MINIMUM || $iCode > ERROR_MAXIMUM)
-    # {
-    #     $iCode = ERROR_INVALID_VALUE;
-    # }
+    if ($iCode < ERROR_MINIMUM || $iCode > ERROR_MAXIMUM)
+    {
+        $iCode = ERROR_INVALID_VALUE;
+    }
 
     # Create the class hash
     my $self = {};
     bless $self, $class;
 
     # Initialize exception
+    $self->{strLevel} = $strLevel;
     $self->{iCode} = $iCode;
     $self->{strMessage} = $strMessage;
     $self->{strTrace} = $strTrace;
+    $self->{rExtra} = $rExtra;
 
     return $self;
 }
 
+####################################################################################################################################
+# level
+####################################################################################################################################
+sub level
+{
+    my $self = shift;
+
+    return $self->{strLevel};
+}
+
 ####################################################################################################################################
 # CODE
 ####################################################################################################################################
@@ -179,6 +209,16 @@ sub code
     return $self->{iCode};
 }
 
+####################################################################################################################################
+# extra
+####################################################################################################################################
+sub extra
+{
+    my $self = shift;
+
+    return $self->{rExtra};
+}
+
 ####################################################################################################################################
 # MESSAGE
 ####################################################################################################################################
@@ -199,4 +239,18 @@ sub trace
     return $self->{strTrace};
 }
 
+####################################################################################################################################
+# isException
+#
+# Is this a structured exception?
+####################################################################################################################################
+sub isException
+{
+    my $oException = shift;
+
+    return defined($oException) && blessed($oException) && $oException->isa('pgBackRest::Common::Exception') ? 1 : 0;
+}
+
+push @EXPORT, qw(isException);
+
 1;
