@@ -6,24 +6,17 @@ package pgBackRest::Common::Exit;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use Exporter qw(import);
     our @EXPORT = qw();
 use File::Basename qw(dirname);
-use Scalar::Util qw(blessed);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Lock;
 use pgBackRest::Common::Log;
 use pgBackRest::Config::Config;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_EXIT                                                => 'Exit';
-
-use constant OP_EXIT_SAFE                                           => OP_EXIT . '::exitSafe';
+use pgBackRest::Protocol::Protocol;
 
 ####################################################################################################################################
 # Signal constants
@@ -35,47 +28,14 @@ use constant SIGNAL_TERM                                            => 'TERM';
 ####################################################################################################################################
 # Hook important signals into exitSafe function
 ####################################################################################################################################
-$SIG{&SIGNAL_HUP} = sub {exitSafe(-1, SIGNAL_HUP)};
-$SIG{&SIGNAL_INT} = sub {exitSafe(-1, SIGNAL_INT)};
-$SIG{&SIGNAL_TERM} = sub {exitSafe(-1, SIGNAL_TERM)};
-
-####################################################################################################################################
-# Module variables
-####################################################################################################################################
-my $iThreadMax = 1;                                                 # Total threads that were started for processing
-my $bRemote = false;                                                # Is the process a remote?
-
-####################################################################################################################################
-# exitInit
-#
-# Initialize exit so it knows if threads need to be terminated.
-####################################################################################################################################
-sub exitInit
-{
-    my $iThreadMaxParam = shift;
-    my $bRemoteParam = shift;
-
-    if (defined($iThreadMaxParam) && $iThreadMaxParam > 1)
-    {
-        # Load module dynamically
-        require pgBackRest::Protocol::ThreadGroup;
-        pgBackRest::Protocol::ThreadGroup->import();
-
-        $iThreadMax = $iThreadMaxParam;
-    }
-
-    if (defined($bRemoteParam))
-    {
-        $bRemote = $bRemoteParam;
-    }
-}
-
-push @EXPORT, qw(exitInit);
+$SIG{&SIGNAL_HUP} = sub {exitSafe(ERROR_TERM, undef, SIGNAL_HUP)};
+$SIG{&SIGNAL_INT} = sub {exitSafe(ERROR_TERM, undef, SIGNAL_INT)};
+$SIG{&SIGNAL_TERM} = sub {exitSafe(ERROR_TERM, undef, SIGNAL_TERM)};
 
 ####################################################################################################################################
 # exitSafe
 #
-# Terminate all threads and SSH connections when the script is terminated.
+# Terminate all remotes and release locks.
 ####################################################################################################################################
 sub exitSafe
 {
@@ -84,62 +44,66 @@ sub exitSafe
     (
         $strOperation,
         $iExitCode,
-        $strSignal
+        $oException,
+        $strSignal,
     ) =
         logDebugParam
         (
-            OP_EXIT_SAFE, \@_,
-            {name => 'iExitCode'},
-            {name => 'strSignal', required => false}
+            __PACKAGE__ . '::exitSafe', \@_,
+            {name => 'iExitCode', required => false},
+            {name => 'oException', required => false},
+            {name => 'strSignal', required => false},
         );
 
-    commandStop();
-
-    # Stop threads if threading is enabled
-    my $iThreadsStopped = 0;
-
-    if ($iThreadMax > 1)
-    {
-        &log(DEBUG, "stop ${iThreadMax} threads");
-
-        # Don't fail if the threads cannot be stopped
-        eval
-        {
-            $iThreadsStopped = threadGroupDestroy();
-        };
-
-        if ($@ && defined($iExitCode))
-        {
-            &log(WARN, "unable to stop threads: $@");
-        }
-    }
+    # Reset logging in case it was disabled when the exception/signal occurred
+    configLogging();
 
     # Close the remote
-    protocolDestroy();
+    protocolDestroy(undef, undef, defined($iExitCode) && ($iExitCode == 0 || $iExitCode == 1));
 
     # Don't fail if the lock can't be released
     eval
     {
         lockRelease(false);
-    };
+    }
+    or do {};
 
-    # Exit with code when defined
-    if ($iExitCode != -1)
+    # If exit code is not defined then try to get it from the exception
+    if (!defined($iExitCode))
     {
-        exit $iExitCode;
+        # If a backrest exception
+        if (isException($oException))
+        {
+            $iExitCode = $oException->code();
+            logException($oException);
+        }
+        else
+        {
+            $iExitCode = ERROR_UNHANDLED;
+
+            &log(
+                ERROR,
+                'process terminated due to an unhandled exception' .
+                    (defined($oException) ? ":\n${oException}" : ': [exception not defined]'),
+                $iExitCode);
+        }
+    }
+    elsif ($iExitCode == ERROR_TERM)
+    {
+        &log(ERROR, "terminated on signal [SIG${strSignal}]", ERROR_TERM);
     }
 
-    # Log error based on where the signal came from
-    &log(ERROR, 'process terminated ' .
-                (defined($strSignal) ? "on a ${strSignal} signal" :  'due to an unhandled exception') .
-                ($iThreadsStopped > 0 ? ", ${iThreadsStopped} threads stopped" : ''),
-                defined($strSignal) ? ERROR_TERM : ERROR_UNHANDLED_EXCEPTION);
+    # Log command end
+    commandEnd($iExitCode, $strSignal);
 
-    # If terminated by a signal exit with 0 code
-    exit ERROR_TERM if defined($strSignal);
+    # Log return values if any
+    logDebugReturn
+    (
+        $strOperation,
+        {name => 'iExitCode', value => $iExitCode}
+    );
 
-    # Return from function and log return values if any
-    return logDebugReturn($strOperation);
+    exit $iExitCode;
 }
 
 push @EXPORT, qw(exitSafe);
