@@ -15,21 +15,12 @@ use IO::Handle;
 use JSON::PP;
 use Storable qw(dclone);
 
-use lib dirname($0);
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::FileCommon;
 use pgBackRest::Version;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_INI                                                 => 'Ini';
-
-use constant OP_INI_INI_SAVE                                        => OP_INI . "::iniSave";
-use constant OP_INI_SET                                             => OP_INI . "->set";
-
 ####################################################################################################################################
 # Boolean constants
 ####################################################################################################################################
@@ -217,6 +208,9 @@ sub save
 
     $self->hash();
     iniSave($self->{strFileName}, $self->{oContent}, false, true);
+
+    # Indicate the file now exists
+    $self->{bExists} = true;
 }
 
 ####################################################################################################################################
@@ -239,7 +233,7 @@ sub iniSave
     ) =
         logDebugParam
         (
-            OP_INI_INI_SAVE, \@_,
+            __PACKAGE__ . '::iniSave', \@_,
             {name => 'strFileName', trace => true},
             {name => 'oContent', trace => true},
             {name => 'bRelaxed', default => false, trace => true},
@@ -340,10 +334,7 @@ sub iniSave
     filePathSync(dirname($strFileName));
 
     # Return from function and log return values if any
-    return logDebugReturn
-    (
-        $strOperation
-    );
+    return logDebugReturn($strOperation);
 }
 
 ####################################################################################################################################
@@ -572,10 +563,14 @@ sub remove
     {
         delete(${$oContent}{$strSection}{$strKey}{$strSubKey});
     }
-    else
+    elsif (defined($strKey))
     {
         delete(${$oContent}{$strSection}{$strKey});
     }
+    else
+    {
+        delete(${$oContent}{$strSection});
+    }
 }
 
 ####################################################################################################################################
@@ -587,20 +582,35 @@ sub keys
 {
     my $self = shift;
     my $strSection = shift;
-    my $strKey = shift;
+    my $strSortOrder = shift;
 
-    if (defined($strSection))
+    if (!defined($strSection))
     {
-        if ($self->test($strSection, $strKey))
-        {
-            return (sort(keys(%{$self->get($strSection, $strKey)})));
-        }
-
-        my @stryEmptyArray;
-        return @stryEmptyArray;
+        confess &log(ASSERT, 'strSection must be set');
     }
 
-    return (sort(keys(%{$self->{oContent}})));
+    if ($self->test($strSection))
+    {
+        if (!defined($strSortOrder) || $strSortOrder eq 'forward')
+        {
+            return (sort(keys(%{$self->get($strSection)})));
+        }
+        elsif ($strSortOrder eq 'reverse')
+        {
+            return (sort {$b cmp $a} (keys(%{$self->get($strSection)})));
+        }
+        elsif ($strSortOrder eq 'none')
+        {
+            return (keys(%{$self->get($strSection)}));
+        }
+        else
+        {
+            confess &log(ASSERT, "invalid strSortOrder '${strSortOrder}'");
+        }
+    }
+
+    my @stryEmptyArray;
+    return @stryEmptyArray;
 }
 
 ####################################################################################################################################
