@@ -12,22 +12,12 @@ use Exporter qw(import);
 use Fcntl qw(:DEFAULT :flock);
 use File::Basename qw(dirname);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
 use pgBackRest::FileCommon;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_LOCK                                                => 'Common:::Lock';
-
-use constant OP_LOCK_ACQUIRE                                        => OP_LOCK . "::lockAquire";
-use constant OP_LOCK_RELEASE                                        => OP_LOCK . "::lockRelease";
-use constant OP_LOCK_STOP_TEST                                      => OP_LOCK . "::lockStopTest";
-
 ####################################################################################################################################
 # Global lock type and handle
 ####################################################################################################################################
@@ -59,7 +49,7 @@ sub lockFileName
 ####################################################################################################################################
 sub lockPathCreate
 {
-    filePathCreate(optionGet(OPTION_LOCK_PATH), '770');
+    filePathCreate(optionGet(OPTION_LOCK_PATH), '770', true);
 }
 
 ####################################################################################################################################
@@ -80,7 +70,7 @@ sub lockAcquire
     ) =
         logDebugParam
         (
-            OP_LOCK_ACQUIRE, \@_,
+            __PACKAGE__ . '::lockAcquire', \@_,
             {name => 'strLockType'},
             {name => 'bFailOnNoLock', default => true},
             {name => 'bRemote', default => false},
@@ -159,7 +149,7 @@ sub lockRelease
     ) =
         logDebugParam
         (
-            OP_LOCK_RELEASE, \@_,
+            __PACKAGE__ . '::lockRelease', \@_,
             {name => 'bFailOnNoLock', default => true}
         );
 
@@ -176,12 +166,6 @@ sub lockRelease
         }
         else
         {
-            # # Fail if the lock being released is not the one held
-            # if ($strLockType ne $strCurrentLockType)
-            # {
-            #     confess &log(ASSERT, "cannot remove lock ${strLockType} since ${strCurrentLockType} is currently held");
-            # }
-
             # Remove the file
             unlink($strCurrentLockFile);
             close($hCurrentLockHandle);
@@ -255,9 +239,9 @@ sub lockStop
 
             # Skip if this is a stop file
             next if ($strFile =~ /\.stop$/);
-            #
-            # # Skip if this is a thread lock file (we only send TERMs to the main process)
-            # next if ($strFile =~ /\-[0-9]+\.lock$/);
+
+            # Skip if this is a process lock file (only send TERMs to the main process)
+            next if ($strFile =~ /\-[0-9]+\.lock$/);
 
             # Open the lock file for read
             if (!sysopen($hLockHandle, $strLockFile, O_RDONLY))
@@ -310,7 +294,7 @@ push @EXPORT, qw(lockStop);
 sub lockStopTest
 {
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam(OP_LOCK_STOP_TEST);
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '::lockStopTest');
 
     # Check the stanza first if it is specified
     if (optionTest(OPTION_STANZA))
