@@ -3,7 +3,6 @@
 ####################################################################################################################################
 package pgBackRest::Common::Log;
 
-use threads;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess longmess);
@@ -15,7 +14,6 @@ use File::Basename qw(dirname);
 use Scalar::Util qw(blessed reftype);
 use Time::HiRes qw(gettimeofday usleep);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::String;
 
@@ -40,12 +38,12 @@ use constant INFO                                                   => 'INFO';
     push @EXPORT, qw(INFO);
 use constant WARN                                                   => 'WARN';
     push @EXPORT, qw(WARN);
+use constant PROTOCOL                                               => 'PROTOCOL';
+    push @EXPORT, qw(PROTOCOL);
 use constant ERROR                                                  => 'ERROR';
     push @EXPORT, qw(ERROR);
 use constant ASSERT                                                 => 'ASSERT';
     push @EXPORT, qw(ASSERT);
-use constant REMOTE                                                 => 'REMOTE';
-    push @EXPORT, qw(REMOTE);
 use constant OFF                                                    => 'OFF';
     push @EXPORT, qw(OFF);
 
@@ -59,17 +57,20 @@ $oLogLevelRank{DEBUG}{rank} = 7;
 $oLogLevelRank{DETAIL}{rank} = 6;
 $oLogLevelRank{INFO}{rank} = 5;
 $oLogLevelRank{WARN}{rank} = 4;
-$oLogLevelRank{ERROR}{rank} = 3;
-$oLogLevelRank{ASSERT}{rank} = 2;
-$oLogLevelRank{REMOTE}{rank} = 1;
+$oLogLevelRank{PROTOCOL}{rank} = 3;
+$oLogLevelRank{ERROR}{rank} = 2;
+$oLogLevelRank{ASSERT}{rank} = 1;
 $oLogLevelRank{OFF}{rank} = 0;
 
 ####################################################################################################################################
 # Module globals
 ####################################################################################################################################
-my $hLogFile;
-my $strLogLevelFile = ERROR;
-my $strLogLevelConsole = REMOTE;
+my $hLogFile = undef;
+my $strLogFileCache = undef;
+
+my $strLogLevelFile = OFF;
+my $strLogLevelConsole = OFF;
+my $strLogLevelStdErr = WARN;
 
 # Test globals
 my $bTest = false;
@@ -92,10 +93,10 @@ use constant TEST_BACKUP_NORESUME                                   => 'BACKUP-N
     push @EXPORT, qw(TEST_BACKUP_NORESUME);
 use constant TEST_BACKUP_START                                      => 'BACKUP-START';
     push @EXPORT, qw(TEST_BACKUP_START);
+use constant TEST_BACKUP_STOP                                       => 'BACKUP-STOP';
+    push @EXPORT, qw(TEST_BACKUP_STOP);
 use constant TEST_KEEP_ALIVE                                        => 'KEEP_ALIVE';
     push @EXPORT, qw(TEST_KEEP_ALIVE);
-use constant TEST_PROCESS_EXIT                                      => 'TEST_PROCESS_EXIT';
-    push @EXPORT, qw(TEST_PROCESS_EXIT);
 use constant TEST_ARCHIVE_PUSH_ASYNC_START                          => 'ARCHIVE-PUSH-ASYNC-START';
     push @EXPORT, qw(TEST_ARCHIVE_PUSH_ASYNC_START);
 
@@ -127,6 +128,13 @@ sub logFileSet
         }
 
         syswrite($hLogFile, "-------------------PROCESS START-------------------\n");
+
+        # Write out anything that was cached before the file was opened
+        if (defined($strLogFileCache))
+        {
+            syswrite($hLogFile, $strLogFileCache);
+            undef($strLogFileCache);
+        }
     }
 }
 
@@ -139,6 +147,7 @@ sub logLevelSet
 {
     my $strLevelFileParam = shift;
     my $strLevelConsoleParam = shift;
+    my $strLevelStdErrParam = shift;
 
     # Load FileCommon module
     require pgBackRest::FileCommon;
@@ -163,6 +172,16 @@ sub logLevelSet
 
         $strLogLevelConsole = uc($strLevelConsoleParam);
     }
+
+    if (defined($strLevelStdErrParam))
+    {
+        if (!defined($oLogLevelRank{uc($strLevelStdErrParam)}{rank}))
+        {
+            confess &log(ERROR, "stdout log level ${strLevelStdErrParam} does not exist");
+        }
+
+        $strLogLevelStdErr = uc($strLevelStdErrParam);
+    }
 }
 
 push @EXPORT, qw(logLevelSet);
@@ -240,6 +259,9 @@ sub logDebugProcess
     # Process each parameter hash
     my $oParam = shift;
 
+    # Strip the package name off strFunction if it's pgBackRest
+    $strFunction =~ s/^pgBackRest[^\:]*\:\://;
+
     while (defined($oParam))
     {
         my $strParamName = $$oParam{name};
@@ -331,6 +353,59 @@ sub logDebugProcess
     return @oyResult;
 }
 
+####################################################################################################################################
+# logDebugBuild
+####################################################################################################################################
+sub logDebugBuild
+{
+    my $strValue = shift;
+
+    my $rResult;
+
+    # Value is undefined
+    if (!defined($strValue))
+    {
+        $rResult = \'[undef]';
+    }
+    # Value is not a ref, but return it as a ref for efficiency
+    elsif (!ref($strValue))
+    {
+        $rResult = \$strValue;
+    }
+    # Value is a hash
+    elsif (ref($strValue) eq 'HASH')
+    {
+        my $strValueHash;
+
+        for my $strSubValue (sort(keys(%{$strValue})))
+        {
+            $strValueHash .=
+                (defined($strValueHash) ? ', ' : '{') . "${strSubValue} => " . ${logDebugBuild($strValue->{$strSubValue})};
+        }
+
+        $rResult = \(defined($strValueHash) ?  $strValueHash . '}' : '{}');
+    }
+    # Value is an array
+    elsif (ref($strValue) eq 'ARRAY')
+    {
+        my $strValueArray;
+
+        for my $strSubValue (@{$strValue})
+        {
+            $strValueArray .= (defined($strValueArray) ? ', ' : '(') . ${logDebugBuild($strSubValue)};
+        }
+
+        $rResult = \(defined($strValueArray) ?  $strValueArray . ')' : '()');
+    }
+    # Else some other type ??? For the moment this is forced to object to not make big log changes
+    else
+    {
+        $rResult = \('[object]');
+    }
+
+    return $rResult;
+}
+
 ####################################################################################################################################
 # logDebugOut
 ####################################################################################################################################
@@ -347,7 +422,8 @@ sub logDebugOut
     $strLevel = defined($strLevel) ? $strLevel : DEBUG;
 
     if ($oLogLevelRank{$strLevel}{rank} <= $oLogLevelRank{$strLogLevelConsole}{rank} ||
-        $oLogLevelRank{$strLevel}{rank} <= $oLogLevelRank{$strLogLevelFile}{rank})
+        $oLogLevelRank{$strLevel}{rank} <= $oLogLevelRank{$strLogLevelFile}{rank} ||
+        $oLogLevelRank{$strLevel}{rank} <= $oLogLevelRank{$strLogLevelStdErr}{rank})
     {
         if (defined($oParamHash))
         {
@@ -360,41 +436,9 @@ sub logDebugOut
                     $strParamSet .= ', ';
                 }
 
-                my $strValueRef;
-                my $bDefault = false;
-
-                if (ref($$oParamHash{$strParam}) eq 'HASH')
-                {
-                    if (blessed($$oParamHash{$strParam}{value}))
-                    {
-                        $strValueRef = \'[object]';
-                    }
-                    else
-                    {
-                        if (ref($$oParamHash{$strParam}{value}) eq 'ARRAY')
-                        {
-                            $strValueRef = \('(' . join(', ', @{$$oParamHash{$strParam}{value}}) . ')');
-                        }
-                        else
-                        {
-                            $strValueRef = ref($$oParamHash{$strParam}{value}) ? $$oParamHash{$strParam}{value} :
-                                                                                 \$$oParamHash{$strParam}{value};
-
-                            $bDefault = defined($$strValueRef) &&
-                                        defined($$oParamHash{$strParam}{default}) ? $$oParamHash{$strParam}{default} : false;
-                        }
-                    }
-                }
-                # If this is an ARRAY ref then create a comma-separated list
-                elsif (ref($$oParamHash{$strParam}) eq 'ARRAY')
-                {
-                    $strValueRef = \(join(', ', @{$$oParamHash{$strParam}}));
-                }
-                # Else get a reference if a reference was not passed
-                else
-                {
-                    $strValueRef = ref($$oParamHash{$strParam}) ? $$oParamHash{$strParam} : \$$oParamHash{$strParam};
-                }
+                my $strValueRef = defined($oParamHash->{$strParam}{value}) ? logDebugBuild($oParamHash->{$strParam}{value}) : undef;
+                my $bDefault =
+                    defined($$strValueRef) && defined($$oParamHash{$strParam}{default}) ? $$oParamHash{$strParam}{default} : false;
 
                 $strParamSet .= "${strParam} = " .
                                 ($bDefault ? '<' : '') .
@@ -420,6 +464,18 @@ sub logDebugOut
     }
 }
 
+####################################################################################################################################
+# logException
+####################################################################################################################################
+sub logException
+{
+    my $oException = shift;
+
+    return &log($oException->level(), $oException->message(), $oException->code(), undef, undef, undef, $oException->extra());
+}
+
+push @EXPORT, qw(logException);
+
 ####################################################################################################################################
 # LOG - log messages
 ####################################################################################################################################
@@ -430,10 +486,22 @@ sub log
     my $iCode = shift;
     my $bSuppressLog = shift;
     my $iIndent = shift;
+    my $iProcessId = shift;
+    my $rExtra = shift;
 
     # Set defaults
     $bSuppressLog = defined($bSuppressLog) ? $bSuppressLog : false;
 
+    # Initialize rExtra
+    if (!defined($rExtra))
+    {
+        $rExtra =
+        {
+            bLogFile => false,
+            bLogConsole => false,
+        };
+    }
+
     # Set operational variables
     my $strMessageFormat = $strMessage;
     my $iLogLevelRank = $oLogLevelRank{"${strLevel}"}{rank};
@@ -461,6 +529,16 @@ sub log
         $strMessageFormat = '(undefined)';
     }
 
+    # Set the error code
+    if ($strLevel eq ASSERT)
+    {
+        $iCode = ERROR_ASSERT;
+    }
+    elsif ($strLevel eq ERROR && !defined($iCode))
+    {
+        $iCode = ERROR_UNKNOWN;
+    }
+
     $strMessageFormat = (defined($iCode) ? "[${iCode}]: " : '') . $strMessageFormat;
 
     # Indent subsequent lines of the message if it has more than one line - makes the log more readable
@@ -484,26 +562,32 @@ sub log
         $strMessageFormat =~ s/\n/\n    /g;
         $strMessageFormat = '    ' . $strMessageFormat;
     }
-    elsif ($strLevel eq ERROR)
+    elsif ($strLevel eq ERROR || $strLevel eq ASSERT)
     {
-        if (!defined($iCode))
-        {
-            $iCode = ERROR_UNKNOWN;
-        }
-
         $strMessageFormat =~ s/\n/\n       /g;
     }
 
     # Format the message text
     my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
 
-    $strMessageFormat = timestampFormat() . sprintf('.%03d T%02d', (gettimeofday() - int(gettimeofday())) * 1000,
-                        threads->tid()) .
-                        (' ' x (7 - length($strLevel))) . "${strLevel}: ${strMessageFormat}\n";
+    $strMessageFormat =
+        timestampFormat() . sprintf('.%03d P%02d', (gettimeofday() - int(gettimeofday())) * 1000,
+        (defined($iProcessId) ? $iProcessId : 0)) .
+        (' ' x (7 - length($strLevel))) . "${strLevel}: ${strMessageFormat}\n";
 
-    # Output to console depending on log level and test flag
-    if ($iLogLevelRank <= $oLogLevelRank{$strLogLevelConsole}{rank} ||
-        $bTest && $strLevel eq TEST)
+    # Output to stderr depending on log level
+    if (!$rExtra->{bLogConsole} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelStdErr}{rank})
+    {
+        if ($strLogLevelStdErr ne PROTOCOL)
+        {
+            syswrite(*STDERR, $strLevel . (defined($iCode) ? " [${iCode}]" : '') . ': ');
+        }
+
+        syswrite(*STDERR, "${strMessage}\n");
+        $rExtra->{bLogConsole} = true;
+    }
+    # Else output to stdout depending on log level and test flag
+    elsif (!$rExtra->{bLogConsole} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelConsole}{rank} || $bTest && $strLevel eq TEST)
     {
         if (!$bSuppressLog)
         {
@@ -515,20 +599,25 @@ sub log
         {
             usleep($fTestDelay * 1000000);
         }
-    }
-    elsif ($strLogLevelConsole eq REMOTE && ($strLevel eq ASSERT || $strLevel eq ERROR))
-    {
-        syswrite(*STDERR, $strLevel . (defined($iCode) ? " [${iCode}]" : '') . ": $strMessage\n");
+
+        $rExtra->{bLogConsole} = true;
     }
 
     # Output to file depending on log level and test flag
-    if ($iLogLevelRank <= $oLogLevelRank{$strLogLevelFile}{rank})
+    if (!$rExtra->{bLogLogFile} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelFile}{rank})
     {
-        if (defined($hLogFile))
+        if (defined($hLogFile) || (defined($strLogLevelFile) && $strLogLevelFile ne OFF))
         {
             if (!$bSuppressLog)
             {
-                syswrite($hLogFile, $strMessageFormat);
+                if (defined($hLogFile))
+                {
+                    syswrite($hLogFile, $strMessageFormat);
+                }
+                else
+                {
+                    $strLogFileCache .= $strMessageFormat;
+                }
 
                 if ($strLevel eq ASSERT ||
                     ($strLevel eq ERROR && ($strLogLevelFile eq DEBUG || $strLogLevelFile eq TRACE)))
@@ -536,16 +625,25 @@ sub log
                     my $strStackTrace = longmess() . "\n";
                     $strStackTrace =~ s/\n/\n                                   /g;
 
-                    syswrite($hLogFile, $strStackTrace);
+                    if (defined($hLogFile))
+                    {
+                        syswrite($hLogFile, $strStackTrace);
+                    }
+                    else
+                    {
+                        $strLogFileCache .= $strStackTrace;
+                    }
                 }
             }
         }
+
+        $rExtra->{bLogFile} = true;
     }
 
     # Throw a typed exception if code is defined
     if (defined($iCode))
     {
-        return new pgBackRest::Common::Exception($iCode, $strMessage, longmess());
+        return new pgBackRest::Common::Exception($strLevel, $iCode, $strMessage, longmess(), $rExtra);
     }
 
     # Return the message test so it can be used in a confess
