@@ -3,7 +3,6 @@
 ####################################################################################################################################
 package pgBackRest::Expire;
 
-use threads;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
@@ -13,24 +12,17 @@ use File::Basename qw(dirname);
 use File::Path qw(remove_tree);
 use Scalar::Util qw(looks_like_number);
 
-use lib dirname($0);
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
+use pgBackRest::ArchiveCommon;
 use pgBackRest::BackupCommon;
 use pgBackRest::BackupInfo;
 use pgBackRest::Config::Config;
 use pgBackRest::File;
 use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_EXPIRE                                              => 'Expire';
-
-use constant OP_EXPIRE_DESTROY                                      => OP_EXPIRE . '->DESTROY';
-use constant OP_EXPIRE_NEW                                          => OP_EXPIRE . '->new';
-use constant OP_EXPIRE_PROCESS                                      => OP_EXPIRE . '->process';
+use pgBackRest::Protocol::Common;
+use pgBackRest::Protocol::Protocol;
 
 ####################################################################################################################################
 # new
@@ -44,15 +36,14 @@ sub new
     bless $self, $class;
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam(OP_EXPIRE_NEW);
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->new');
 
     # Initialize file object
     $self->{oFile} = new pgBackRest::File
     (
         optionGet(OPTION_STANZA),
         optionGet(OPTION_REPO_PATH),
-        optionRemoteType(),
-        protocolGet(true)
+        protocolGet(NONE)
     );
 
     # Initialize total archive expired
@@ -74,22 +65,12 @@ sub DESTROY
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_EXPIRE_DESTROY
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->DESTROY');
 
     undef($self->{oFile});
 
     # Return from function and log return values if any
-    return logDebugReturn
-    (
-        $strOperation
-    );
+    return logDebugReturn($strOperation);
 }
 
 ####################################################################################################################################
@@ -139,14 +120,7 @@ sub process
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-        (
-            OP_EXPIRE_PROCESS
-        );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->process');
 
     my @stryPath;
 
@@ -204,12 +178,18 @@ sub process
             confess &log(ERROR, 'retention-diff must be a number >= 1');
         }
 
-        @stryPath = $oBackupInfo->list(backupRegExpGet(false, true));
+        # Get a list of full and differential backups. Full are considered differential for the purpose of retention.
+        # Example: F1, D1, D2, F2 and retention-diff=2, then F1,D2,F2 will be retained, not D2 and D1 as might be expected.
+        @stryPath = $oBackupInfo->list(backupRegExpGet(true, true));
 
         if (@stryPath > $iDifferentialRetention)
         {
             for (my $iDiffIdx = 0; $iDiffIdx < @stryPath - $iDifferentialRetention; $iDiffIdx++)
             {
+                # Skip if this is a full backup.  Full backups only count as differential when deciding which differential backups
+                # to expire.
+                next if ($stryPath[$iDiffIdx] =~ backupRegExpGet(true));
+
                 # Get a list of all differential and incremental backups
                 my @stryRemoveList;
 
@@ -250,28 +230,15 @@ sub process
         }
     }
 
-    # Default archive retention if not explicily set
-    if (defined($iFullRetention))
+    # If archive retention is still undefined, then ignore archiving
+    if  (!defined($iArchiveRetention))
     {
-        if (!defined($iArchiveRetention))
-        {
-            $iArchiveRetention = $iFullRetention;
-        }
-
-        if (!defined($strArchiveRetentionType))
-        {
-            $strArchiveRetentionType = BACKUP_TYPE_FULL;
-        }
-    }
-
-    # If no archive retention type is set then exit
-    if (!defined($strArchiveRetentionType))
-    {
-        &log(INFO, 'archive retention type not set - archive logs will not be expired');
+         &log(INFO, "option '" . &OPTION_RETENTION_ARCHIVE . "' is not set - archive logs will not be expired");
     }
     else
     {
-        # Determine which backup type to use for archive retention (full, differential, incremental)
+        # Determine which backup type to use for archive retention (full, differential, incremental) and get a list of the
+        # remaining non-expired backups based on the type.
         if ($strArchiveRetentionType eq BACKUP_TYPE_FULL)
         {
             @stryPath = $oBackupInfo->list(backupRegExpGet(true), 'reverse');
@@ -285,7 +252,7 @@ sub process
             @stryPath = $oBackupInfo->list(backupRegExpGet(true, true, true), 'reverse');
         }
 
-        # if no backups were found then preserve current archive logs - too soon to expire them
+        # If no backups were found then preserve current archive logs - too soon to expire them
         my $iBackupTotal = scalar @stryPath;
 
         if ($iBackupTotal > 0)
@@ -318,9 +285,9 @@ sub process
 
                     my $strArchiveExpireMax;
 
-                    # Get archive ranges to preserve.  Because archive retention can be less than total retention it is
-                    # important to preserve older archive that is required to make the older backups consistent even though
-                    # they cannot be played any further forward with PITR.
+                    # Get archive ranges to preserve.  Because archive retention can be less than total retention it is important
+                    # to preserve archive that is required to make the older backups consistent even though they cannot be played
+                    # any further forward with PITR.
                     my @oyArchiveRange;
 
                     foreach my $strBackup ($oBackupInfo->list())
@@ -351,7 +318,7 @@ sub process
                     }
 
                     # Get all major archive paths (timeline and first 64 bits of LSN)
-                    foreach my $strPath ($oFile->list(PATH_BACKUP_ARCHIVE, $strArchiveId, "^[0-F]{16}\$"))
+                    foreach my $strPath ($oFile->list(PATH_BACKUP_ARCHIVE, $strArchiveId, REGEX_ARCHIVE_DIR_WAL))
                     {
                         logDebugMisc($strOperation, "found major WAL path: ${strPath}");
                         $bRemove = true;
@@ -431,10 +398,7 @@ sub process
     }
 
     # Return from function and log return values if any
-    return logDebugReturn
-    (
-        $strOperation
-    );
+    return logDebugReturn($strOperation);
 }
 
 1;
