@@ -14,21 +14,13 @@ use File::Basename qw(dirname basename);
 use Digest::SHA;
 use Time::Local qw(timelocal);
 
-use lib dirname($0);
+use pgBackRest::DbVersion;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::File;
 use pgBackRest::FileCommon;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_MANIFEST                                            => 'Manifest';
-
-use constant OP_MANIFEST_BUILD                                      => OP_MANIFEST . '->build';
-use constant OP_MANIFEST_NEW                                        => OP_MANIFEST . '->new';
-use constant OP_MANIFEST_SAVE                                       => OP_MANIFEST . '->save';
+use pgBackRest::Config::Config;
 
 ####################################################################################################################################
 # File/path constants
@@ -100,12 +92,16 @@ use constant MANIFEST_KEY_TYPE                                      => 'backup-t
     push @EXPORT, qw(MANIFEST_KEY_TYPE);
 
 # Options that were set when the backup was made
+use constant MANIFEST_KEY_BACKUP_STANDBY                            => 'option-' . OPTION_BACKUP_STANDBY;
+    push @EXPORT, qw(MANIFEST_KEY_BACKUP_STANDBY);
 use constant MANIFEST_KEY_HARDLINK                                  => 'option-hardlink';
     push @EXPORT, qw(MANIFEST_KEY_HARDLINK);
 use constant MANIFEST_KEY_ARCHIVE_CHECK                             => 'option-archive-check';
     push @EXPORT, qw(MANIFEST_KEY_ARCHIVE_CHECK);
 use constant MANIFEST_KEY_ARCHIVE_COPY                              => 'option-archive-copy';
     push @EXPORT, qw(MANIFEST_KEY_ARCHIVE_COPY);
+use constant MANIFEST_KEY_CHECKSUM_PAGE                             => 'option-' . OPTION_CHECKSUM_PAGE;
+    push @EXPORT, qw(MANIFEST_KEY_CHECKSUM_PAGE);
 use constant MANIFEST_KEY_COMPRESS                                  => 'option-compress';
     push @EXPORT, qw(MANIFEST_KEY_COMPRESS);
 use constant MANIFEST_KEY_ONLINE                                    => 'option-online';
@@ -128,6 +124,10 @@ use constant MANIFEST_KEY_DB_VERSION                                => 'db-versi
 # Subkeys used for path/file/link info
 use constant MANIFEST_SUBKEY_CHECKSUM                               => 'checksum';
     push @EXPORT, qw(MANIFEST_SUBKEY_CHECKSUM);
+use constant MANIFEST_SUBKEY_CHECKSUM_PAGE                          => 'checksum-page';
+    push @EXPORT, qw(MANIFEST_SUBKEY_CHECKSUM_PAGE);
+use constant MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR                    => 'checksum-page-error';
+    push @EXPORT, qw(MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR);
 use constant MANIFEST_SUBKEY_DESTINATION                            => 'destination';
     push @EXPORT, qw(MANIFEST_SUBKEY_DESTINATION);
 use constant MANIFEST_SUBKEY_FILE                                   => 'file';
@@ -136,6 +136,8 @@ use constant MANIFEST_SUBKEY_FUTURE                                 => 'future';
     push @EXPORT, qw(MANIFEST_SUBKEY_FUTURE);
 use constant MANIFEST_SUBKEY_GROUP                                  => 'group';
     push @EXPORT, qw(MANIFEST_SUBKEY_GROUP);
+use constant MANIFEST_SUBKEY_MASTER                                 => 'master';
+    push @EXPORT, qw(MANIFEST_SUBKEY_MASTER);
 use constant MANIFEST_SUBKEY_MODE                                   => 'mode';
     push @EXPORT, qw(MANIFEST_SUBKEY_MODE);
 use constant MANIFEST_SUBKEY_TIMESTAMP                              => 'timestamp';
@@ -160,8 +162,28 @@ use constant MANIFEST_SUBKEY_USER                                   => 'user';
 ####################################################################################################################################
 # Database locations for important files/paths
 ####################################################################################################################################
+use constant DB_PATH_BASE                                           => 'base';
+    push @EXPORT, qw(DB_PATH_BASE);
+use constant DB_PATH_PGCLOG                                         => 'pg_clog';
+    push @EXPORT, qw(DB_PATH_PGCLOG);
 use constant DB_PATH_GLOBAL                                         => 'global';
     push @EXPORT, qw(DB_PATH_GLOBAL);
+use constant DB_PATH_PGDYNSHMEM                                     => 'pg_dynshmem';
+    push @EXPORT, qw(DB_PATH_PGDYNSHMEM);
+use constant DB_PATH_PGMULTIXACT                                    => 'pg_multixact';
+    push @EXPORT, qw(DB_PATH_PGMULTIXACT);
+use constant DB_PATH_PGNOTIFY                                       => 'pg_notify';
+    push @EXPORT, qw(DB_PATH_PGNOTIFY);
+use constant DB_PATH_PGREPLSLOT                                     => 'pg_replslot';
+    push @EXPORT, qw(DB_PATH_PGREPLSLOT);
+use constant DB_PATH_PGSERIAL                                       => 'pg_serial';
+    push @EXPORT, qw(DB_PATH_PGSERIAL);
+use constant DB_PATH_PGSNAPSHOTS                                    => 'pg_snapshots';
+    push @EXPORT, qw(DB_PATH_PGSNAPSHOTS);
+use constant DB_PATH_PGSTATTMP                                      => 'pg_stat_tmp';
+    push @EXPORT, qw(DB_PATH_PGSTATTMP);
+use constant DB_PATH_PGSUBTRANS                                     => 'pg_subtrans';
+    push @EXPORT, qw(DB_PATH_PGSUBTRANS);
 use constant DB_PATH_PGTBLSPC                                       => 'pg_tblspc';
     push @EXPORT, qw(DB_PATH_PGTBLSPC);
 use constant DB_PATH_PGXLOG                                         => 'pg_xlog';
@@ -173,8 +195,14 @@ use constant DB_FILE_BACKUPLABELOLD                                 => DB_FILE_B
     push @EXPORT, qw(DB_FILE_BACKUPLABELOLD);
 use constant DB_FILE_PGCONTROL                                      => DB_PATH_GLOBAL . '/pg_control';
     push @EXPORT, qw(DB_FILE_PGCONTROL);
+use constant DB_FILE_PGFILENODEMAP                                  => 'pg_filenode.map';
+    push @EXPORT, qw(DB_FILE_PGFILENODEMAP);
+use constant DB_FILE_PGINTERNALINIT                                 => 'pg_internal.init';
+    push @EXPORT, qw(DB_FILE_PGINTERNALINIT);
 use constant DB_FILE_PGVERSION                                      => 'PG_VERSION';
     push @EXPORT, qw(DB_FILE_PGVERSION);
+use constant DB_FILE_POSTGRESQLAUTOCONFTMP                          => 'postgresql.auto.conf.tmp';
+    push @EXPORT, qw(DB_FILE_POSTGRESQLAUTOCONFTMP);
 use constant DB_FILE_POSTMASTEROPTS                                 => 'postmaster.opts';
     push @EXPORT, qw(DB_FILE_POSTMASTEROPTS);
 use constant DB_FILE_POSTMASTERPID                                  => 'postmaster.pid';
@@ -186,16 +214,55 @@ use constant DB_FILE_RECOVERYDONE                                   => 'recovery
 use constant DB_FILE_TABLESPACEMAP                                  => 'tablespace_map';
     push @EXPORT, qw(DB_FILE_TABLESPACEMAP);
 
+use constant DB_FILE_PREFIX_TMP                                     => 'pgsql_tmp';
+    push @EXPORT, qw(DB_FILE_PREFIX_TMP);
+
 ####################################################################################################################################
 # Manifest locations for important files/paths
 ####################################################################################################################################
+use constant MANIFEST_PATH_BASE                                     => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_BASE;
+    push @EXPORT, qw(MANIFEST_PATH_BASE);
+use constant MANIFEST_PATH_PGCLOG                                   => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGCLOG;
+    push @EXPORT, qw(MANIFEST_PATH_PGCLOG);
+use constant MANIFEST_PATH_GLOBAL                                   => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_GLOBAL;
+    push @EXPORT, qw(MANIFEST_PATH_GLOBAL);
+use constant MANIFEST_PATH_PGDYNSHMEM                               => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGDYNSHMEM;
+    push @EXPORT, qw(MANIFEST_PATH_PGDYNSHMEM);
+use constant MANIFEST_PATH_PGMULTIXACT                              => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGMULTIXACT;
+    push @EXPORT, qw(MANIFEST_PATH_PGMULTIXACT);
+use constant MANIFEST_PATH_PGNOTIFY                                 => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGNOTIFY;
+    push @EXPORT, qw(MANIFEST_PATH_PGNOTIFY);
+use constant MANIFEST_PATH_PGREPLSLOT                               => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGREPLSLOT;
+    push @EXPORT, qw(MANIFEST_PATH_PGREPLSLOT);
+use constant MANIFEST_PATH_PGSERIAL                                 => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGSERIAL;
+    push @EXPORT, qw(MANIFEST_PATH_PGSERIAL);
+use constant MANIFEST_PATH_PGSNAPSHOTS                              => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGSNAPSHOTS;
+    push @EXPORT, qw(MANIFEST_PATH_PGSNAPSHOTS);
+use constant MANIFEST_PATH_PGSTATTMP                                => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGSTATTMP;
+    push @EXPORT, qw(MANIFEST_PATH_PGSTATTMP);
+use constant MANIFEST_PATH_PGSUBTRANS                               => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGSUBTRANS;
+    push @EXPORT, qw(MANIFEST_PATH_PGSUBTRANS);
+use constant MANIFEST_PATH_PGTBLSPC                                 => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGTBLSPC;
+    push @EXPORT, qw(MANIFEST_PATH_PGTBLSPC);
 use constant MANIFEST_PATH_PGXLOG                                   => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGXLOG;
     push @EXPORT, qw(MANIFEST_PATH_PGXLOG);
 
 use constant MANIFEST_FILE_BACKUPLABEL                              => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_BACKUPLABEL;
     push @EXPORT, qw(MANIFEST_FILE_BACKUPLABEL);
+use constant MANIFEST_FILE_BACKUPLABELOLD                           => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_BACKUPLABELOLD;
+    push @EXPORT, qw(MANIFEST_FILE_BACKUPLABELOLD);
 use constant MANIFEST_FILE_PGCONTROL                                => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_PGCONTROL;
     push @EXPORT, qw(MANIFEST_FILE_PGCONTROL);
+use constant MANIFEST_FILE_POSTGRESQLAUTOCONFTMP                    => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_POSTGRESQLAUTOCONFTMP;
+    push @EXPORT, qw(MANIFEST_FILE_PGCONTROL);
+use constant MANIFEST_FILE_POSTMASTEROPTS                           => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_POSTMASTEROPTS;
+    push @EXPORT, qw(MANIFEST_FILE_POSTMASTEROPTS);
+use constant MANIFEST_FILE_POSTMASTERPID                            => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_POSTMASTERPID;
+    push @EXPORT, qw(MANIFEST_FILE_POSTMASTERPID);
+use constant MANIFEST_FILE_RECOVERYCONF                             => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_RECOVERYCONF;
+    push @EXPORT, qw(MANIFEST_FILE_RECOVERYCONF);
+use constant MANIFEST_FILE_RECOVERYDONE                             => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_RECOVERYDONE;
+    push @EXPORT, qw(MANIFEST_FILE_RECOVERYDONE);
 use constant MANIFEST_FILE_TABLESPACEMAP                            => MANIFEST_TARGET_PGDATA . '/' . DB_FILE_TABLESPACEMAP;
     push @EXPORT, qw(MANIFEST_FILE_TABLESPACEMAP);
 
@@ -205,6 +272,14 @@ use constant MANIFEST_FILE_TABLESPACEMAP                            => MANIFEST_
 use constant DB_USER_OBJECT_MINIMUM_ID                              => 16384;
     push @EXPORT, qw(DB_USER_OBJECT_MINIMUM_ID);
 
+####################################################################################################################################
+# Expression to determine whether files can be copied from a standby
+####################################################################################################################################
+use constant COPY_STANDBY_EXPRESSION                                => '^(' . MANIFEST_TARGET_PGDATA . '\/' .
+                                                                       '(' . DB_PATH_BASE . '|' . DB_PATH_GLOBAL . '|' .
+                                                                       DB_PATH_PGCLOG . '|' . DB_PATH_PGMULTIXACT . ')|' .
+                                                                       DB_PATH_PGTBLSPC . ')\/';
+
 ####################################################################################################################################
 # new
 ####################################################################################################################################
@@ -221,7 +296,7 @@ sub new
     ) =
         logDebugParam
         (
-            OP_MANIFEST_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'strFileName', trace => true},
             {name => 'bLoad', required => false, trace => true}
         );
@@ -250,26 +325,15 @@ sub save
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation,
-    ) =
-        logDebugParam
-        (
-            OP_MANIFEST_SAVE
-        );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->save');
 
     # Call inherited save
     $self->SUPER::save();
 
     # Return from function and log return values if any
-    return logDebugReturn
-    (
-        $strOperation
-    );
+    return logDebugReturn($strOperation);
 }
 
-
 ####################################################################################################################################
 # get
 #
@@ -290,7 +354,7 @@ sub get
         ($strSection eq MANIFEST_SECTION_TARGET_FILE || $strSection eq MANIFEST_SECTION_TARGET_PATH ||
          $strSection eq MANIFEST_SECTION_TARGET_LINK) &&
         ($strSubKey eq MANIFEST_SUBKEY_USER || $strSubKey eq MANIFEST_SUBKEY_GROUP ||
-         $strSubKey eq MANIFEST_SUBKEY_MODE) &&
+         $strSubKey eq MANIFEST_SUBKEY_MODE || $strSubKey eq MANIFEST_SUBKEY_MASTER) &&
         $self->test($strSection, $strKey))
     {
         $oValue = $self->SUPER::get("${strSection}:default", $strSubKey, undef, $bRequired, $oDefault);
@@ -491,11 +555,12 @@ sub build
     (
         $strOperation,
         $oFile,
+        $strDbVersion,
         $strPath,
         $oLastManifest,
         $bOnline,
-        $oTablespaceMapRef,
-        $oDatabaseMapRef,
+        $hTablespaceMap,
+        $hDatabaseMap,
         $strLevel,
         $bTablespace,
         $strParentPath,
@@ -503,13 +568,14 @@ sub build
     ) =
         logDebugParam
         (
-            OP_MANIFEST_BUILD, \@_,
+            __PACKAGE__ . '->build', \@_,
             {name => 'oFile'},
+            {name => 'strDbVersion'},
             {name => 'strPath'},
             {name => 'oLastManifest', required => false},
             {name => 'bOnline'},
-            {name => 'oTablespaceMapRef', required => false},
-            {name => 'oDatabaseMapRef', required => false},
+            {name => 'hTablespaceMap', required => false},
+            {name => 'hDatabaseMap', required => false},
             {name => 'strLevel', required => false},
             {name => 'bTablespace', required => false},
             {name => 'strParentPath', required => false},
@@ -521,23 +587,21 @@ sub build
         $strLevel = MANIFEST_TARGET_PGDATA;
 
         # If not online then build the tablespace map from pg_tblspc path
-        if (!$bOnline && !defined($oTablespaceMapRef))
+        if (!$bOnline && !defined($hTablespaceMap))
         {
-            $oTablespaceMapRef = {};
+            my $hTablespaceManifest = $oFile->manifest(PATH_DB_ABSOLUTE, $strPath . '/' . DB_PATH_PGTBLSPC);
+            $hTablespaceMap = {};
 
-            my %oTablespaceManifestHash;
-            $oFile->manifest(PATH_DB_ABSOLUTE, $strPath . '/' . DB_PATH_PGTBLSPC, \%oTablespaceManifestHash);
-
-            foreach my $strName (sort(CORE::keys(%{$oTablespaceManifestHash{name}})))
+            foreach my $strOid (sort(CORE::keys(%{$hTablespaceManifest})))
             {
-                if ($strName eq '.' or $strName eq '..')
+                if ($strOid eq '.' or $strOid eq '..')
                 {
                     next;
                 }
 
-                logDebugMisc($strOperation, "found tablespace ${strName}");
+                logDebugMisc($strOperation, "found tablespace ${strOid} in offline mode");
 
-                ${$oTablespaceMapRef}{oid}{$strName}{name} = "ts${strName}";
+                $hTablespaceMap->{$strOid} = "ts${strOid}";
             }
         }
     }
@@ -550,7 +614,7 @@ sub build
     {
         my $iTablespaceId = (split('\/', $strLevel))[1];
 
-        if (!defined(${$oTablespaceMapRef}{oid}{$iTablespaceId}{name}))
+        if (!defined($hTablespaceMap->{$iTablespaceId}))
         {
             confess &log(ASSERT, "tablespace with oid ${iTablespaceId} not found in tablespace map\n" .
                                  "HINT: was a tablespace created or dropped during the backup?");
@@ -558,7 +622,7 @@ sub build
 
         $self->set(MANIFEST_SECTION_BACKUP_TARGET, $strLevel, MANIFEST_SUBKEY_TABLESPACE_ID, $iTablespaceId);
         $self->set(MANIFEST_SECTION_BACKUP_TARGET, $strLevel, MANIFEST_SUBKEY_TABLESPACE_NAME,
-                   ${$oTablespaceMapRef}{oid}{$iTablespaceId}{name});
+                   $hTablespaceMap->{$iTablespaceId});
     }
 
     if (index($strPath, '/') != 0)
@@ -572,17 +636,25 @@ sub build
     }
 
     # Get the manifest for this level
-    my %oManifestHash;
-    $oFile->manifest(PATH_DB_ABSOLUTE, $strPath, \%oManifestHash);
+    my $hManifest = $oFile->manifest(PATH_DB_ABSOLUTE, $strPath);
     my $strManifestType = MANIFEST_VALUE_LINK;
 
     # Loop though all paths/files/links in the manifest
-    foreach my $strName (sort(CORE::keys(%{$oManifestHash{name}})))
+    foreach my $strName (sort(CORE::keys(%{$hManifest})))
     {
         my $strFile = $strLevel;
 
         if ($strName ne '.')
         {
+            if ($strManifestType eq MANIFEST_VALUE_LINK && $hManifest->{$strName}{type} eq 'l')
+            {
+                confess &log(ERROR, 'link \'' .
+                    $self->dbPathGet(
+                        $self->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH), $strLevel) .
+                        '\' -> \'' . $self->get(MANIFEST_SECTION_BACKUP_TARGET, $strLevel, MANIFEST_SUBKEY_PATH) .
+                        '\' cannot reference another link', ERROR_LINK_DESTINATION);
+            }
+
             # Make sure the current file matches the filter or any files under the filter
             if (defined($strFilter) && $strName ne $strFilter && index($strName, "${strFilter}/") != 0)
             {
@@ -591,15 +663,6 @@ sub build
 
             if ($strManifestType eq MANIFEST_VALUE_LINK)
             {
-                if ($oManifestHash{name}{$strName}{type} eq 'l')
-                {
-                    confess &log(ERROR, 'link ' .
-                        $self->dbPathGet(
-                            $self->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH), $strLevel) .
-                            ' (' . $self->get(MANIFEST_SECTION_BACKUP_TARGET, $strLevel, MANIFEST_SUBKEY_PATH) . ')' .
-                            ' cannot reference another link', ERROR_LINK_DESTINATION);
-                }
-
                 $strFile = dirname($strFile);
                 $self->set(MANIFEST_SECTION_BACKUP_TARGET, $strLevel, MANIFEST_SUBKEY_PATH,
                            dirname($self->get(MANIFEST_SECTION_BACKUP_TARGET, $strLevel, MANIFEST_SUBKEY_PATH)));
@@ -613,19 +676,37 @@ sub build
             $strManifestType = MANIFEST_VALUE_PATH;
         }
 
-        # Skip certain files during backup
-        if ($strFile =~ ('^' . MANIFEST_PATH_PGXLOG . '.*\/') && $bOnline ||  # pg_xlog/ - this will be reconstructed
-            $strLevel eq MANIFEST_TARGET_PGDATA &&
-            ($strName eq DB_FILE_BACKUPLABELOLD ||                  # backup_label.old - old backup labels are not useful
-             $strName eq DB_FILE_POSTMASTEROPTS ||                  # postmaster.opts - not useful for backup
-             $strName eq DB_FILE_POSTMASTERPID ||                   # postmaster.pid - to avoid confusing postgres after restore
-             $strName eq DB_FILE_RECOVERYCONF ||                    # recovery.conf - doesn't make sense to backup this file
-             $strName eq DB_FILE_RECOVERYDONE))                     # recovery.done - doesn't make sense to backup this file
+        # Skip pg_xlog/* when doing an online backup.  WAL will be restored from the archive or stored in pg_xlog at the end of the
+        # backup if the archive-copy option is set.
+        next if ($strFile =~ ('^' . MANIFEST_PATH_PGXLOG . '\/') && $bOnline);
+
+        # Skip all directories and files that start with pgsql_tmp.  The files are removed when the server is restarted and the
+        # directories are recreated.  Since temp files cannnot be created on the replica it makes sense to delete the directories
+        # and let the server recreate them when they are needed.
+        next if $strName =~ ('(^|\/)' . DB_FILE_PREFIX_TMP);
+
+        # Skip temporary statistics in pg_stat_tmp even when stats_temp_directory is set because PGSS_TEXT_FILE is always created
+        # there.
+        next if $strFile =~ ('^' . MANIFEST_PATH_PGSTATTMP . '\/') && $strDbVersion >= PG_VERSION_84;
+
+        # Skip pg_replslot/* since these files are generally not useful after a restore
+        next if $strFile =~ ('^' . MANIFEST_PATH_PGREPLSLOT . '\/') && $strDbVersion >= PG_VERSION_94;
+
+        # Skip pg_subtrans/* since these files are reset
+        next if $strFile =~ ('^' . MANIFEST_PATH_PGSUBTRANS . '\/');
+
+        # Skip ignored files
+        if ($strFile eq MANIFEST_FILE_POSTGRESQLAUTOCONFTMP ||      # postgresql.auto.conf.tmp - temp file for safe writes
+            $strFile eq MANIFEST_FILE_BACKUPLABELOLD ||             # backup_label.old - old backup labels are not useful
+            $strFile eq MANIFEST_FILE_POSTMASTEROPTS ||             # postmaster.opts - not useful for backup
+            $strFile eq MANIFEST_FILE_POSTMASTERPID ||              # postmaster.pid - to avoid confusing postgres after restore
+            $strFile eq MANIFEST_FILE_RECOVERYCONF ||               # recovery.conf - doesn't make sense to backup this file
+            $strFile eq MANIFEST_FILE_RECOVERYDONE)                 # recovery.done - doesn't make sense to backup this file
         {
             next;
         }
 
-        my $cType = $oManifestHash{name}{$strName}{type};
+        my $cType = $hManifest->{$strName}{type};
         my $strSection = MANIFEST_SECTION_TARGET_PATH;
 
         if ($cType eq 'f')
@@ -650,36 +731,36 @@ sub build
             $strFile = MANIFEST_TARGET_PGDATA . '/' . $strName;
 
             # Check for files in DB_PATH_PGTBLSPC that are not links
-            if ($oManifestHash{name}{$strName}{type} ne 'l')
+            if ($hManifest->{$strName}{type} ne 'l')
             {
                 confess &log(ERROR, "${strName} is not a symlink - " . DB_PATH_PGTBLSPC . ' should contain only symlinks',
                              ERROR_LINK_EXPECTED);
             }
 
             # Check for tablespaces in PGDATA
-            if (index($oManifestHash{name}{$strName}{link_destination}, $strPath) == 0 ||
-                (index($oManifestHash{name}{$strName}{link_destination}, '/') != 0 &&
+            if (index($hManifest->{$strName}{link_destination}, "${strPath}/") == 0 ||
+                (index($hManifest->{$strName}{link_destination}, '/') != 0 &&
                  index(pathAbsolute($strPath . '/' . DB_PATH_PGTBLSPC,
-                       $oManifestHash{name}{$strName}{link_destination}), $strPath) == 0))
+                       $hManifest->{$strName}{link_destination}) . '/', "${strPath}/") == 0))
             {
-                confess &log(ERROR, 'tablespace symlink ' . $oManifestHash{name}{$strName}{link_destination} .
+                confess &log(ERROR, 'tablespace symlink ' . $hManifest->{$strName}{link_destination} .
                              ' destination must not be in $PGDATA', ERROR_TABLESPACE_IN_PGDATA);
             }
         }
 
         # User and group required for all types
-        if (defined($oManifestHash{name}{$strName}{user}))
+        if (defined($hManifest->{$strName}{user}))
         {
-            $self->set($strSection, $strFile, MANIFEST_SUBKEY_USER, $oManifestHash{name}{$strName}{user});
+            $self->set($strSection, $strFile, MANIFEST_SUBKEY_USER, $hManifest->{$strName}{user});
         }
         else
         {
             $self->boolSet($strSection, $strFile, MANIFEST_SUBKEY_USER, false);
         }
 
-        if (defined($oManifestHash{name}{$strName}{group}))
+        if (defined($hManifest->{$strName}{group}))
         {
-            $self->set($strSection, $strFile, MANIFEST_SUBKEY_GROUP, $oManifestHash{name}{$strName}{group});
+            $self->set($strSection, $strFile, MANIFEST_SUBKEY_GROUP, $hManifest->{$strName}{group});
         }
         else
         {
@@ -689,21 +770,23 @@ sub build
         # Mode for required file and path type only
         if ($cType eq 'f' || $cType eq 'd')
         {
-            $self->set($strSection, $strFile, MANIFEST_SUBKEY_MODE, $oManifestHash{name}{$strName}{mode});
+            $self->set($strSection, $strFile, MANIFEST_SUBKEY_MODE, $hManifest->{$strName}{mode});
         }
 
         # Modification time and size required for file type only
         if ($cType eq 'f')
         {
             $self->set($strSection, $strFile, MANIFEST_SUBKEY_TIMESTAMP,
-                       $oManifestHash{name}{$strName}{modification_time} + 0);
-            $self->set($strSection, $strFile, MANIFEST_SUBKEY_SIZE, $oManifestHash{name}{$strName}{size} + 0);
+                       $hManifest->{$strName}{modification_time} + 0);
+            $self->set($strSection, $strFile, MANIFEST_SUBKEY_SIZE, $hManifest->{$strName}{size} + 0);
+            $self->boolSet($strSection, $strFile, MANIFEST_SUBKEY_MASTER,
+                ($strFile eq MANIFEST_FILE_PGCONTROL || $strFile !~ COPY_STANDBY_EXPRESSION) ? true : false);
         }
 
         # Link destination required for link type only
         if ($cType eq 'l')
         {
-            my $strLinkDestination = $oManifestHash{name}{$strName}{link_destination};
+            my $strLinkDestination = $hManifest->{$strName}{link_destination};
             $self->set($strSection, $strFile, MANIFEST_SUBKEY_DESTINATION, $strLinkDestination);
 
             # If this is a tablespace then set the filter to use for the next level
@@ -722,7 +805,7 @@ sub build
 
             $strPath = dirname("${strPath}/${strName}");
 
-            $self->build($oFile, $strLinkDestination, undef, $bOnline, $oTablespaceMapRef, $oDatabaseMapRef,
+            $self->build($oFile, $strDbVersion, $strLinkDestination, undef, $bOnline, $hTablespaceMap, $hDatabaseMap,
                          $strFile, $bTablespace, $strPath, $strFilter, $strLinkDestination);
         }
     }
@@ -749,12 +832,12 @@ sub build
         }
 
         # Store database map information when provided during an online backup.
-        foreach my $strDbName (sort(keys(%{${$oDatabaseMapRef}{name}})))
+        foreach my $strDbName (sort(keys(%{$hDatabaseMap})))
         {
             $self->numericSet(MANIFEST_SECTION_DB, $strDbName, MANIFEST_KEY_DB_ID,
-                              ${$oDatabaseMapRef}{'name'}{$strDbName}{&MANIFEST_KEY_DB_ID});
+                              $hDatabaseMap->{$strDbName}{&MANIFEST_KEY_DB_ID});
             $self->numericSet(MANIFEST_SECTION_DB, $strDbName, MANIFEST_KEY_DB_LAST_SYSTEM_ID,
-                              ${$oDatabaseMapRef}{'name'}{$strDbName}{&MANIFEST_KEY_DB_LAST_SYSTEM_ID});
+                              $hDatabaseMap->{$strDbName}{&MANIFEST_KEY_DB_LAST_SYSTEM_ID});
         }
 
         # Loop though all files
@@ -801,11 +884,37 @@ sub build
                                $oLastManifest->get(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM));
                 }
 
+                # Copy repo size from the previous manifest
                 if ($oLastManifest->test(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_REPO_SIZE))
                 {
                     $self->set(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_REPO_SIZE,
                                $oLastManifest->get(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_REPO_SIZE));
                 }
+
+                # Copy master flag from the previous manifest (if it exists)
+                if ($oLastManifest->test(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_MASTER))
+                {
+                    $self->set(
+                        MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_MASTER,
+                        $oLastManifest->get(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_MASTER));
+                }
+
+                # Copy checksum page from the previous manifest (if it exists)
+                my $bChecksumPage = $oLastManifest->get(
+                    MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE, false);
+
+                if (defined($bChecksumPage))
+                {
+                    $self->boolSet(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE, $bChecksumPage);
+
+                    if (!$bChecksumPage &&
+                        $oLastManifest->test(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR))
+                    {
+                        $self->set(
+                            MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR,
+                            $oLastManifest->get(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR));
+                    }
+                }
             }
         }
 
@@ -837,7 +946,7 @@ sub linkCheck
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam(OP_MANIFEST . '->linkCheck');
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->linkCheck');
 
     # Working variable
     my $strBasePath = $self->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH);
@@ -854,7 +963,8 @@ sub linkCheck
                 {
                     my $strChildPath = $self->get(MANIFEST_SECTION_BACKUP_TARGET, $strTargetChild, MANIFEST_SUBKEY_PATH);
 
-                    if (index(pathAbsolute($strBasePath, $strChildPath), pathAbsolute($strBasePath, $strParentPath)) == 0)
+                    if (index(
+                        pathAbsolute($strBasePath, $strChildPath) . '/', pathAbsolute($strBasePath, $strParentPath) . '/') == 0)
                     {
                         confess &log(ERROR, 'link ' . $self->dbPathGet($strBasePath, $strTargetChild) .
                                             " (${strChildPath}) references a subdirectory of or" .
@@ -884,7 +994,8 @@ sub fileAdd
         $strManifestFile,
         $lModificationTime,
         $lSize,
-        $strChecksum
+        $strChecksum,
+        $bMaster,
     ) =
         logDebugParam
         (
@@ -892,7 +1003,8 @@ sub fileAdd
             {name => 'strManifestFile'},
             {name => 'lModificationTime'},
             {name => 'lSize'},
-            {name => 'lChecksum'}
+            {name => 'lChecksum'},
+            {name => 'bMaster'},
         );
 
     # Set manifest values
@@ -921,6 +1033,7 @@ sub fileAdd
     $self->set(MANIFEST_SECTION_TARGET_FILE, $strManifestFile, MANIFEST_SUBKEY_TIMESTAMP, $lModificationTime);
     $self->set(MANIFEST_SECTION_TARGET_FILE, $strManifestFile, MANIFEST_SUBKEY_SIZE, $lSize);
     $self->set(MANIFEST_SECTION_TARGET_FILE, $strManifestFile, MANIFEST_SUBKEY_CHECKSUM, $strChecksum);
+    $self->boolSet(MANIFEST_SECTION_TARGET_FILE, $strManifestFile, MANIFEST_SUBKEY_MASTER, $bMaster);
 }
 
 ####################################################################################################################################
@@ -933,33 +1046,37 @@ sub buildDefault
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam(OP_MANIFEST . '->buildDefault');
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->buildDefault');
 
     # Set defaults for subkeys that tend to repeat
     foreach my $strSection (&MANIFEST_SECTION_TARGET_FILE, &MANIFEST_SECTION_TARGET_PATH, &MANIFEST_SECTION_TARGET_LINK)
     {
-        foreach my $strSubKey (&MANIFEST_SUBKEY_USER, &MANIFEST_SUBKEY_GROUP, &MANIFEST_SUBKEY_MODE)
+        foreach my $strSubKey (&MANIFEST_SUBKEY_USER, &MANIFEST_SUBKEY_GROUP, &MANIFEST_SUBKEY_MODE, &MANIFEST_SUBKEY_MASTER)
         {
             # Links don't have a mode so skip
             next if ($strSection eq MANIFEST_SECTION_TARGET_LINK && $strSubKey eq &MANIFEST_SUBKEY_MODE);
 
+            # Only files have the master subkey
+            next if ($strSection ne MANIFEST_SECTION_TARGET_FILE && $strSubKey eq &MANIFEST_SUBKEY_MASTER);
+
             my %oDefault;
             my $iSectionTotal = 0;
 
             foreach my $strFile ($self->keys($strSection))
             {
-                if (!$self->boolTest($strSection, $strFile, $strSubKey, false))
-                {
-                    my $strValue = $self->get($strSection, $strFile, $strSubKey);
+                # Don't count false values when subkey in (MANIFEST_SUBKEY_USER, MANIFEST_SUBKEY_GROUP)
+                next if (($strSubKey eq MANIFEST_SUBKEY_USER || $strSubKey eq MANIFEST_SUBKEY_GROUP) &&
+                         $self->boolTest($strSection, $strFile, $strSubKey, false));
 
-                    if (defined($oDefault{$strValue}))
-                    {
-                        $oDefault{$strValue}++;
-                    }
-                    else
-                    {
-                        $oDefault{$strValue} = 1;
-                    }
+                my $strValue = $self->get($strSection, $strFile, $strSubKey);
+
+                if (defined($oDefault{$strValue}))
+                {
+                    $oDefault{$strValue}++;
+                }
+                else
+                {
+                    $oDefault{$strValue} = 1;
                 }
 
                 $iSectionTotal++;
@@ -979,7 +1096,14 @@ sub buildDefault
 
             if (defined($strMaxValue) > 0 && $iMaxValueTotal > $iSectionTotal * MANIFEST_DEFAULT_MATCH_FACTOR)
             {
-                $self->set("${strSection}:default", $strSubKey, undef, $strMaxValue);
+                if ($strSubKey eq MANIFEST_SUBKEY_MASTER)
+                {
+                    $self->boolSet("${strSection}:default", $strSubKey, undef, $strMaxValue);
+                }
+                else
+                {
+                    $self->set("${strSection}:default", $strSubKey, undef, $strMaxValue);
+                }
 
                 foreach my $strFile ($self->keys($strSection))
                 {
@@ -996,4 +1120,62 @@ sub buildDefault
     return logDebugReturn($strOperation);
 }
 
+####################################################################################################################################
+# validate
+#
+# Checks for any mising values or inconsistencies in the manifest.
+####################################################################################################################################
+sub validate
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my ($strOperation) = logDebugParam(__PACKAGE__ . 'validate');
+
+    # Make sure that all files have size and checksum (when size > 0).  Since these values are removed before the backup file copy
+    # starts this ensures that all files had results stored in the manifest during the file copy.
+    foreach my $strFile ($self->keys(MANIFEST_SECTION_TARGET_FILE))
+    {
+        # Ensure size is set
+        if (!$self->test(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_SIZE))
+        {
+            confess &log(ASSERT, "manifest subvalue 'size' not set for file '${strFile}'");
+        }
+
+        # If size > 0 then checksum must also be set
+        if ($self->numericGet(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_SIZE) > 0 &&
+            !$self->test(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_CHECKSUM))
+        {
+            confess &log(ASSERT, "manifest subvalue 'checksum' not set for file '${strFile}'");
+        }
+    }
+
+    # Return from function and log return values if any
+    return logDebugReturn($strOperation);
+}
+
+####################################################################################################################################
+# isChecksumPage
+#
+# Can this file have page checksums in PG >= 9.3?
+####################################################################################################################################
+sub isChecksumPage
+{
+    my $strFile = shift;
+
+    if (($strFile =~ ('^' . MANIFEST_TARGET_PGDATA . '\/' . DB_PATH_BASE) &&
+            $strFile !~ ('(' . DB_FILE_PGFILENODEMAP . '|' . DB_FILE_PGINTERNALINIT . '|' . DB_FILE_PGVERSION . ')$')) ||
+        ($strFile =~ ('^' . MANIFEST_TARGET_PGDATA . '\/' . DB_PATH_GLOBAL) &&
+            $strFile !~ ('(' . DB_FILE_PGFILENODEMAP . '|' . DB_FILE_PGINTERNALINIT . '|' . DB_FILE_PGVERSION . '|' .
+            DB_FILE_PGCONTROL . ')$')) ||
+        ($strFile =~ ('^' . MANIFEST_TARGET_PGTBLSPC . '\/') && $strFile !~ (DB_FILE_PGVERSION . '$')))
+    {
+        return true;
+    }
+
+    return false;
+}
+
+push @EXPORT, qw(isChecksumPage);
+
 1;
