@@ -11,21 +11,22 @@ use Exporter qw(import);
     our @EXPORT = qw();
 use Compress::Raw::Zlib qw(WANT_GZIP Z_OK Z_BUF_ERROR Z_STREAM_END);
 use File::Basename qw(dirname);
+use JSON::PP;
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
-use pgBackRest::Config::Config;
 use pgBackRest::Protocol::IO;
-use pgBackRest::Version;
 
 ####################################################################################################################################
-# Operation constants
+# DB/BACKUP Constants
 ####################################################################################################################################
-use constant OP_PROTOCOL_COMMON                                     => 'Protocol::Common';
-
-use constant OP_PROTOCOL_COMMON_NEW                                 => OP_PROTOCOL_COMMON . "->new";
+use constant DB                                                     => 'db';
+    push @EXPORT, qw(DB);
+use constant BACKUP                                                 => 'backup';
+    push @EXPORT, qw(BACKUP);
+use constant NONE                                                   => 'none';
+    push @EXPORT, qw(NONE);
 
 ####################################################################################################################################
 # Operation constants
@@ -35,6 +36,58 @@ use constant OP_NOOP                                                => 'noop';
 use constant OP_EXIT                                                => 'exit';
     push @EXPORT, qw(OP_EXIT);
 
+# Backup module
+use constant OP_BACKUP_FILE                                          => 'backupFile';
+    push @EXPORT, qw(OP_BACKUP_FILE);
+
+# Archive Module
+use constant OP_ARCHIVE_GET_ARCHIVE_ID                              => 'archiveId';
+    push @EXPORT, qw(OP_ARCHIVE_GET_ARCHIVE_ID);
+use constant OP_ARCHIVE_GET_BACKUP_INFO_CHECK                       => 'backupInfoCheck';
+    push @EXPORT, qw(OP_ARCHIVE_GET_BACKUP_INFO_CHECK);
+use constant OP_ARCHIVE_GET_CHECK                                   => 'archiveCheck';
+    push @EXPORT, qw(OP_ARCHIVE_GET_CHECK);
+use constant OP_ARCHIVE_PUSH_CHECK                                  => 'archivePushCheck';
+    push @EXPORT, qw(OP_ARCHIVE_PUSH_CHECK);
+
+# Db Module
+use constant OP_DB_CONNECT                                          => 'dbConnect';
+    push @EXPORT, qw(OP_DB_CONNECT);
+use constant OP_DB_EXECUTE_SQL                                      => 'dbExecSql';
+    push @EXPORT, qw(OP_DB_EXECUTE_SQL);
+use constant OP_DB_INFO                                             => 'dbInfo';
+    push @EXPORT, qw(OP_DB_INFO);
+
+# File Module
+use constant OP_FILE_COPY                                           => 'fileCopy';
+    push @EXPORT, qw(OP_FILE_COPY);
+use constant OP_FILE_COPY_IN                                        => 'fileCopyIn';
+    push @EXPORT, qw(OP_FILE_COPY_IN);
+use constant OP_FILE_COPY_OUT                                       => 'fileCopyOut';
+    push @EXPORT, qw(OP_FILE_COPY_OUT);
+use constant OP_FILE_EXISTS                                         => 'fileExists';
+    push @EXPORT, qw(OP_FILE_EXISTS);
+use constant OP_FILE_LIST                                           => 'fileList';
+    push @EXPORT, qw(OP_FILE_LIST);
+use constant OP_FILE_MANIFEST                                       => 'fileManifest';
+    push @EXPORT, qw(OP_FILE_MANIFEST);
+use constant OP_FILE_PATH_CREATE                                    => 'pathCreate';
+    push @EXPORT, qw(OP_FILE_PATH_CREATE);
+use constant OP_FILE_WAIT                                           => 'wait';
+    push @EXPORT, qw(OP_FILE_WAIT);
+
+# Info module
+use constant OP_INFO_STANZA_LIST                                    => 'infoStanzList';
+    push @EXPORT, qw(OP_INFO_STANZA_LIST);
+
+# Restore module
+use constant OP_RESTORE_FILE                                         => 'restoreFile';
+    push @EXPORT, qw(OP_RESTORE_FILE);
+
+# To be run after each command
+use constant OP_POST                                                 => 'post';
+    push @EXPORT, qw(OP_POST);
+
 ####################################################################################################################################
 # CONSTRUCTOR
 ####################################################################################################################################
@@ -57,7 +110,7 @@ sub new
     ) =
         logDebugParam
         (
-            OP_PROTOCOL_COMMON_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'iBufferMax', trace => true},
             {name => 'iCompressLevel', trace => true},
             {name => 'iCompressNetworkLevel', trace => true},
@@ -65,11 +118,11 @@ sub new
             {name => 'strName', required => false, trace => true}
         );
 
-    # Create the greeting that will be used to check versions with the remote
-    if (defined($self->{strName}))
-    {
-        $self->{strGreeting} = uc(BACKREST_NAME) . uc($self->{strName}) . ' ' . BACKREST_VERSION;
-    }
+    # By default remote type is NONE
+    $self->{strRemoteType} = NONE;
+
+    # Create JSON object
+    $self->{oJSON} = JSON::PP->new()->allow_nonref();
 
     # Return from function and log return values if any
     return logDebugReturn
@@ -110,7 +163,7 @@ sub blockRead
     my $bProtocol = shift;
 
     my $iBlockSize;
-    my $strMessage;
+    my $hMessage;
 
     if ($bProtocol)
     {
@@ -125,7 +178,11 @@ sub blockRead
         # Get block size from the header
         my @stryToken = split(/ /, $strBlockHeader);
         $iBlockSize = $stryToken[1];
-        $strMessage = $stryToken[2];
+
+        if (defined($stryToken[2]))
+        {
+            $hMessage = $self->{oJSON}->decode($stryToken[2]);
+        }
 
         # If block size is 0 or an error code then undef the buffer
         if ($iBlockSize <= 0)
@@ -144,7 +201,7 @@ sub blockRead
     }
 
     # Return the block size
-    return $iBlockSize, $strMessage;
+    return $iBlockSize, $hMessage;
 }
 
 ####################################################################################################################################
@@ -159,7 +216,7 @@ sub blockWrite
     my $tBlockRef = shift;
     my $iBlockSize = shift;
     my $bProtocol = shift;
-    my $strMessage = shift;
+    my $hMessage = shift;
 
     # If block size is not defined, get it from buffer length
     $iBlockSize = defined($iBlockSize) ? $iBlockSize : length($$tBlockRef);
@@ -167,7 +224,7 @@ sub blockWrite
     # Write block header to the protocol stream
     if ($bProtocol)
     {
-        $oOut->lineWrite("block ${iBlockSize}" . (defined($strMessage) ? " ${strMessage}" : ''));
+        $oOut->lineWrite("block ${iBlockSize}" . (defined($hMessage) ? " " . $self->{oJSON}->encode($hMessage) : ''));
     }
 
     # Write block if size > 0
@@ -192,6 +249,7 @@ sub binaryXfer
     my $bSourceCompressed = shift;
     my $bDestinationCompress = shift;
     my $bProtocol = shift;
+    my $fnExtra = shift;
 
     # The input stream must be defined
     my $oIn;
@@ -235,11 +293,12 @@ sub binaryXfer
 
     # Default protocol to true
     $bProtocol = defined($bProtocol) ? $bProtocol : true;
-    my $strMessage = undef;
+    my $hMessage = undef;
 
-    # Checksum and size
+    # Checksum, size, and extra
     my $strChecksum = undef;
     my $iFileSize = undef;
+    my $rExtra = undef;
 
     # Read from the protocol stream
     if ($strRemote eq 'in')
@@ -274,13 +333,13 @@ sub binaryXfer
             do
             {
                 # Read a block from the input stream
-                ($iBlockSize, $strMessage) = $self->blockRead($oIn, \$tCompressedBuffer, $bProtocol);
+                ($iBlockSize, $hMessage) = $self->blockRead($oIn, \$tCompressedBuffer, $bProtocol);
 
                 # Process protocol messages
-                if (defined($strMessage) && $strMessage eq 'nochecksum')
+                if (defined($hMessage) && defined($hMessage->{bChecksum}) && !$hMessage->{bChecksum})
                 {
                     $oSHA = Digest::SHA->new('sha1');
-                    undef($strMessage);
+                    undef($hMessage);
                 }
 
                 # If the block contains data, decompress it
@@ -348,23 +407,30 @@ sub binaryXfer
             {
                 $oSHA = Digest::SHA->new('sha1');
                 $iFileSize = 0;
+                $rExtra = defined($fnExtra) ? {} : undef;
             }
 
             do
             {
                 # Read a block from the protocol stream
-                ($iBlockSize, $strMessage) = $self->blockRead($oIn, \$tBuffer, $bProtocol);
+                ($iBlockSize, $hMessage) = $self->blockRead($oIn, \$tBuffer, $bProtocol);
 
-                # If the block contains data, write it
+                # Add data to checksum and size
+                if ($iBlockSize > 0 && !$bProtocol)
+                {
+                    $oSHA->add($tBuffer);
+                    $iFileSize += $iBlockSize;
+                }
+
+                # Do extra processing on the buffer if requested
+                if (!$bProtocol && defined($fnExtra))
+                {
+                    $fnExtra->(\$tBuffer, $iBlockSize, $iFileSize - $iBlockSize, $rExtra);
+                }
+
+                # Write buffer
                 if ($iBlockSize > 0)
                 {
-                    # Add data to checksum and size
-                    if (!$bProtocol)
-                    {
-                        $oSHA->add($tBuffer);
-                        $iFileSize += $iBlockSize;
-                    }
-
                     $oOut->bufferWrite(\$tBuffer, $iBlockSize);
                     undef($tBuffer);
                 }
@@ -392,11 +458,12 @@ sub binaryXfer
             # Initialize message to indicate that a checksum will be sent
             if ($bProtocol && defined($oOut))
             {
-                $strMessage = 'checksum';
+                $hMessage->{bChecksum} = true;
             }
 
             # Initialize checksum
             my $oSHA = Digest::SHA->new('sha1');
+            $rExtra = defined($fnExtra) ? {} : undef;
 
             # Initialize inflate object and check for errors
             my ($oZLib, $iZLibStatus) =
@@ -415,12 +482,22 @@ sub binaryXfer
                 # Read a block from the stream
                 $iBlockSize = $oIn->bufferRead(\$tUncompressedBuffer, $self->{iBufferMax});
 
-                # If block size > 0 then compress
+                # If block size > 0 then update checksum and size
                 if ($iBlockSize > 0)
                 {
                     # Update checksum and filesize
                     $oSHA->add($tUncompressedBuffer);
+                }
 
+                # Do extra processing on the buffer if requested
+                if (defined($fnExtra))
+                {
+                    $fnExtra->(\$tUncompressedBuffer, $iBlockSize, $oZLib->total_in(), $rExtra);
+                }
+
+                # If block size > 0 then compress
+                if ($iBlockSize > 0)
+                {
                     # Compress the data
                     $iZLibStatus = $oZLib->deflate($tUncompressedBuffer, $tCompressedBuffer);
                     $iCompressedBufferSize = length($tCompressedBuffer);
@@ -431,9 +508,9 @@ sub binaryXfer
                         # The compressed data is larger than block size, then write
                         if ($iCompressedBufferSize > $self->{iBufferMax})
                         {
-                            $self->blockWrite($oOut, \$tCompressedBuffer, $iCompressedBufferSize, $bProtocol, $strMessage);
+                            $self->blockWrite($oOut, \$tCompressedBuffer, $iCompressedBufferSize, $bProtocol, $hMessage);
                             undef($tCompressedBuffer);
-                            undef($strMessage);
+                            undef($hMessage);
                         }
                     }
                     # Else if error
@@ -468,11 +545,12 @@ sub binaryXfer
 
                 if ($iCompressedBufferSize > 0)
                 {
-                    $self->blockWrite($oOut, \$tCompressedBuffer, $iCompressedBufferSize, $bProtocol, $strMessage);
-                    undef($strMessage);
+                    $self->blockWrite($oOut, \$tCompressedBuffer, $iCompressedBufferSize, $bProtocol, $hMessage);
+                    undef($hMessage);
                 }
 
-                $self->blockWrite($oOut, undef, 0, $bProtocol, "${strChecksum}-${iFileSize}");
+                $self->blockWrite(
+                    $oOut, undef, 0, $bProtocol, {strChecksum => $strChecksum, iFileSize => $iFileSize, rExtra => $rExtra});
             }
         }
         # If source is already compressed or transfer is not compressed then just read the stream
@@ -492,7 +570,7 @@ sub binaryXfer
             {
                 if ($bProtocol)
                 {
-                    $strMessage = 'checksum';
+                    $hMessage->{bChecksum} = true;
                 }
 
                 # Initialize checksum and size
@@ -511,7 +589,7 @@ sub binaryXfer
             # Initialize message to indicate that a checksum will not be sent
             elsif ($bProtocol)
             {
-                $strMessage = 'nochecksum';
+                $hMessage->{bChecksum} = false;
             }
 
             # Read input
@@ -522,8 +600,8 @@ sub binaryXfer
                 # Write a block if size > 0
                 if ($iBlockSize > 0)
                 {
-                    $self->blockWrite($oOut, \$tBuffer, $iBlockSize, $bProtocol, $strMessage);
-                    undef($strMessage);
+                    $self->blockWrite($oOut, \$tBuffer, $iBlockSize, $bProtocol, $hMessage);
+                    undef($hMessage);
                 }
 
                 # Decompress the buffer to calculate checksum/size
@@ -570,7 +648,7 @@ sub binaryXfer
             }
             while ($iBlockSize > 0);
 
-            # Check decompression get checksum
+            # Check decompression, get checksum
             if ($bDestinationCompress)
             {
                 # Make sure the decompression succeeded (iBlockSize < 0 indicates remote error, handled later)
@@ -579,13 +657,15 @@ sub binaryXfer
                     confess &log(ERROR, "unable to inflate stream: ${iZLibStatus}");
                 }
 
-                # Get checksum
-                $strChecksum = $oSHA->hexdigest();
-
-                # Set protocol message
+                # If protocol then create the message
                 if ($bProtocol)
                 {
-                    $strMessage = "${strChecksum}-${iFileSize}";
+                    $hMessage = {strChecksum => $oSHA->hexdigest(), iFileSize => $iFileSize, rExtra => $rExtra};
+                }
+                # Otherwise just set checksum
+                else
+                {
+                    $strChecksum = $oSHA->hexdigest();
                 }
             }
 
@@ -593,21 +673,52 @@ sub binaryXfer
             if ($bProtocol)
             {
                 # Write 0 block to indicate end of stream
-                $self->blockWrite($oOut, undef, 0, $bProtocol, $strMessage);
+                $self->blockWrite($oOut, undef, 0, $bProtocol, $hMessage);
             }
         }
     }
 
-    # If message is defined then the checksum and size should be in it
-    if (defined($strMessage))
+    # If message is defined then the checksum, size, and extra should be in it
+    if (defined($hMessage))
     {
-        my @stryToken = split(/-/, $strMessage);
-        $strChecksum = $stryToken[0];
-        $iFileSize = $stryToken[1];
+        return $hMessage->{strChecksum}, $hMessage->{iFileSize}, $hMessage->{rExtra};
     }
 
     # Return the checksum and size if they are available
-    return $strChecksum, $iFileSize;
+    return $strChecksum, $iFileSize, $rExtra;
+}
+
+####################################################################################################################################
+# remoteType
+#
+# Return the remote type.
+####################################################################################################################################
+sub remoteType
+{
+    return shift->{strRemoteType};
+}
+
+####################################################################################################################################
+# remoteTypeTest
+#
+# Determine if the remote matches the specified remote.
+####################################################################################################################################
+sub remoteTypeTest
+{
+    my $self = shift;
+    my $strRemoteType = shift;
+
+    return $self->{strRemoteType} eq $strRemoteType ? true : false;
+}
+
+####################################################################################################################################
+# isRemote
+#
+# Determine if the protocol object is communicating with a remote.
+####################################################################################################################################
+sub isRemote
+{
+    return shift->{strRemoteType} ne NONE ? true : false;
 }
 
 1;
