@@ -7,27 +7,17 @@ use parent 'pgBackRest::Protocol::Common';
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use File::Basename qw(dirname);
 use Time::HiRes qw(gettimeofday);
-use Scalar::Util qw(blessed);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
-use pgBackRest::Config::Config;
 use pgBackRest::Protocol::Common;
 use pgBackRest::Protocol::IO;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_PROTOCOL_COMMON_MASTER                              => 'Protocol::CommonMaster';
-
-use constant OP_PROTOCOL_COMMON_MASTER_NEW                          => OP_PROTOCOL_COMMON_MASTER . "->new";
-use constant OP_PROTOCOL_COMMON_MASTER_COMMAND_WRITE                => OP_PROTOCOL_COMMON_MASTER . "->commandWrite";
-use constant OP_PROTOCOL_COMMON_MASTER_OUTPUT_READ                  => OP_PROTOCOL_COMMON_MASTER . "->outputRead";
+use pgBackRest::Version;
 
 ####################################################################################################################################
 # CONSTRUCTOR
@@ -40,30 +30,35 @@ sub new
     my
     (
         $strOperation,
+        $strRemoteType,                             # Type of remote (DB or BACKUP)
         $strName,                                   # Name of the protocol
         $strId,                                     # Id of this process for error messages
         $strCommand,                                # Command to execute on local/remote
         $iBufferMax,                                # Maximum buffer size
         $iCompressLevel,                            # Set compression level
         $iCompressLevelNetwork,                     # Set compression level for network only compression
-        $iProtocolTimeout                           # Protocol timeout
+        $iProtocolTimeout,                          # Protocol timeout
     ) =
         logDebugParam
         (
-            OP_PROTOCOL_COMMON_MASTER_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
+            {name => 'strRemoteType'},
             {name => 'strName'},
             {name => 'strId'},
             {name => 'strCommand'},
             {name => 'iBufferMax'},
             {name => 'iCompressLevel'},
             {name => 'iCompressLevelNetwork'},
-            {name => 'iProtocolTimeout'}
+            {name => 'iProtocolTimeout'},
         );
 
     # Create the class hash
     my $self = $class->SUPER::new($iBufferMax, $iCompressLevel, $iCompressLevelNetwork, $iProtocolTimeout, $strName);
     bless $self, $class;
 
+    # Set remote to specificied value
+    $self->{strRemoteType} = $strRemoteType;
+
     # Set command
     if (!defined($strCommand))
     {
@@ -101,8 +96,21 @@ sub close
 {
     my $self = shift;
 
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $bComplete,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->close', \@_,
+            {name => 'bComplete', default => false, trace => true},
+        );
+
     # Exit status defaults to success
     my $iExitStatus = 0;
+    my $bClosed = false;
 
     # Only send the exit command if the process is running
     if (defined($self->{io}) && defined($self->{io}->pIdGet()))
@@ -112,37 +120,44 @@ sub close
         eval
         {
             $self->cmdWrite('exit');
-        };
-
-        if ($@)
+            return true;
+        }
+        or do
         {
-            my $oMessage = $@;
+            my $oException = $EVAL_ERROR;
             my $strError = 'unable to shutdown protocol';
-            my $strHint = 'HINT: the process completed successfully but protocol-timeout may need to be increased.';
+            my $strHint = 'HINT: the process completed all operations successfully but protocol-timeout may need to be increased.';
 
-            if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
+            if (isException($oException))
             {
-                $iExitStatus = $oMessage->code();
+                $iExitStatus = $oException->code();
             }
             else
             {
-                if (!defined($oMessage))
+                if (!defined($oException))
                 {
-                    $oMessage = 'unknown error';
+                    $oException = 'unknown error';
                 }
 
                 $iExitStatus = ERROR_UNKNOWN;
             }
 
             &log(WARN,
-                $strError . ($iExitStatus == ERROR_UNKNOWN ? '' : ' [' . $oMessage->code() . ']') . ': ' .
-                ($iExitStatus == ERROR_UNKNOWN ? $oMessage : $oMessage->message()) . "\n${strHint}");
-        }
+                $strError . ($iExitStatus == ERROR_UNKNOWN ? '' : ' [' . $oException->code() . ']') . ': ' .
+                ($iExitStatus == ERROR_UNKNOWN ? $oException : $oException->message()) .
+                ($bComplete ? "\n${strHint}" : ''));
+        };
 
         undef($self->{io});
+        $bClosed = true;
     }
 
-    return $iExitStatus;
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'iExitStatus', value => $iExitStatus, trace => !$bClosed}
+    );
 }
 
 ####################################################################################################################################
@@ -165,16 +180,24 @@ sub greetingRead
     my $self = shift;
 
     # Get the first line of output from the remote if possible
-    my $strLine = $self->{io}->lineRead(undef, undef, undef, true);
+    my $hGreeting = $self->{oJSON}->decode($self->{io}->lineRead(undef, undef, undef, true));
 
-    # If the line could not be read or does equal the greeting then error and exit
-    if (!defined($strLine) || $strLine ne $self->{strGreeting})
+    # Error if greeting parameters do not match
+    for my $hParam ({strName => 'name', strExpected => BACKREST_NAME},
+                    {strName => 'version', strExpected => BACKREST_VERSION},
+                    {strName => 'service', strExpected => $self->{strName}})
     {
-        $self->{io}->kill();
+        if (!defined($hGreeting->{$hParam->{strName}}) || $hGreeting->{$hParam->{strName}} ne $hParam->{strExpected})
+        {
+            $self->{io}->kill();
 
-        confess &log(ERROR, 'protocol version mismatch' . (defined($strLine) ? ": ${strLine}" : ''), ERROR_HOST_CONNECT);
+            confess &log(ERROR,
+                'found name \'' . (defined($hGreeting->{$hParam->{strName}}) ? $hGreeting->{$hParam->{strName}} : '[undef]') .
+                "' in protocol greeting instead of expected '$hParam->{strName}'", ERROR_HOST_CONNECT);
+        }
     }
 
+    # Exchange one protocol message to catch errors early
     $self->outputRead();
 }
 
@@ -192,91 +215,62 @@ sub outputRead
     (
         $strOperation,
         $bOutputRequired,
-        $bSuppressLog
+        $bSuppressLog,
+        $bWarnOnError,
+        $bRef,
     ) =
         logDebugParam
         (
-            OP_PROTOCOL_COMMON_MASTER_OUTPUT_READ, \@_,
+            __PACKAGE__ . '->outputRead', \@_,
             {name => 'bOutputRequired', default => false, trace => true},
-            {name => 'bSuppressLog', required => false, trace => true}
+            {name => 'bSuppressLog', required => false, trace => true},
+            {name => 'bWarnOnError', default => false, trace => true},
+            {name => 'bRef', default => false, trace => true},
         );
 
-    my $strLine;
-    my $strOutput;
-    my $bError = false;
-    my $iErrorCode;
-    my $strError;
+    my $strProtocolResult = $self->{io}->lineRead();
 
-    # Read output lines
-    while ($strLine = $self->{io}->lineRead())
-    {
-        # Exit if an error is found
-        if ($strLine =~ /^ERROR.*/)
-        {
-            $bError = true;
-            $iErrorCode = (split(' ', $strLine))[1];
-            last;
-        }
+    logDebugMisc
+    (
+        $strOperation, undef,
+        {name => 'strProtocolResult', value => $strProtocolResult, trace => true}
+    );
 
-        # Exit if OK is found
-        if ($strLine =~ /^OK$/)
-        {
-            last;
-        }
-
-        # Else append to output
-        $strOutput .= (defined($strOutput) ? "\n" : '') . substr($strLine, 1);
-    }
+    my $hResult = $self->{oJSON}->decode($strProtocolResult);
 
     # Raise any errors
-    if ($bError)
+    if (defined($hResult->{err}))
     {
-        confess &log(ERROR, (defined($self->{strErrorPrefix}) ? "$self->{strErrorPrefix}: " : '') .
-                            (defined($strOutput) ? "${strOutput}" : ''), $iErrorCode, $bSuppressLog);
+        my $strError = $self->{strErrorPrefix} . (defined($hResult->{out}) ? ": $hResult->{out}" : '');
+
+        # Raise the error if a warning is not requested
+        if (!$bWarnOnError)
+        {
+            confess &log(ERROR, $strError, $hResult->{err}, $bSuppressLog);
+        }
+
+        &log(WARN, $strError, $hResult->{err});
+        undef($hResult->{out});
     }
 
     # Reset the keep alive time
     $self->{fKeepAliveTime} = gettimeofday();
 
     # If output is required and there is no output, raise exception
-    if (defined($bOutputRequired) && $bOutputRequired && !defined($strOutput))
+    if ($bOutputRequired && !defined($hResult->{out}))
     {
         $self->{io}->waitPid();
-        confess &log(ERROR, (defined($self->{strErrorPrefix}) ? "$self->{strErrorPrefix}: " : '') . 'output is not defined');
+        confess &log(ERROR, "$self->{strErrorPrefix}: output is not defined", ERROR_PROTOCOL_OUTPUT_REQUIRED);
     }
 
     # Return from function and log return values if any
     return logDebugReturn
     (
         $strOperation,
-        {name => 'strOutput', value => $strOutput, trace => true}
+        {name => 'hOutput', value => $hResult->{out}, ref => $bRef, trace => true}
     );
 }
 
-####################################################################################################################################
-# commandParamString
-#
-# Output command parameters in the hash as a string (used for debugging).
-####################################################################################################################################
-sub commandParamString
-{
-    my $self = shift;
-    my $oParamHashRef = shift;
-
-    my $strParamList = '';
-
-    if (defined($oParamHashRef))
-    {
-        foreach my $strParam (sort(keys(%$oParamHashRef)))
-        {
-            $strParamList .= (defined($strParamList) ? ',' : '') . "${strParam}=" .
-                             (defined(${$oParamHashRef}{"${strParam}"}) ? ${$oParamHashRef}{"${strParam}"} : '[undef]');
-        }
-    }
-
-    return $strParamList;
-}
-
 ####################################################################################################################################
 # cmdWrite
 #
@@ -291,50 +285,28 @@ sub cmdWrite
     (
         $strOperation,
         $strCommand,
-        $oParamRef,
+        $hParam,
     ) =
         logDebugParam
         (
-            OP_PROTOCOL_COMMON_MASTER_COMMAND_WRITE, \@_,
+            __PACKAGE__ . '->cmdWrite', \@_,
             {name => 'strCommand', trace => true},
-            {name => 'oParamRef', required => false, trace => true}
+            {name => 'hParam', required => false, trace => true},
         );
 
-    if (defined($oParamRef))
-    {
-        $strCommand .= ":\n";
-
-        foreach my $strParam (sort(keys(%$oParamRef)))
-        {
-            if ($strParam =~ /=/)
-            {
-                confess &log(ASSERT, "param \"${strParam}\" cannot contain = character");
-            }
-
-            my $strValue = ${$oParamRef}{"${strParam}"};
-
-            if ($strParam =~ /\n\$/)
-            {
-                confess &log(ASSERT, "param \"${strParam}\" value cannot end with LF");
-            }
-
-            if (defined(${strValue}))
-            {
-                $strCommand .= "${strParam}=${strValue}\n";
-            }
-        }
-
-        $strCommand .= 'end';
-    }
+    my $strProtocolCommand = $self->{oJSON}->encode({cmd => $strCommand, param => $hParam});
 
     logDebugMisc
     (
         $strOperation, undef,
-        {name => 'strCommand', value => $strCommand, trace => true}
+        {name => 'strProtocolCommand', value => $strProtocolCommand, trace => true}
     );
 
     # Write out the command
-    $self->{io}->lineWrite($strCommand);
+    $self->{io}->lineWrite($strProtocolCommand);
+
+    # Reset the keep alive time
+    $self->{fKeepAliveTime} = gettimeofday();
 
     # Return from function and log return values if any
     logDebugReturn($strOperation);
@@ -351,10 +323,11 @@ sub cmdExecute
     my $strCommand = shift;
     my $oParamRef = shift;
     my $bOutputRequired = shift;
+    my $bWarnOnError = shift;
 
     $self->cmdWrite($strCommand, $oParamRef);
 
-    return $self->outputRead($bOutputRequired);
+    return $self->outputRead($bOutputRequired, undef, $bWarnOnError);
 }
 
 ####################################################################################################################################
@@ -369,7 +342,6 @@ sub keepAlive
     if (gettimeofday() - $self->{fKeepAliveTimeout} > $self->{fKeepAliveTime})
     {
         $self->noOp();
-        $self->{fKeepAliveTime} = gettimeofday();
 
         # Keep alive test point
         &log(TEST, TEST_KEEP_ALIVE);
@@ -386,6 +358,7 @@ sub noOp
     my $self = shift;
 
     $self->cmdExecute(OP_NOOP, undef, false);
+    $self->{fKeepAliveTime} = gettimeofday();
 }
 
 1;
