@@ -7,25 +7,17 @@ use parent 'pgBackRest::Protocol::Common';
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
-use File::Basename qw(dirname);
-use Scalar::Util qw(blessed);
+use JSON::PP;
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
-use pgBackRest::Config::Config;
 use pgBackRest::Protocol::Common;
 use pgBackRest::Protocol::IO;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_PROTOCOL_COMMON_MINION                              => 'Protocol::CommonMinion';
-
-use constant OP_PROTOCOL_COMMON_MINION_NEW                          => OP_PROTOCOL_COMMON_MINION . "->new";
+use pgBackRest::Version;
 
 ####################################################################################################################################
 # CONSTRUCTOR
@@ -43,17 +35,17 @@ sub new
         $iBufferMax,                                # Maximum buffer size
         $iCompressLevel,                            # Set compression level
         $iCompressLevelNetwork,                     # Set compression level for network only compression
-        $iProtocolTimeout                           # Protocol timeout
+        $iProtocolTimeout,                          # Protocol timeout
     ) =
         logDebugParam
         (
-            OP_PROTOCOL_COMMON_MINION_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'strName'},
             {name => 'strCommand'},
             {name => 'iBufferMax'},
             {name => 'iCompressLevel'},
             {name => 'iCompressLevelNetwork'},
-            {name => 'iProtocolTimeout'}
+            {name => 'iProtocolTimeout'},
         );
 
     # Create the class hash
@@ -68,7 +60,15 @@ sub new
     # Write the greeting so master process knows who we are
     $self->greetingWrite();
 
-    return $self;
+    # Initialize module variables
+    $self->{hCommandMap} = $self->init();
+
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'self', value => $self}
+    );
 }
 
 ####################################################################################################################################
@@ -80,23 +80,12 @@ sub greetingWrite
 {
     my $self = shift;
 
-    $self->{io}->lineWrite($self->{strGreeting});
-    $self->{io}->lineWrite('OK');
-}
+    # Write the greeting
+    $self->{io}->lineWrite((JSON::PP->new()->canonical()->allow_nonref())->encode(
+        {name => BACKREST_NAME, service => $self->{strName}, version => BACKREST_VERSION}));
 
-####################################################################################################################################
-# textWrite
-#
-# Write text out to the protocol layer prefixing each line with a period.
-####################################################################################################################################
-sub textWrite
-{
-    my $self = shift;
-    my $strBuffer = shift;
-
-    $strBuffer =~ s/\n/\n\./g;
-
-    $self->{io}->lineWrite('.' . $strBuffer);
+    # Exchange one protocol message to catch errors early
+    $self->outputWrite();
 }
 
 ####################################################################################################################################
@@ -120,40 +109,16 @@ sub binaryXferAbort
 sub errorWrite
 {
     my $self = shift;
-    my $oMessage = shift;
+    my $oException = shift;
 
-    my $iCode;
-    my $strMessage;
-
-    # If the message is blessed it may be a standard exception
-    if (blessed($oMessage))
+    # Throw hard error if this is not a standard exception
+    if (!isException($oException))
     {
-        # Check if it is a standard exception
-        if ($oMessage->isa('pgBackRest::Common::Exception'))
-        {
-            $iCode = $oMessage->code();
-            $strMessage = $oMessage->message();
-        }
-        # Else terminate the process with an error
-        else
-        {
-            confess &log(ERROR, 'unknown error object', ERROR_UNKNOWN);
-        }
-    }
-    # Else terminate the process with an error
-    else
-    {
-        confess &log(ERROR, 'unknown error: ' . $oMessage, ERROR_UNKNOWN);
+        confess &log(ERROR, 'unknown error: ' . $oException, ERROR_UNKNOWN);
     }
 
-    # Write the message text into protocol
-    if (defined($strMessage))
-    {
-        $self->textWrite(trim($strMessage));
-    }
-
-    # Indicate that an error ocurred and provide the code
-    $self->{io}->lineWrite("ERROR" . (defined($iCode) ? " $iCode" : ''));
+    # Write error code and message
+    $self->{io}->lineWrite($self->{oJSON}->encode({err => $oException->code(), out => $oException->message()}));
 }
 
 ####################################################################################################################################
@@ -164,14 +129,8 @@ sub errorWrite
 sub outputWrite
 {
     my $self = shift;
-    my $strOutput = shift;
 
-    if (defined($strOutput))
-    {
-        $self->textWrite($strOutput);
-    }
-
-    $self->{io}->lineWrite('OK');
+    $self->{io}->lineWrite($self->{oJSON}->encode({out => \@_}));
 }
 
 ####################################################################################################################################
@@ -182,47 +141,83 @@ sub outputWrite
 sub cmdRead
 {
     my $self = shift;
-    my $oParamHashRef = shift;
 
-    my $strLine;
-    my $strCommand;
+    my $hCommand = $self->{oJSON}->decode($self->{io}->lineRead());
 
-    while ($strLine = $self->{io}->lineRead())
+    return $hCommand->{cmd}, $hCommand->{param};
+}
+
+####################################################################################################################################
+# process
+####################################################################################################################################
+sub process
+{
+    my $self = shift;
+
+    # Reset stderr log level so random errors do not get output
+    logLevelSet(undef, undef, OFF);
+
+    # Loop until the exit command is received
+    eval
     {
-        if (!defined($strCommand))
+        while (true)
         {
-            if ($strLine =~ /:$/)
+            my ($strCommand, $rParam) = $self->cmdRead();
+
+            last if ($strCommand eq OP_EXIT);
+
+            eval
             {
-                $strCommand = substr($strLine, 0, length($strLine) - 1);
+                # Check for the command in the map and run it if found
+                if (defined($self->{hCommandMap}{$strCommand}))
+                {
+                    $self->outputWrite($self->{hCommandMap}{$strCommand}->($rParam));
+                }
+                # Run the standard NOOP command.  This this can be overridden in hCommandMap to implement a custom NOOP.
+                elsif ($strCommand eq OP_NOOP)
+                {
+                    $self->outputWrite();
+                }
+                else
+                {
+                    confess "invalid command: ${strCommand}";
+                }
+
+                # Run the post command if defined
+                if (defined($self->{hCommandMap}{&OP_POST}))
+                {
+                    $self->{hCommandMap}{&OP_POST}->($strCommand, $rParam);
+                }
+
+                return true;
             }
-            else
+            # Process errors
+            or do
             {
-                $strCommand = $strLine;
-                last;
-            }
+                $self->errorWrite($EVAL_ERROR);
+            };
         }
-        else
-        {
-            if ($strLine eq 'end')
-            {
-                last;
-            }
 
-            my $iPos = index($strLine, '=');
-
-            if ($iPos == -1)
-            {
-                confess "param \"${strLine}\" is missing = character";
-            }
-
-            my $strParam = substr($strLine, 0, $iPos);
-            my $strValue = substr($strLine, $iPos + 1);
-
-            ${$oParamHashRef}{"${strParam}"} = ${strValue};
-        }
+        return true;
     }
+    or do
+    {
+        my $oException = $EVAL_ERROR;
 
-    return $strCommand;
+        # Change log level so error will go to stderr
+        logLevelSet(undef, undef, PROTOCOL);
+
+        # If standard exception
+        if (isException($oException))
+        {
+            confess &log($oException->level(), $oException->message(), $oException->code());
+        }
+
+        # Else unexpected Perl exception
+        confess &log(ERROR, 'unknown error: ' . $oException, ERROR_UNKNOWN);
+    };
+
+    return 0;
 }
 
 1;
