@@ -6,6 +6,7 @@ package pgBackRest::Protocol::IO;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use Exporter qw(import);
     our @EXPORT = qw();
@@ -16,20 +17,11 @@ use POSIX qw(:sys_wait_h);
 use Symbol 'gensym';
 use Time::HiRes qw(gettimeofday);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Common::Wait;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_IO_PROTOCOL                                                  => 'IO';
-
-use constant OP_IO_PROTOCOL_NEW                                              => OP_IO_PROTOCOL . "->new";
-use constant OP_IO_PROTOCOL_NEW3                                             => OP_IO_PROTOCOL . "->new3";
-
 ####################################################################################################################################
 # Amount of time to attempt to retrieve errors when a process terminates unexpectedly
 ####################################################################################################################################
@@ -60,7 +52,7 @@ sub new
     ) =
         logDebugParam
         (
-            OP_IO_PROTOCOL_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'hIn', required => false, trace => true},
             {name => 'hOut', required => false, trace => true},
             {name => 'hErr', required => false, trace => true},
@@ -104,7 +96,7 @@ sub new3
     ) =
         logDebugParam
         (
-            OP_IO_PROTOCOL_NEW3, \@_,
+            __PACKAGE__ . '->new3', \@_,
             {name => 'strId', trace => true},
             {name => 'strCommand', trace => true},
             {name => 'iProtocolTimeout', trace => true},
@@ -302,7 +294,7 @@ sub lineRead
         {
             if (!defined($bError) || $bError)
             {
-                confess &log(ERROR, "unable to read line after $self->{iProtocolTimeout} seconds", ERROR_PROTOCOL_TIMEOUT);
+                confess &log(ERROR, "unable to read line after ${fTimeout} second(s)", ERROR_PROTOCOL_TIMEOUT);
             }
 
             return;
@@ -378,7 +370,6 @@ sub bufferRead
     # Set working variables
     my $iRemainingSize = $iRequestSize;
     $iOffset = defined($iOffset) ? $iOffset : 0;
-    $bBlock = defined($bBlock) ? $bBlock : false;
 
     # If there is data left over in the buffer from lineRead then use it
     if (defined($self->{strBuffer}) && $self->{iBufferPos} < $self->{iBufferSize})
@@ -398,80 +389,72 @@ sub bufferRead
             $iRemainingSize -= $iReadSize;
             undef($self->{strBuffer});
 
-            return $iReadSize if !$bBlock || $iRemainingSize == 0;
+            return $iReadSize if $iRemainingSize == 0;
 
             $iOffset += $iReadSize;
         }
     }
 
-    # If this is a blocking read then loop until all bytes have been read - else error.
+    # If this is a blocking read then loop until all bytes have been read, else error.  If not blocking read until the request size
+    # has been met or EOF.
     #
     # This link (http://docstore.mik.ua/orelly/perl/cookbook/ch07_15.htm) demonstrates a way to implement this same loop without
     # using select.  Not sure if it would be better but the link has been left here so it can be found in the future if the
     # implementation needs to be changed.
-    if ($bBlock)
+    my $fTimeStart = gettimeofday();
+    my $fRemaining = 1; #$self->{iProtocolTimeout};
+
+    do
     {
-        my $fTimeStart = gettimeofday();
-        my $fRemaining = $self->{iProtocolTimeout};
-
-        do
+        # Check if the sysread call will block
+        if ($self->{oInSelect}->can_read($fRemaining))
         {
-            # Check if the sysread call will block
-            if ($self->{oInSelect}->can_read($fRemaining))
+            # Read a data into the buffer
+            my $iReadSize = sysread($self->{hIn}, $$tBufferRef, $iRemainingSize, $iOffset);
+
+            # Process errors from the sysread
+            if (!defined($iReadSize))
             {
-                # Read a data into the buffer
-                my $iReadSize = sysread($self->{hIn}, $$tBufferRef, $iRemainingSize, $iOffset);
+                my $strError = $!;
 
-                # Process errors from the sysread
-                if (!defined($iReadSize))
-                {
-                    my $strError = $!;
-
-                    $self->waitPid();
-                    confess &log(ERROR, 'unable to read' . (defined($strError) ? ": ${strError}" : ''));
-                }
-
-                # Check for EOF
-                if ($iReadSize == 0)
-                {
-                    confess &log(ERROR, 'unexpected EOF', ERROR_FILE_READ);
-                }
-
-                # Update remaining size and return when it reaches 0
-                $iRemainingSize -= $iReadSize;
-
-                if ($iRemainingSize == 0)
-                {
-                    return $iRequestSize;
-                }
-
-                # Update the offset to advance the next read further into the buffer
-                $iOffset += $iReadSize;
+                $self->waitPid();
+                confess &log(ERROR, 'unable to read' . (defined($strError) ? ": ${strError}" : ''));
             }
 
-            # Calculate time remaining before timeout
-            $fRemaining = $self->{iProtocolTimeout} - (gettimeofday() - $fTimeStart);
+            # Check for EOF
+            if ($iReadSize == 0)
+            {
+                if (defined($bBlock) && $bBlock)
+                {
+                    confess &log(ERROR,
+                        'EOF after ' . ($iRequestSize - $iRemainingSize) . " bytes but expected ${iRequestSize}",
+                        ERROR_FILE_READ);
+                }
+                else
+                {
+                    return $iRequestSize - $iRemainingSize;
+                }
+            }
+
+            # Update remaining size and return when it reaches 0
+            $iRemainingSize -= $iReadSize;
+
+            if ($iRemainingSize == 0)
+            {
+                return $iRequestSize;
+            }
+
+            # Update the offset to advance the next read further into the buffer
+            $iOffset += $iReadSize;
         }
-        while ($fRemaining > 0);
 
-        # Throw an error if timeout happened before required bytes were read
-        confess &log(ERROR, "unable to read ${iRequestSize} bytes after $self->{iProtocolTimeout} seconds", ERROR_PROTOCOL_TIMEOUT);
+        # Calculate time remaining before timeout
+        $fRemaining = $self->{iProtocolTimeout} - (gettimeofday() - $fTimeStart);
     }
+    while ($fRemaining > 0);
 
-    # Otherwise do a non-blocking read and return whatever bytes are ready
-    my $iReadSize = sysread($self->{hIn}, $$tBufferRef, $iRemainingSize, $iOffset);
-
-    # Process errors from sysread
-    if (!defined($iReadSize))
-    {
-        my $strError = $!;
-
-        $self->waitPid();
-        confess &log(ERROR, 'unable to read' . (defined($strError) ? ": ${strError}" : ''));
-    }
-
-    # Return the number of bytes read
-    return $iReadSize;
+    # Throw an error if timeout happened before required bytes were read
+    confess &log(ERROR, "unable to read ${iRequestSize} bytes after $self->{iProtocolTimeout} seconds", ERROR_PROTOCOL_TIMEOUT);
 }
 
 ####################################################################################################################################
@@ -497,7 +480,7 @@ sub bufferWrite
         my $strError = $!;
 
         $self->waitPid();
-        confess "unable to write ${iWriteSize} bytes" . (defined($strError) ? ': ' . $strError : '');
+        confess &log(ERROR, "unable to write ${iWriteSize} bytes" . (defined($strError) ? ': ' . $strError : ''), ERROR_FILE_WRITE);
     }
 }
 
@@ -537,20 +520,18 @@ sub waitPid
                 if (!defined($bReportError) || $bReportError)
                 {
                     # Default error
-                    my $strError = 'no error on stderr';
+                    my $strError = undef;
 
                     # If the error stream is already closed then we can't fetch the real error
                     if (!defined($self->{hErr}))
                     {
-                            $strError = 'no error captured because stderr is already closed';
-                        }
+                        $strError = 'no error captured because stderr is already closed';
+                    }
                     # Get whatever text we can from the error stream
                     else
                     {
                         eval
                         {
-                            $strError = undef;
-
                             while (my $strLine = $self->lineRead(0, false, false))
                             {
                                 if (defined($strError))
@@ -560,16 +541,20 @@ sub waitPid
 
                                 $strError .= $strLine;
                             }
-                        };
 
-                        if ($@ || !defined($strError))
-                        {
-                            my $strMessage = $@;
-
-                            $strError =
-                                'no output from terminated process' .
-                                (defined($strMessage) && ${strMessage} ne '' ? ": ${strMessage}" : '');
+                            return true;
                         }
+                        or do
+                        {
+                            if (!defined($strError))
+                            {
+                                my $strException = $EVAL_ERROR;
+
+                                $strError =
+                                    'no output from terminated process' .
+                                    (defined($strException) && ${strException} ne '' ? ": ${strException}" : '');
+                            }
+                        };
                     }
 
                     $self->{pId} = undef;
@@ -581,7 +566,7 @@ sub waitPid
                     confess &log(
                         ERROR, 'remote process terminated on ' . $self->{strId} . ' host' .
                         ($iExitStatus < ERROR_MINIMUM || $iExitStatus > ERROR_MAXIMUM ? " (exit status ${iExitStatus})" : '') .
-                        ": ${strError}",
+                        ': ' . (defined($strError) ? $strError : 'no error on stderr'),
                         $iExitStatus >= ERROR_MINIMUM && $iExitStatus <= ERROR_MAXIMUM ? $iExitStatus : ERROR_HOST_CONNECT);
                 }
 
