@@ -10,18 +10,10 @@ use Carp qw(confess);
 
 use File::Basename qw(dirname);
 
-use lib dirname($0) . '/../lib';
 use pgBackRest::Common::Log;
 use pgBackRest::Config::Config;
 use pgBackRest::Protocol::CommonMaster;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_PROTOCOL_REMOTE_MASTER                              => 'Protocol::RemoteMaster';
-
-use constant OP_PROTOCOL_REMOTE_MASTER_NEW                          => OP_PROTOCOL_REMOTE_MASTER . "->new";
-
 ####################################################################################################################################
 # CONSTRUCTOR
 ####################################################################################################################################
@@ -33,6 +25,8 @@ sub new
     my
     (
         $strOperation,
+        $strRemoteType,                             # Type of remote (DB or BACKUP)
+        $strCommandSSH,                             # SSH client
         $strCommand,                                # Command to execute on local/remote
         $iBufferMax,                                # Maximum buffer size
         $iCompressLevel,                            # Set compression level
@@ -43,7 +37,9 @@ sub new
     ) =
         logDebugParam
         (
-            OP_PROTOCOL_REMOTE_MASTER_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
+            {name => 'strRemoteType'},
+            {name => 'strCommandSSH'},
             {name => 'strCommand'},
             {name => 'iBufferMax'},
             {name => 'iCompressLevel'},
@@ -54,13 +50,17 @@ sub new
         );
 
     # Create SSH command
-    $strCommand = "ssh -o LogLevel=error -o Compression=no -o PasswordAuthentication=no ${strUser}\@${strHost} '${strCommand}'";
+    $strCommand =
+        "${strCommandSSH} -o LogLevel=error -o Compression=no -o PasswordAuthentication=no ${strUser}\@${strHost} '${strCommand}'";
 
     # Init object and store variables
     my $self = $class->SUPER::new(
-        'remote', $strHost, $strCommand, $iBufferMax, $iCompressLevel, $iCompressLevelNetwork, $iProtocolTimeout);
+        $strRemoteType, 'remote', $strHost, $strCommand, $iBufferMax, $iCompressLevel, $iCompressLevelNetwork, $iProtocolTimeout);
     bless $self, $class;
 
+    # Store the host
+    $self->{strHost} = $strHost;
+
     # Return from function and log return values if any
     return logDebugReturn
     (
