@@ -3,9 +3,6 @@
 ####################################################################################################################################
 package pgBackRest::Restore;
 
-use threads;
-use threads::shared;
-use Thread::Queue;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
@@ -14,30 +11,19 @@ use Cwd qw(abs_path);
 use File::Basename qw(basename dirname);
 use File::stat qw(lstat);
 
-use lib dirname($0);
 use pgBackRest::BackupInfo;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Config::Config;
-use pgBackRest::Db;
+use pgBackRest::DbVersion;
 use pgBackRest::File;
 use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
 use pgBackRest::RestoreFile;
-
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_RESTORE                                             => 'Restore';
-
-use constant OP_RESTORE_BUILD                                       => OP_RESTORE . '->build';
-use constant OP_RESTORE_CLEAN                                       => OP_RESTORE . '->clean';
-use constant OP_RESTORE_DESTROY                                     => OP_RESTORE . '->destroy';
-use constant OP_RESTORE_MANIFEST_LOAD                               => OP_RESTORE . '->manifestLoad';
-use constant OP_RESTORE_MANIFEST_OWNERSHIP_CHECK                    => OP_RESTORE . '->manifestOwnershipCheck';
-use constant OP_RESTORE_NEW                                         => OP_RESTORE . '->new';
-use constant OP_RESTORE_PROCESS                                     => OP_RESTORE . '->process';
-use constant OP_RESTORE_RECOVERY                                    => OP_RESTORE . '->recovery';
+use pgBackRest::Protocol::Common;
+use pgBackRest::Protocol::LocalProcess;
+use pgBackRest::Protocol::Protocol;
+use pgBackRest::Version;
 
 ####################################################################################################################################
 # CONSTRUCTOR
@@ -51,17 +37,16 @@ sub new
     bless $self, $class;
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam(OP_RESTORE_NEW);
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->new');
 
     # Initialize protocol
-    $self->{oProtocol} = protocolGet();
+    $self->{oProtocol} = protocolGet(BACKUP);
 
     # Initialize default file object
     $self->{oFile} = new pgBackRest::File
     (
         optionGet(OPTION_STANZA),
         optionGet(OPTION_REPO_PATH),
-        optionRemoteType(),
         $self->{oProtocol}
     );
 
@@ -85,22 +70,12 @@ sub DESTROY
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    my
-    (
-        $strOperation
-    ) =
-        logDebugParam
-    (
-        OP_RESTORE_DESTROY
-    );
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->DESTROY');
 
     undef($self->{oFile});
 
     # Return from function and log return values if any
-    return logDebugReturn
-    (
-        $strOperation
-    );
+    return logDebugReturn($strOperation);
 }
 
 ####################################################################################################################################
@@ -121,7 +96,7 @@ sub manifestOwnershipCheck
     ) =
         logDebugParam
         (
-            OP_RESTORE_MANIFEST_OWNERSHIP_CHECK, \@_,
+            __PACKAGE__ . '->manifestOwnershipCheck', \@_,
             {name => 'oManifest'}
         );
 
@@ -247,7 +222,7 @@ sub manifestLoad
     my $self = shift;           # Class hash
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam (OP_RESTORE_MANIFEST_LOAD);
+    my ($strOperation) = logDebugParam (__PACKAGE__ . '->manifestLoad');
 
     # Error if the backup set does not exist
     if (!$self->{oFile}->exists(PATH_BACKUP_CLUSTER, $self->{strBackupSet}))
@@ -432,13 +407,22 @@ sub manifestLoad
                     # Set the file part
                     $oManifest->set(MANIFEST_SECTION_BACKUP_TARGET, $strTarget, MANIFEST_SUBKEY_FILE,
                                     substr(${$oLinkRemap}{$strTarget}, length($strTargetPath) + 1));
+
+                    # Set the link target
+                    $oManifest->set(
+                        MANIFEST_SECTION_TARGET_LINK, $strTarget, MANIFEST_SUBKEY_DESTINATION, ${$oLinkRemap}{$strTarget});
+                }
+                else
+                {
+                    # Set the link target
+                    $oManifest->set(MANIFEST_SECTION_TARGET_LINK, $strTarget, MANIFEST_SUBKEY_DESTINATION, $strTargetPath);
+
+                    # Since this will be a link remove the associated path (??? perhaps this should be in build like it is for ts?)
+                    $oManifest->remove(MANIFEST_SECTION_TARGET_PATH, $strTarget);
                 }
 
                 # Set the target path
                 $oManifest->set(MANIFEST_SECTION_BACKUP_TARGET, $strTarget, MANIFEST_SUBKEY_PATH, $strTargetPath);
-
-                # Since this will be a link remove the associated path (??? perhaps this should be in build like it is for ts?)
-                $oManifest->remove(MANIFEST_SECTION_TARGET_PATH, $strTarget);
             }
             # Else the link will be restored directly into $PGDATA instead
             else
@@ -490,7 +474,7 @@ sub clean
     ) =
         logDebugParam
         (
-            OP_RESTORE_CLEAN, \@_,
+            __PACKAGE__ . '->clean', \@_,
             {name => 'oManifest'}
         );
 
@@ -579,10 +563,9 @@ sub clean
             }
 
             # Build a manifest of the path to check for existing files
-            my %oTargetManifest;
-            $self->{oFile}->manifest(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget}, \%oTargetManifest);
+            my $hTargetManifest = $self->{oFile}->manifest(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget});
 
-            for my $strName (keys(%{$oTargetManifest{name}}))
+            for my $strName (keys(%{$hTargetManifest}))
             {
                 # Skip the root path and backup.manifest in the base path
                 if ($strName eq '.' ||
@@ -621,8 +604,7 @@ sub clean
             }
 
             # Load path manifest so it can be compared to deleted files/paths/links that are not in the backup
-            my %oTargetManifest;
-            $self->{oFile}->manifest(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget}, \%oTargetManifest);
+            my $hTargetManifest = $self->{oFile}->manifest(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget});
 
             # If the target is a file it doesn't matter whether it already exists or not.
             if ($oManifest->isTargetFile($strTarget))
@@ -630,7 +612,7 @@ sub clean
                 next;
             }
 
-            foreach my $strName (sort {$b cmp $a} (keys(%{$oTargetManifest{name}})))
+            foreach my $strName (sort {$b cmp $a} (keys(%{$hTargetManifest})))
             {
                 # Skip the root path
                 if ($strName eq '.' || ($strName eq FILE_MANIFEST && $strTarget eq MANIFEST_TARGET_PGDATA))
@@ -644,11 +626,11 @@ sub clean
                 # Determine the file/path/link type
                 my $strSection = MANIFEST_SECTION_TARGET_FILE;
 
-                if ($oTargetManifest{name}{$strName}{type} eq 'd')
+                if ($hTargetManifest->{$strName}{type} eq 'd')
                 {
                     $strSection = MANIFEST_SECTION_TARGET_PATH;
                 }
-                elsif ($oTargetManifest{name}{$strName}{type} eq 'l')
+                elsif ($hTargetManifest->{$strName}{type} eq 'l')
                 {
                     $strSection = MANIFEST_SECTION_TARGET_LINK;
                 }
@@ -661,10 +643,10 @@ sub clean
                     my $strGroup = $oManifest->get($strSection, $strManifestFile, MANIFEST_SUBKEY_GROUP);
 
                     # If ownership does not match, fix it
-                    if (!defined($oTargetManifest{name}{$strName}{user}) ||
-                        $strUser ne $oTargetManifest{name}{$strName}{user} ||
-                        !defined($oTargetManifest{name}{$strName}{group}) ||
-                        $strGroup ne $oTargetManifest{name}{$strName}{group})
+                    if (!defined($hTargetManifest->{$strName}{user}) ||
+                        $strUser ne $hTargetManifest->{$strName}{user} ||
+                        !defined($hTargetManifest->{$strName}{group}) ||
+                        $strGroup ne $hTargetManifest->{$strName}{group})
                     {
                         &log(DETAIL, "set ownership ${strUser}:${strGroup} on ${strOsFile}");
 
@@ -675,7 +657,7 @@ sub clean
                     if ($strSection eq MANIFEST_SECTION_TARGET_LINK)
                     {
                         if ($oManifest->get($strSection, $strManifestFile, MANIFEST_SUBKEY_DESTINATION) ne
-                            $oTargetManifest{name}{$strName}{link_destination})
+                            $hTargetManifest->{$strName}{link_destination})
                         {
                             &log(DETAIL, "remove link ${strOsFile} - destination changed");
                             fileRemove($strOsFile);
@@ -686,7 +668,7 @@ sub clean
                     {
                         my $strMode = $oManifest->get($strSection, $strManifestFile, MANIFEST_SUBKEY_MODE);
 
-                        if ($strMode ne $oTargetManifest{name}{$strName}{mode})
+                        if ($strMode ne $hTargetManifest->{$strName}{mode})
                         {
                             &log(DETAIL, "set mode ${strMode} on ${strOsFile}");
 
@@ -722,6 +704,9 @@ sub clean
                             &log(DETAIL, "remove ${strType} ${strOsFile}");
                             fileRemove($strOsFile);
 
+                            # Removing a file can be expensive so send protocol keep alive
+                            $self->{oProtocol}->keepAlive();
+
                             $oRemoveHash{$strSection} += 1;
                         }
                     }
@@ -789,7 +774,7 @@ sub build
     ) =
         logDebugParam
         (
-            OP_RESTORE_BUILD, \@_,
+            __PACKAGE__ . '->build', \@_,
             {name => 'oManifest'}
         );
 
@@ -914,7 +899,7 @@ sub recovery
     my $strDbVersion = shift;   # Version to restore
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam (OP_RESTORE_RECOVERY);
+    my ($strOperation) = logDebugParam (__PACKAGE__ . '->recovery');
 
     # Create recovery.conf path/file
     my $strRecoveryConf = $self->{strDbClusterPath} . '/' . DB_FILE_RECOVERYCONF;
@@ -1027,7 +1012,7 @@ sub recovery
             close($hFile)
                 or confess "unable to close ${strRecoveryConf}: $!";
 
-            &log(INFO, "wrote $strRecoveryConf");
+            &log(INFO, "write $strRecoveryConf");
         }
     }
 
@@ -1045,7 +1030,7 @@ sub process
     my $self = shift;       # Class hash
 
     # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam (OP_RESTORE_PROCESS);
+    my ($strOperation) = logDebugParam (__PACKAGE__ . '->process');
 
     if (!fileExists($self->{strDbClusterPath}))
     {
@@ -1078,7 +1063,7 @@ sub process
 
     my $oBackupInfo = new pgBackRest::BackupInfo($self->{strDbClusterPath}, false);
 
-    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_BACKUP_INFO, undef, false);
+    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_BACKUP_INFO, undef, false, true);
 
     # If set to restore is latest then get the actual set
     if ($self->{strBackupSet} eq OPTION_DEFAULT_RESTORE_SET)
@@ -1109,7 +1094,7 @@ sub process
     # from being started by PostgreSQL.
     $self->{oFile}->remove(PATH_DB_ABSOLUTE, $oManifest->dbPathGet(
                            $oManifest->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH),
-                           MANIFEST_FILE_PGCONTROL));
+                           MANIFEST_FILE_PGCONTROL), true, true);
 
     # Clean the restore paths
     $self->clean($oManifest);
@@ -1118,8 +1103,6 @@ sub process
     $self->build($oManifest);
 
     # Get variables required for restore
-    my $lCopyTimeBegin = $oManifest->numericGet(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_COPY_START);
-    my $bSourceCompression = $oManifest->boolGet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS);
     my $strCurrentUser = getpwuid($<);
     my $strCurrentGroup = getgrgid($();
 
@@ -1206,171 +1189,97 @@ sub process
         &log(DETAIL, "database filter: $strDbFilter");
     }
 
-    # Create hash containing files to restore
-    my %oRestoreHash;
+    # Initialize the restore process
+    my $oRestoreProcess = new pgBackRest::Protocol::LocalProcess(BACKUP);
+    $oRestoreProcess->hostAdd(1, optionGet(OPTION_PROCESS_MAX));
+
+    # Variables used for parallel copy
     my $lSizeTotal = 0;
     my $lSizeCurrent = 0;
 
-    foreach my $strFile ($oManifest->keys(MANIFEST_SECTION_TARGET_FILE))
+    foreach my $strRepoFile (
+        sort {sprintf("%016d-${b}", $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $b, MANIFEST_SUBKEY_SIZE)) cmp
+              sprintf("%016d-${a}", $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $a, MANIFEST_SUBKEY_SIZE))}
+        ($oManifest->keys(MANIFEST_SECTION_TARGET_FILE, 'none')))
     {
         # Skip the tablespace_map file in versions >= 9.5 so Postgres does not rewrite links in DB_PATH_PGTBLSPC.
         # The tablespace links have already been created by Restore::build().
-        if ($strFile eq MANIFEST_FILE_TABLESPACEMAP &&
+        if ($strRepoFile eq MANIFEST_FILE_TABLESPACEMAP &&
             $oManifest->get(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_VERSION) >= PG_VERSION_95)
         {
             next;
         }
 
-        my $lSize = $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_SIZE);
-
         # By default put everything into a single queue
         my $strQueueKey = MANIFEST_TARGET_PGDATA;
 
-        # If multiple threads and the file belongs in a tablespace then put in a tablespace-specific queue
-        if (optionGet(OPTION_THREAD_MAX) > 1 && index($strFile, DB_PATH_PGTBLSPC . '/') == 0)
+        # If the file belongs in a tablespace then put in a tablespace-specific queue
+        if (index($strRepoFile, DB_PATH_PGTBLSPC . '/') == 0)
         {
-            $strQueueKey = DB_PATH_PGTBLSPC . '/' . (split('\/', $strFile))[1];
-        }
-
-        # Preface the file key with the size. This allows for sorting the files to restore by size.
-        my $strFileKey;
-
-        # Skip this for global/pg_control since it will be copied as the last step and needs to named in a way that it
-        # can be found for the copy.
-        if ($strFile eq MANIFEST_FILE_PGCONTROL)
-        {
-            $strFileKey = $strFile;
-            $oRestoreHash{$strQueueKey}{$strFileKey}{skip} = true;
-        }
-        # Else continue normally
-        else
-        {
-            $strFileKey = sprintf("%016d-${strFile}", $lSize);
-            $oRestoreHash{$strQueueKey}{$strFileKey}{skip} = false;
-
-            $lSizeTotal += $lSize;
-        }
-
-        # Zero files that should be filtered
-        if (defined($strDbFilter) && $strFile =~ $strDbFilter && $strFile !~ /\/PG\_VERSION$/)
-        {
-            $oRestoreHash{$strQueueKey}{$strFileKey}{zero} = true;
+            $strQueueKey = DB_PATH_PGTBLSPC . '/' . (split('\/', $strRepoFile))[1];
         }
 
         # Get restore information
-        $oRestoreHash{$strQueueKey}{$strFileKey}{repo_file} = $strFile;
-        $oRestoreHash{$strQueueKey}{$strFileKey}{db_file} =  $oManifest->dbPathGet($self->{strDbClusterPath}, $strFile);
-        $oRestoreHash{$strQueueKey}{$strFileKey}{size} = $lSize;
-        $oRestoreHash{$strQueueKey}{$strFileKey}{reference} =
-            $oManifest->boolTest(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_HARDLINK, undef, true) ? undef :
-            $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_REFERENCE, false);
-        $oRestoreHash{$strQueueKey}{$strFileKey}{modification_time} =
-            $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_TIMESTAMP);
-        $oRestoreHash{$strQueueKey}{$strFileKey}{mode} =
-            $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_MODE);
-        $oRestoreHash{$strQueueKey}{$strFileKey}{user} =
-            $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_USER);
-        $oRestoreHash{$strQueueKey}{$strFileKey}{group} =
-            $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_GROUP);
+        my $strDbFile = $oManifest->dbPathGet($self->{strDbClusterPath}, $strRepoFile);
+        my $lSize = $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_SIZE);
 
-        # Checksum is only stored if size > 0
-        if ($lSize > 0)
+        # Copy pg_control to a temporary file that will be renamed later
+        if ($strRepoFile eq MANIFEST_TARGET_PGDATA . '/' . DB_FILE_PGCONTROL)
         {
-            $oRestoreHash{$strQueueKey}{$strFileKey}{checksum} =
-                $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strFile, MANIFEST_SUBKEY_CHECKSUM);
+            $strDbFile .= '.' . BACKREST_EXE;
         }
+
+        # Increment file size
+        $lSizeTotal += $lSize;
+
+        # Queue for parallel restore
+        $oRestoreProcess->queueJob(
+            1, $strQueueKey, $strRepoFile, OP_RESTORE_FILE,
+            [$strDbFile, $lSize,
+                $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_TIMESTAMP),
+                $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_CHECKSUM, $lSize > 0),
+                defined($strDbFilter) && $strRepoFile =~ $strDbFilter && $strRepoFile !~ /\/PG\_VERSION$/ ? true : false,
+                optionGet(OPTION_FORCE), $strRepoFile,
+                $oManifest->boolTest(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_HARDLINK, undef, true) ? undef :
+                    $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_REFERENCE, false),
+                $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_MODE),
+                $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_USER),
+                $oManifest->get(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_GROUP),
+                $oManifest->numericGet(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_COPY_START),  optionGet(OPTION_DELTA),
+                $self->{strBackupSet}, $oManifest->boolGet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS)]);
     }
 
-    # If multi-threaded then create threads to copy files
-    if (optionGet(OPTION_THREAD_MAX) > 1)
+    # Run the restore jobs and process results
+    while (my $hyJob = $oRestoreProcess->process())
     {
-        # Load module dynamically
-        require pgBackRest::Protocol::ThreadGroup;
-        pgBackRest::Protocol::ThreadGroup->import();
-
-        logDebugMisc
-        (
-            $strOperation, 'restore with threads',
-            {name => 'iThreadTotal', value => optionGet(OPTION_THREAD_MAX)}
-        );
-
-        # Initialize the thread queues
-        my @oyRestoreQueue;
-
-        foreach my $strQueueKey (sort (keys %oRestoreHash))
+        foreach my $hJob (@{$hyJob})
         {
-            push(@oyRestoreQueue, Thread::Queue->new());
-
-            foreach my $strFileKey (sort {$b cmp $a} (keys(%{$oRestoreHash{$strQueueKey}})))
-            {
-                # Skip files marked to be copied later
-                next if ($oRestoreHash{$strQueueKey}{$strFileKey}{skip});
-
-                $oyRestoreQueue[@oyRestoreQueue - 1]->enqueue($oRestoreHash{$strQueueKey}{$strFileKey});
-            }
+            ($lSizeCurrent) = restoreLog(
+                $hJob->{iProcessId}, @{$hJob->{rParam}}[0..5], @{$hJob->{rResult}}, $lSizeTotal, $lSizeCurrent);
         }
 
-        # Initialize the param hash
-        my %oParam;
-
-        $oParam{copy_time_begin} = $lCopyTimeBegin;
-        $oParam{size_total} = $lSizeTotal;
-        $oParam{delta} = optionGet(OPTION_DELTA);
-        $oParam{force} = optionGet(OPTION_FORCE);
-        $oParam{backup_path} = $self->{strBackupSet};
-        $oParam{source_compression} = $bSourceCompression;
-        $oParam{current_user} = $strCurrentUser;
-        $oParam{current_group} = $strCurrentGroup;
-        $oParam{queue} = \@oyRestoreQueue;
-
-        # Run the threads
-        for (my $iThreadIdx = 0; $iThreadIdx < optionGet(OPTION_THREAD_MAX); $iThreadIdx++)
-        {
-            threadGroupRun($iThreadIdx, 'restore', \%oParam);
-        }
-
-        # Complete thread queues
-        while (!threadGroupComplete())
-        {
-            # Keep the protocol layer from timing out
-            $self->{oProtocol}->keepAlive();
-        };
-    }
-    else
-    {
-        logDebugMisc($strOperation, 'restore in main process');
-
-        # Restore file in main process
-        foreach my $strQueueKey (sort (keys %oRestoreHash))
-        {
-            foreach my $strFileKey (sort {$b cmp $a} (keys(%{$oRestoreHash{$strQueueKey}})))
-            {
-                # Skip files marked to be copied later
-                next if ($oRestoreHash{$strQueueKey}{$strFileKey}{skip});
-
-                $lSizeCurrent = restoreFile($oRestoreHash{$strQueueKey}{$strFileKey}, $lCopyTimeBegin, optionGet(OPTION_DELTA),
-                                            optionGet(OPTION_FORCE), $self->{strBackupSet}, $bSourceCompression, $strCurrentUser,
-                                            $strCurrentGroup, $self->{oFile}, $lSizeTotal, $lSizeCurrent);
-
-                # Keep the protocol layer from timing out while checksumming
-                $self->{oProtocol}->keepAlive();
-            }
-        }
+        # A keep-alive is required here because if there are a large number of resumed files that need to be checksummed
+        # then the remote might timeout while waiting for a command.
+        $self->{oProtocol}->keepAlive();
     }
 
     # Create recovery.conf file
     $self->recovery($oManifest->get(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_VERSION));
 
-    # Copy pg_control last
-    &log(INFO, 'restore ' . $oManifest->dbPathGet(undef, MANIFEST_FILE_PGCONTROL) .
-         ' (copied last to ensure aborted restores cannot be started)');
+    # Sync db cluster path
+    $self->{oFile}->pathSync(PATH_DB_ABSOLUTE, $self->{strDbClusterPath}, true);
 
-    restoreFile($oRestoreHash{&MANIFEST_TARGET_PGDATA}{&MANIFEST_FILE_PGCONTROL}, $lCopyTimeBegin, optionGet(OPTION_DELTA),
-                optionGet(OPTION_FORCE), $self->{strBackupSet}, $bSourceCompression, $strCurrentUser, $strCurrentGroup,
-                $self->{oFile});
+    # Copy pg_control last
+    &log(INFO,
+        'restore ' . $oManifest->dbPathGet(undef, MANIFEST_FILE_PGCONTROL) .
+        ' (copied last to ensure aborted restores cannot be started)');
+
+    $self->{oFile}->move(
+        PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . DB_FILE_PGCONTROL . '.' . BACKREST_EXE,
+        PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . DB_FILE_PGCONTROL, undef, true);
 
     # Finally remove the manifest to indicate the restore is complete
-    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_MANIFEST, undef, false);
+    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_MANIFEST, undef, false, true);
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
