@@ -8,6 +8,7 @@ package pgBackRest::Version;
 use strict;
 use warnings FATAL => qw(all);
 
+use Cwd qw(abs_path);
 use Exporter qw(import);
     our @EXPORT = qw();
 
@@ -22,12 +23,19 @@ use constant BACKREST_EXE                                           => lc(BACKRE
 use constant BACKREST_CONF                                          => BACKREST_EXE . '.conf';
     push @EXPORT, qw(BACKREST_CONF);
 
+# Binary location
+#
+# Stores the exe location.
+#-----------------------------------------------------------------------------------------------------------------------------------
+use constant BACKREST_BIN                                           => abs_path($0);
+    push @EXPORT, qw(BACKREST_BIN);
+
 # BackRest Version Number
 #
 # Defines the current version of the BackRest executable.  The version number is used to track features but does not affect what
 # repositories or manifests can be read - that's the job of the format number.
 #-----------------------------------------------------------------------------------------------------------------------------------
-use constant BACKREST_VERSION                                       => '1.04';
+use constant BACKREST_VERSION                                       => '1.12';
     push @EXPORT, qw(BACKREST_VERSION);
 
 # Format Format Number
