@@ -15,6 +15,7 @@ use Exporter qw(import);
     our @EXPORT = qw();
 use Storable qw(dclone);
 
+use pgBackRest::BackupCommon;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
@@ -25,6 +26,7 @@ use pgBackRest::Manifest;
 use pgBackRest::Version;
 
 use pgBackRestTest::Backup::Common::HostBaseTest;
+use pgBackRestTest::Common::ContainerTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::HostGroupTest;
 use pgBackRestTest::CommonTest;
@@ -32,8 +34,6 @@ use pgBackRestTest::CommonTest;
 ####################################################################################################################################
 # Host constants
 ####################################################################################################################################
-use constant HOST_BACKUP                                            => 'backup';
-    push @EXPORT, qw(HOST_BACKUP);
 use constant HOST_BACKUP_USER                                       => 'backup-user';
     push @EXPORT, qw(HOST_BACKUP_USER);
 
@@ -42,8 +42,6 @@ use constant HOST_BACKUP_USER                                       => 'backup-u
 ####################################################################################################################################
 use constant HOST_PARAM_BACKREST_CONFIG                             => 'backrest-config';
     push @EXPORT, qw(HOST_PARAM_BACKREST_CONFIG);
-use constant HOST_PARAM_BACKREST_EXE                                => 'backrest-exe';
-    push @EXPORT, qw(HOST_PARAM_BACKREST_EXE);
 use constant HOST_PARAM_LOCK_PATH                                   => 'lock-path';
     push @EXPORT, qw(HOST_PARAM_LOCK_PATH);
 use constant HOST_PARAM_LOG_PATH                                    => 'log-path';
@@ -52,8 +50,8 @@ use constant HOST_PARAM_REPO_PATH                                   => 'repo-pat
     push @EXPORT, qw(HOST_PARAM_REPO_PATH);
 use constant HOST_PARAM_STANZA                                      => 'stanza';
     push @EXPORT, qw(HOST_PARAM_STANZA);
-use constant HOST_PARAM_THREAD_MAX                                  => 'thread-max';
-    push @EXPORT, qw(HOST_PARAM_THREAD_MAX);
+use constant HOST_PARAM_PROCESS_MAX                                 => 'process-max';
+    push @EXPORT, qw(HOST_PARAM_PROCESS_MAX);
 
 ####################################################################################################################################
 # Host paths
@@ -76,7 +74,7 @@ use constant HOST_PROTOCOL_TIMEOUT                                  => 10;
 ####################################################################################################################################
 # Cached data sections
 ####################################################################################################################################
-use constant SECTION_FILE_NAME                                        => 'strFileName';
+use constant SECTION_FILE_NAME                                      => 'strFileName';
     push @EXPORT, qw(SECTION_FILE_NAME);
 
 ####################################################################################################################################
@@ -105,14 +103,9 @@ sub new
     if (!defined($$oParam{strName}) || $$oParam{strName} eq HOST_BACKUP)
     {
         $strName = HOST_BACKUP;
-        $strImage = 'backrest/' . $oHostGroup->paramGet(HOST_PARAM_VM) . '-backup-test-pre';
+        $strImage = containerNamespace() . '/' . $oHostGroup->paramGet(HOST_PARAM_VM) . '-backup-test-pre';
         $strUser = $oHostGroup->paramGet(HOST_BACKUP_USER);
         $strVm = $oHostGroup->paramGet(HOST_PARAM_VM);
-
-        if (!defined($$oParam{strDbMaster}))
-        {
-            confess &log(ERROR, "strDbMaster must be specified for dedicated backup hosts");
-        }
     }
     else
     {
@@ -127,13 +120,11 @@ sub new
     bless $self, $class;
 
     # Set parameters
-    if (defined($$oParam{oHostBackup}))
+    $self->paramSet(
+        HOST_PARAM_REPO_PATH, $oHostGroup->paramGet(HOST_PARAM_TEST_PATH) . "/$$oParam{strBackupDestination}/" . HOST_PATH_REPO);
+
+    if ($$oParam{strBackupDestination} eq $self->nameGet())
     {
-        $self->paramSet(HOST_PARAM_REPO_PATH, $$oParam{oHostBackup}->repoPath());
-    }
-    else
-    {
-        $self->paramSet(HOST_PARAM_REPO_PATH, $self->testPath() . '/' . HOST_PATH_REPO);
         $self->paramSet(HOST_PARAM_LOG_PATH, $self->repoPath() . '/' . HOST_PATH_LOG);
         $self->paramSet(HOST_PARAM_LOCK_PATH, $self->repoPath() . '/' . HOST_PATH_LOCK);
         filePathCreate($self->repoPath(), '0770');
@@ -142,20 +133,24 @@ sub new
     $self->paramSet(HOST_PARAM_BACKREST_CONFIG, $self->testPath() . '/' . BACKREST_CONF);
     $self->paramSet(HOST_PARAM_BACKREST_EXE, $oHostGroup->paramGet(HOST_PARAM_BACKREST_EXE));
     $self->paramSet(HOST_PARAM_STANZA, HOST_STANZA);
-    $self->paramSet(HOST_PARAM_THREAD_MAX, $oHostGroup->paramGet(HOST_PARAM_THREAD_MAX));
+    $self->paramSet(HOST_PARAM_PROCESS_MAX, $oHostGroup->paramGet(HOST_PARAM_PROCESS_MAX));
 
     # Set LogTest object
     $self->{oLogTest} = $$oParam{oLogTest};
 
-    # Set db master host (this is the host where the backups are run)
-    $self->{strDbMaster} = $$oParam{strDbMaster};
+    # Set synthetic
     $self->{bSynthetic} = defined($$oParam{bSynthetic}) && $$oParam{bSynthetic} ? true : false;
 
+    # Set the backup destination
+    $self->{strBackupDestination} = $$oParam{strBackupDestination};
+
+    # Default hardlink to false
+    $self->{bHardLink} = false;
+
     # Create the local file object
     $self->{oFile} = new pgBackRest::File(
         $self->stanza(),
         $self->repoPath(),
-        undef,
         new pgBackRest::Protocol::Common
         (
             OPTION_DEFAULT_BUFFER_SIZE,                 # Buffer size
@@ -203,11 +198,6 @@ sub backupBegin
     my $fTestDelay = defined($$oParam{fTestDelay}) ? $$oParam{fTestDelay} : .2;
     my $oExpectedManifest = defined($$oParam{oExpectedManifest}) ? $$oParam{oExpectedManifest} : undef;
 
-    if (!defined($$oParam{iExpectedExitStatus}) && $self->threadMax() > 1)
-    {
-        $$oParam{iExpectedExitStatus} = -1;
-    }
-
     $strComment =
         "${strType} backup" . (defined($strComment) ? " - ${strComment}" : '') .
         ' (' . $self->nameGet() . ' host)';
@@ -220,10 +210,13 @@ sub backupBegin
         ' --config=' . $self->backrestConfig() .
         (defined($oExpectedManifest) ? " --no-online" : '') .
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
+        (defined($$oParam{bStandby}) && $$oParam{bStandby} ? " --backup-standby" : '') .
+        (defined($oParam->{bRepoLink}) && !$oParam->{bRepoLink} ? ' --no-repo-link' : '') .
         ($strType ne 'incr' ? " --type=${strType}" : '') .
-        ' --stanza=' . $self->stanza() . ' backup' .
+        ' --stanza=' . (defined($oParam->{strStanza}) ? $oParam->{strStanza} : $self->stanza()) . ' backup' .
         (defined($strTest) ? " --test --test-delay=${fTestDelay} --test-point=" . lc($strTest) . '=y' : ''),
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest}});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus},
+         oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
 
     $oExecuteBackup->begin();
 
@@ -265,45 +258,151 @@ sub backupEnd
         );
 
     # Set defaults
-    my $oExpectedManifest = defined($$oParam{oExpectedManifest}) ? $$oParam{oExpectedManifest} : undef;
+    my $oExpectedManifest = defined($$oParam{oExpectedManifest}) ? dclone($$oParam{oExpectedManifest}) : undef;
 
     my $iExitStatus = $oExecuteBackup->end();
 
-    if ($oExecuteBackup->{iExpectedExitStatus} != 0 && $oExecuteBackup->{iExpectedExitStatus} != -1)
+    return if ($oExecuteBackup->{iExpectedExitStatus} != 0);
+
+    # If an alternate stanza was specified
+    if (defined($oParam->{strStanza}))
     {
-        return;
+        confess &log(ASSERT,
+            'if an alternate stanza is specified it must generate an error - the remaining code will not be aware of the stanza');
     }
 
     my $strBackup = $self->backupLast();
 
-    # Only do compare for synthetic backups
-    if (defined($oExpectedManifest))
+    # If a real backup then load the expected manifest from the actual manifest.  An expected manifest can't be generated perfectly
+    # because a running database is always in flux.  Even so, it allows us test many things.
+    if (!$self->synthetic())
+    {
+        $oExpectedManifest =
+            iniLoad($self->repoPath() . '/backup/' . $self->stanza() . "/${strBackup}/" . FILE_MANIFEST, $oExpectedManifest);
+    }
+
+    # Make sure tablespace links are correct
+    if ($strType eq BACKUP_TYPE_FULL || $self->hardLink())
+    {
+        my $hTablespaceManifest = $self->{oFile}->manifest(
+            PATH_BACKUP_CLUSTER, "${strBackup}/" . MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGTBLSPC);
+
+        # Remove . and ..
+        delete($hTablespaceManifest->{'.'});
+        delete($hTablespaceManifest->{'..'});
+
+        # Only check links if repo links are disabled
+        if (!defined($oParam->{bRepoLink}) || $oParam->{bRepoLink})
+        {
+            # Iterate file links
+            for my $strFile (sort(keys(%{$hTablespaceManifest})))
+            {
+                # Make sure the link is in the expected manifest
+                my $hManifestTarget =
+                    $oExpectedManifest->{&MANIFEST_SECTION_BACKUP_TARGET}{&MANIFEST_TARGET_PGTBLSPC . "/${strFile}"};
+
+                if (!defined($hManifestTarget) || $hManifestTarget->{&MANIFEST_SUBKEY_TYPE} ne MANIFEST_VALUE_LINK ||
+                    $hManifestTarget->{&MANIFEST_SUBKEY_TABLESPACE_ID} ne $strFile)
+                {
+                    confess &log(ERROR, "'${strFile}' is not in expected manifest as a link with the correct tablespace id");
+                }
+
+                # Make sure the link really is a link
+                if ($hTablespaceManifest->{$strFile}{type} ne 'l')
+                {
+                    confess &log(ERROR, "'${strFile}' in tablespace directory is not a link");
+                }
+
+                # Make sure the link destination is correct
+                my $strLinkDestination = '../../' . MANIFEST_TARGET_PGTBLSPC . "/${strFile}";
+
+                if ($hTablespaceManifest->{$strFile}{link_destination} ne $strLinkDestination)
+                {
+                    confess &log(ERROR,
+                        "'${strFile}' link should reference '${strLinkDestination}' but actually references " .
+                        "'$hTablespaceManifest->{$strFile}{link_destination}'");
+                }
+            }
+
+            # Iterate manifest targets
+            for my $strTarget (sort(keys(%{$oExpectedManifest->{&MANIFEST_SECTION_BACKUP_TARGET}})))
+            {
+                my $hManifestTarget = $oExpectedManifest->{&MANIFEST_SECTION_BACKUP_TARGET}{$strTarget};
+                my $strTablespaceId = $hManifestTarget->{&MANIFEST_SUBKEY_TABLESPACE_ID};
+
+                # Make sure the target exists as a link on disk
+                if ($hManifestTarget->{&MANIFEST_SUBKEY_TYPE} eq MANIFEST_VALUE_LINK && defined($strTablespaceId) &&
+                    !defined($hTablespaceManifest->{$strTablespaceId}))
+                {
+                    confess &log(ERROR,
+                        "target '${strTarget}' does not have a link at '" . DB_PATH_PGTBLSPC. "/${strTablespaceId}'");
+                }
+            }
+        }
+        # Else make sure the tablespace directory is empty
+        elsif (keys(%{$hTablespaceManifest}) > 0)
+        {
+            confess &log(ERROR, DB_PATH_PGTBLSPC . ' directory should be empty when --no-' . OPTION_REPO_LINK);
+        }
+    }
+    # Else there should not be a tablespace directory at all
+    elsif ($self->{oFile}->exists(PATH_BACKUP_CLUSTER, "${strBackup}/" . MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGTBLSPC))
+    {
+        confess &log(ERROR, 'backup must be full or hard-linked to have ' . DB_PATH_PGTBLSPC . ' directory');
+    }
+
+    # Check that latest link exists unless repo links are disabled
+    my $strLatestLink = $self->{oFile}->pathGet(PATH_BACKUP_CLUSTER, LINK_LATEST);
+    my $bLatestLinkExists = fileExists($strLatestLink);
+
+    if (!defined($oParam->{bRepoLink}) || $oParam->{bRepoLink})
+    {
+        my $strLatestLinkDestination = readlink($strLatestLink);
+
+        if ($strLatestLinkDestination ne $strBackup)
+        {
+            confess &log(ERROR, "'" . LINK_LATEST . "' link should be '${strBackup}' but is '${strLatestLinkDestination}");
+        }
+    }
+    elsif ($bLatestLinkExists)
+    {
+        confess &log(ERROR, "'" . LINK_LATEST . "' link should not exist when --no-" . OPTION_REPO_LINK);
+    }
+
+    # Only do compare for synthetic backups since for real backups the expected manifest *is* the actual manifest.
+    if ($self->synthetic())
     {
         # Set backup type in the expected manifest
         ${$oExpectedManifest}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TYPE} = $strType;
 
-
         $self->backupCompare($strBackup, $oExpectedManifest);
     }
 
     # Add files to expect log
     if (defined($self->{oLogTest}) && (!defined($$oParam{bSupplemental}) || $$oParam{bSupplemental}))
     {
-        if ($self->nameTest(HOST_BACKUP))
-        {
-            my $oHostGroup = hostGroupGet();
-            my $oHostDbMaster = $oHostGroup->hostGet($self->{strDbMaster}, true);
+        my $oHostGroup = hostGroupGet();
 
-            if (defined($oHostDbMaster))
-            {
-                $self->{oLogTest}->supplementalAdd($oHostDbMaster->testPath() . '/' . BACKREST_CONF);
-            }
+        if (defined($oHostGroup->hostGet(HOST_DB_MASTER, true)))
+        {
+            $self->{oLogTest}->supplementalAdd($oHostGroup->hostGet(HOST_DB_MASTER)->testPath() . '/' . BACKREST_CONF);
         }
 
-        $self->{oLogTest}->supplementalAdd($self->testPath() . '/' . BACKREST_CONF);
+        if (defined($oHostGroup->hostGet(HOST_DB_STANDBY, true)))
+        {
+            $self->{oLogTest}->supplementalAdd($oHostGroup->hostGet(HOST_DB_STANDBY)->testPath() . '/' . BACKREST_CONF);
+        }
 
-        $self->{oLogTest}->supplementalAdd($self->{oFile}->pathGet(PATH_BACKUP_CLUSTER, "${strBackup}/" . FILE_MANIFEST));
-        $self->{oLogTest}->supplementalAdd($self->repoPath() . '/backup/' . $self->stanza() . '/backup.info');
+        if (defined($oHostGroup->hostGet(HOST_BACKUP, true)))
+        {
+            $self->{oLogTest}->supplementalAdd($oHostGroup->hostGet(HOST_BACKUP)->testPath() . '/' . BACKREST_CONF);
+        }
+
+        if ($self->synthetic())
+        {
+            $self->{oLogTest}->supplementalAdd($self->{oFile}->pathGet(PATH_BACKUP_CLUSTER, "${strBackup}/" . FILE_MANIFEST));
+            $self->{oLogTest}->supplementalAdd($self->repoPath() . '/backup/' . $self->stanza() . '/backup.info');
+        }
     }
 
     # Return from function and log return values if any
@@ -371,7 +470,7 @@ sub backupCompare
 
     ${$oExpectedManifest}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_LABEL} = $strBackup;
 
-    my $oActualManifest = new pgBackRest::Common::Ini(
+    my $oActualManifest = new pgBackRest::Manifest(
         $self->{oFile}->pathGet(PATH_BACKUP_CLUSTER, "${strBackup}/" . FILE_MANIFEST));
 
     ${$oExpectedManifest}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_START} =
@@ -384,38 +483,81 @@ sub backupCompare
         $oActualManifest->get(INI_SECTION_BACKREST, INI_KEY_CHECKSUM);
     ${$oExpectedManifest}{&INI_SECTION_BACKREST}{&INI_KEY_FORMAT} = BACKREST_FORMAT + 0;
 
-    foreach my $strPathKey ($oActualManifest->keys(MANIFEST_SECTION_TARGET_PATH))
-    {
-        my $strFileSection = MANIFEST_SECTION_TARGET_FILE;
+    my $strSectionPath = $oActualManifest->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH);
 
-        foreach my $strFileKey ($oActualManifest->keys($strFileSection))
+    foreach my $strFileKey ($oActualManifest->keys(MANIFEST_SECTION_TARGET_FILE))
+    {
+        # Determine repo size if compression is enabled
+        if ($oExpectedManifest->{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS})
         {
-            if ($oActualManifest->test($strFileSection, $strFileKey, MANIFEST_SUBKEY_REPO_SIZE))
+            my $lRepoSize =
+                $oActualManifest->test(MANIFEST_SECTION_TARGET_FILE, $strFileKey, MANIFEST_SUBKEY_REFERENCE) ?
+                    $oActualManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $strFileKey, MANIFEST_SUBKEY_REPO_SIZE, false) :
+                    (fileStat($self->{oFile}->pathGet(PATH_BACKUP_CLUSTER, "${strBackup}/${strFileKey}.gz")))->size;
+
+            if (defined($lRepoSize) &&
+                $lRepoSize != $oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE}{$strFileKey}{&MANIFEST_SUBKEY_SIZE})
             {
-                ${$oExpectedManifest}{$strFileSection}{$strFileKey}{&MANIFEST_SUBKEY_REPO_SIZE} =
-                    $oActualManifest->get($strFileSection, $strFileKey, MANIFEST_SUBKEY_REPO_SIZE);
+                $oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE}{$strFileKey}{&MANIFEST_SUBKEY_REPO_SIZE} = $lRepoSize;
             }
         }
+
+        # If the backup does not have page checksums then no need to compare
+        if (!$oExpectedManifest->{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_CHECKSUM_PAGE})
+        {
+            delete($oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE}{$strFileKey}{&MANIFEST_SUBKEY_CHECKSUM_PAGE});
+            delete($oExpectedManifest->{&MANIFEST_SECTION_TARGET_FILE}{$strFileKey}{&MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR});
+        }
+        # Else make sure things that should have checks do have checks
+        elsif ($oActualManifest->test(MANIFEST_SECTION_TARGET_FILE, $strFileKey, MANIFEST_SUBKEY_CHECKSUM_PAGE) !=
+               isChecksumPage($strFileKey))
+        {
+            confess
+                "check-page actual for ${strFileKey} is " .
+                ($oActualManifest->test(MANIFEST_SECTION_TARGET_FILE, $strFileKey,
+                    MANIFEST_SUBKEY_CHECKSUM_PAGE) ? 'set' : '[undef]') .
+                ' but isChecksumPage() says it should be ' .
+                (isChecksumPage($strFileKey) ? 'set' : 'undef') . '.';
+        }
     }
 
+    $self->manifestDefault($oExpectedManifest);
+
+    my $strTestPath = $self->testPath();
+
+    iniSave("${strTestPath}/actual.manifest", $oActualManifest->{oContent});
+    iniSave("${strTestPath}/expected.manifest", $oExpectedManifest);
+
+    executeTest("diff ${strTestPath}/expected.manifest ${strTestPath}/actual.manifest");
+
+    fileRemove("${strTestPath}/expected.manifest");
+    fileRemove("${strTestPath}/actual.manifest");
+
+    # Return from function and log return values if any
+    return logDebugReturn($strOperation);
+}
+
+####################################################################################################################################
+# manifestDefault
+####################################################################################################################################
+sub manifestDefault
+{
+    my $self = shift;
+    my $oExpectedManifest = shift;
+
     # Set defaults for subkeys that tend to repeat
     foreach my $strSection (&MANIFEST_SECTION_TARGET_FILE, &MANIFEST_SECTION_TARGET_PATH, &MANIFEST_SECTION_TARGET_LINK)
     {
-        foreach my $strSubKey (&MANIFEST_SUBKEY_USER, &MANIFEST_SUBKEY_GROUP, &MANIFEST_SUBKEY_MODE)
+        foreach my $strSubKey (&MANIFEST_SUBKEY_USER, &MANIFEST_SUBKEY_GROUP, &MANIFEST_SUBKEY_MODE, &MANIFEST_SUBKEY_MASTER)
         {
             my %oDefault;
             my $iSectionTotal = 0;
 
-            foreach my $strFile (keys(%{${$oExpectedManifest}{$strSection}}))
-            {
-                if (!defined(${$oExpectedManifest}{$strSection}{$strFile}{$strSubKey}) &&
-                    defined(${$oExpectedManifest}{"${strSection}:default"}{$strSubKey}))
-                {
-                    ${$oExpectedManifest}{$strSection}{$strFile}{$strSubKey} =
-                        ${$oExpectedManifest}{"${strSection}:default"}{$strSubKey};
-                }
+            next if !defined($oExpectedManifest->{$strSection});
 
-                my $strValue = ${$oExpectedManifest}{$strSection}{$strFile}{$strSubKey};
+            foreach my $strFile (keys(%{$oExpectedManifest->{$strSection}}))
+            {
+                my $strValue = $oExpectedManifest->{$strSection}{$strFile}{$strSubKey};
 
                 if (defined($strValue))
                 {
@@ -446,32 +588,26 @@ sub backupCompare
 
             if (defined($strMaxValue) > 0 && $iMaxValueTotal > $iSectionTotal * MANIFEST_DEFAULT_MATCH_FACTOR)
             {
-                ${$oExpectedManifest}{"${strSection}:default"}{$strSubKey} = $strMaxValue;
-
-                foreach my $strFile (keys(%{${$oExpectedManifest}{$strSection}}))
+                if ($strSubKey eq MANIFEST_SUBKEY_MASTER)
                 {
-                    if (defined(${$oExpectedManifest}{$strSection}{$strFile}{$strSubKey}) &&
-                        ${$oExpectedManifest}{$strSection}{$strFile}{$strSubKey} eq $strMaxValue)
+                    $oExpectedManifest->{"${strSection}:default"}{$strSubKey} = $strMaxValue ? JSON::PP::true : JSON::PP::false;
+                }
+                else
+                {
+                    $oExpectedManifest->{"${strSection}:default"}{$strSubKey} = $strMaxValue;
+                }
+
+                foreach my $strFile (keys(%{$oExpectedManifest->{$strSection}}))
+                {
+                    if (defined($oExpectedManifest->{$strSection}{$strFile}{$strSubKey}) &&
+                        $oExpectedManifest->{$strSection}{$strFile}{$strSubKey} eq $strMaxValue)
                     {
-                        delete(${$oExpectedManifest}{$strSection}{$strFile}{$strSubKey});
+                        delete($oExpectedManifest->{$strSection}{$strFile}{$strSubKey});
                     }
                 }
             }
         }
     }
-
-    my $strTestPath = $self->testPath();
-
-    iniSave("${strTestPath}/actual.manifest", $oActualManifest->{oContent});
-    iniSave("${strTestPath}/expected.manifest", $oExpectedManifest);
-
-    executeTest("diff ${strTestPath}/expected.manifest ${strTestPath}/actual.manifest");
-
-    fileRemove("${strTestPath}/expected.manifest");
-    fileRemove("${strTestPath}/actual.manifest");
-
-    # Return from function and log return values if any
-    return logDebugReturn($strOperation);
 }
 
 ####################################################################################################################################
@@ -481,14 +617,15 @@ sub backupLast
 {
     my $self = shift;
 
-    my @stryBackup = $self->{oFile}->list(PATH_BACKUP_CLUSTER, undef, undef, 'reverse');
+    my @stryBackup = $self->{oFile}->list(
+        PATH_BACKUP_CLUSTER, undef, '[0-9]{8}-[0-9]{6}F(_[0-9]{8}-[0-9]{6}(D|I)){0,1}', 'reverse');
 
-    if (!defined($stryBackup[3]))
+    if (!defined($stryBackup[0]))
     {
         confess 'no backup was found: ' . join(@stryBackup, ', ');
     }
 
-    return $stryBackup[3];
+    return $stryBackup[0];
 }
 
 ####################################################################################################################################
@@ -524,7 +661,8 @@ sub check
         (defined($$oParam{iTimeout}) ? " --archive-timeout=$$oParam{iTimeout}" : '') .
         (defined($$oParam{strOptionalParam}) ? " $$oParam{strOptionalParam}" : '') .
         ' --stanza=' . $self->stanza() . ' check',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest}});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
+         bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -566,7 +704,8 @@ sub expire
         (defined($$oParam{iRetentionFull}) ? " --retention-full=$$oParam{iRetentionFull}" : '') .
         (defined($$oParam{iRetentionDiff}) ? " --retention-diff=$$oParam{iRetentionDiff}" : '') .
         '  --stanza=' . $self->stanza() . ' expire',
-        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest}});
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
+         bLogOutput => $self->synthetic()});
 }
 
 ####################################################################################################################################
@@ -601,7 +740,45 @@ sub info
         ' --log-level-console=warn' .
         (defined($$oParam{strStanza}) ? " --stanza=$$oParam{strStanza}" : '') .
         (defined($$oParam{strOutput}) ? " --output=$$oParam{strOutput}" : '') . ' info',
-        {strComment => $strComment, oLogTest => $self->{oLogTest}});
+        {strComment => $strComment, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
+
+    # Return from function and log return values if any
+    return logDebugReturn($strOperation);
+}
+
+####################################################################################################################################
+# stanzaCreate
+####################################################################################################################################
+sub stanzaCreate
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $strComment,
+        $oParam,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->check', \@_,
+            {name => 'strComment'},
+            {name => 'oParam', required => false},
+        );
+
+    $strComment =
+        'stanza-create ' . $self->stanza() . ' - ' . $strComment .
+        ' (' . $self->nameGet() . ' host)';
+    &log(INFO, "    $strComment");
+
+    $self->executeSimple(
+        $self->backrestExe() .
+        ' --config=' . $self->backrestConfig() .
+        (defined($$oParam{iTimeout}) ? " --archive-timeout=$$oParam{iTimeout}" : '') .
+        ' --stanza=' . $self->stanza() . ' stanza-create',
+        {strComment => $strComment, iExpectedExitStatus => $$oParam{iExpectedExitStatus}, oLogTest => $self->{oLogTest},
+         bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -635,7 +812,7 @@ sub start
         $self->backrestExe() .
         ' --config=' . $self->backrestConfig() .
         (defined($$oParam{strStanza}) ? " --stanza=$$oParam{strStanza}" : '') . ' start',
-        {strComment => $strComment, oLogTest => $self->{oLogTest}});
+        {strComment => $strComment, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
 }
 
 ####################################################################################################################################
@@ -667,99 +844,65 @@ sub stop
         ' --config=' . $self->backrestConfig() .
         (defined($$oParam{strStanza}) ? " --stanza=$$oParam{strStanza}" : '') .
         (defined($$oParam{bForce}) && $$oParam{bForce} ? ' --force' : '') . ' stop',
-        {strComment => $strComment, oLogTest => $self->{oLogTest}});
+        {strComment => $strComment, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
 }
 
-
 ####################################################################################################################################
 # configCreate
 ####################################################################################################################################
 sub configCreate
 {
     my $self = shift;
-    my $oHostRemote = shift;
-    my $bCompress = shift;
-    my $bHardlink = shift;
-    my $bArchiveAsync = shift;
-    my $bCompressAsync = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $oParam,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->stop', \@_,
+            {name => 'oParam', required => false},
+        );
 
     my %oParamHash;
     my $strStanza = $self->stanza();
+    my $oHostGroup = hostGroupGet();
+    my $oHostBackup = $oHostGroup->hostGet($self->backupDestination());
+    my $oHostDbMaster = $oHostGroup->hostGet(HOST_DB_MASTER);
+    my $oHostDbStandby = $oHostGroup->hostGet(HOST_DB_STANDBY, true);
 
-    if (defined($oHostRemote))
-    {
-        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_COMMAND_REMOTE} = $self->backrestExe();
-    }
-
-    if (defined($oHostRemote) && $oHostRemote->nameTest(HOST_BACKUP))
-    {
-        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_BACKUP_HOST} = $oHostRemote->nameGet();
-        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_BACKUP_USER} = $oHostRemote->userGet();
-    }
-    elsif (defined($oHostRemote))
-    {
-        $oParamHash{$strStanza}{&OPTION_DB_HOST} = $oHostRemote->nameGet();
-        $oParamHash{$strStanza}{&OPTION_DB_USER} = $oHostRemote->userGet();
-    }
+    my $bArchiveAsync = defined($$oParam{bArchiveAsync}) ? $$oParam{bArchiveAsync} : false;
 
+    # General options
+    # ------------------------------------------------------------------------------------------------------------------------------
     $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_LEVEL_CONSOLE} = lc(DEBUG);
     $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_LEVEL_FILE} = lc(TRACE);
+    $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_LEVEL_STDERR} = lc(OFF);
 
     $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_REPO_PATH} = $self->repoPath();
     $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_PATH} = $self->logPath();
     $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOCK_PATH} = $self->lockPath();
 
-    if ($self->nameTest(HOST_BACKUP))
+    if ($self->processMax() > 1)
     {
-        $oParamHash{$strStanza}{&OPTION_DB_PATH} = $oHostRemote->dbBasePath();
-
-        if (!$self->synthetic())
-        {
-            $oParamHash{$strStanza}{&OPTION_DB_SOCKET_PATH} = $oHostRemote->dbSocketPath();
-            $oParamHash{$strStanza}{&OPTION_DB_PORT} = $oHostRemote->dbPort();
-        }
-    }
-    else
-    {
-        if ($oHostRemote)
-        {
-            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_PATH} = $self->logPath();
-            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOCK_PATH} = $self->lockPath();
-        }
-
-        if ($bArchiveAsync)
-        {
-            $oParamHash{&CONFIG_SECTION_GLOBAL . ':' . &CMD_ARCHIVE_PUSH}{&OPTION_ARCHIVE_ASYNC} = 'y';
-            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_SPOOL_PATH} =
-                defined($oHostRemote) ? $self->spoolPath() : $self->repoPath();
-        }
-
-        $oParamHash{$strStanza}{&OPTION_DB_PATH} = $self->dbBasePath();
-
-        if (!$self->synthetic())
-        {
-            $oParamHash{$strStanza}{&OPTION_DB_SOCKET_PATH} = $self->dbSocketPath();
-            $oParamHash{$strStanza}{&OPTION_DB_PORT} = $self->dbPort();
-        }
+        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_PROCESS_MAX} = $self->processMax();
     }
 
-    if (defined($oHostRemote))
+    if (defined($$oParam{bCompress}) && !$$oParam{bCompress})
     {
-        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_CONFIG_REMOTE} = $oHostRemote->backrestConfig();
+        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_COMPRESS} = 'n';
     }
 
-    if ($self->threadMax() > 1)
+    if ($self->isHostBackup())
     {
-        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_THREAD_MAX} = $self->threadMax();
-    }
-
-    if ($self->nameTest(HOST_BACKUP) || !defined($oHostRemote))
-    {
-        if (defined($bHardlink) && $bHardlink)
+        if (defined($$oParam{bHardlink}) && $$oParam{bHardlink})
         {
+            $self->{bHardLink} = true;
             $oParamHash{&CONFIG_SECTION_GLOBAL . ':' . &CMD_BACKUP}{&OPTION_HARDLINK} = 'y';
         }
 
@@ -767,9 +910,86 @@ sub configCreate
         $oParamHash{&CONFIG_SECTION_GLOBAL . ':' . &CMD_BACKUP}{&OPTION_START_FAST} = 'y';
     }
 
-    if (defined($bCompress) && !$bCompress)
+    # Host specific options
+    # ------------------------------------------------------------------------------------------------------------------------------
+
+    # If this is the backup host
+    if ($self->isHostBackup())
     {
-        $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_COMPRESS} = 'n';
+        my $bForce = false;
+        my $oHostDb1 = $oHostDbMaster;
+        my $oHostDb2 = $oHostDbStandby;
+
+        if ($self->nameTest(HOST_BACKUP))
+        {
+            $bForce = defined($oHostDbStandby);
+        }
+        elsif ($self->nameTest(HOST_DB_STANDBY))
+        {
+            $oHostDb1 = $oHostDbStandby;
+            $oHostDb2 = $oHostDbMaster;
+        }
+
+        if ($self->nameTest(HOST_BACKUP))
+        {
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_HOST, 1, $bForce)} = $oHostDb1->nameGet();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_USER, 1, $bForce)} = $oHostDb1->userGet();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_CMD, 1, $bForce)} = $oHostDb1->backrestExe();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_CONFIG, 1, $bForce)} = $oHostDb1->backrestConfig();
+
+            # Port can't be configured for a synthetic host
+            if (!$self->synthetic())
+            {
+                $oParamHash{$strStanza}{optionIndex(OPTION_DB_PORT, 1, $bForce)} = $oHostDb1->dbPort();
+            }
+        }
+
+        $oParamHash{$strStanza}{optionIndex(OPTION_DB_PATH, 1, $bForce)} = $oHostDb1->dbBasePath();
+
+        if (defined($oHostDb2))
+        {
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_HOST, 2)} = $oHostDb2->nameGet();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_USER, 2)} = $oHostDb2->userGet();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_CMD, 2)} = $oHostDb2->backrestExe();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_CONFIG, 2)} = $oHostDb2->backrestConfig();
+            $oParamHash{$strStanza}{optionIndex(OPTION_DB_PATH, 2)} = $oHostDb2->dbBasePath();
+
+            # Only test explicit ports on the backup server.  This is so locally configured ports are also tested.
+            if (!$self->synthetic() && $self->nameTest(HOST_BACKUP))
+            {
+                $oParamHash{$strStanza}{optionIndex(OPTION_DB_PORT, 2)} = $oHostDb2->dbPort();
+            }
+        }
+    }
+
+    # If this is a database host
+    if ($self->isHostDb())
+    {
+        $oParamHash{$strStanza}{&OPTION_DB_PATH} = $self->dbBasePath();
+
+        if (!$self->synthetic())
+        {
+            $oParamHash{$strStanza}{&OPTION_DB_SOCKET_PATH} = $self->dbSocketPath();
+            $oParamHash{$strStanza}{&OPTION_DB_PORT} = $self->dbPort();
+        }
+
+        if ($bArchiveAsync)
+        {
+            $oParamHash{&CONFIG_SECTION_GLOBAL . ':' . &CMD_ARCHIVE_PUSH}{&OPTION_ARCHIVE_ASYNC} = 'y';
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_SPOOL_PATH} = $self->spoolPath();
+        }
+
+        # If the the backup host is remote
+        if (!$self->isHostBackup())
+        {
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_BACKUP_HOST} = $oHostBackup->nameGet();
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_BACKUP_USER} = $oHostBackup->userGet();
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_BACKUP_CMD} = $oHostBackup->backrestExe();
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_BACKUP_CONFIG} = $oHostBackup->backrestConfig();
+
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_PATH} = $self->logPath();
+            $oParamHash{&CONFIG_SECTION_GLOBAL}{&OPTION_LOCK_PATH} = $self->lockPath();
+        }
     }
 
     # Write out the configuration file
@@ -952,12 +1172,18 @@ sub infoRestore
 # Getters
 ####################################################################################################################################
 sub backrestConfig {return shift->paramGet(HOST_PARAM_BACKREST_CONFIG);}
+sub backupDestination {return shift->{strBackupDestination};}
 sub backrestExe {return shift->paramGet(HOST_PARAM_BACKREST_EXE);}
+sub hardLink {return shift->{bHardLink};}
+sub isHostBackup {my $self = shift; return $self->backupDestination() eq $self->nameGet();}
+sub isHostDbMaster {return shift->nameGet() eq HOST_DB_MASTER;}
+sub isHostDbStandby {return shift->nameGet() eq HOST_DB_STANDBY;}
+sub isHostDb {my $self = shift; return $self->isHostDbMaster() || $self->isHostDbStandby();}
 sub lockPath {return shift->paramGet(HOST_PARAM_LOCK_PATH);}
 sub logPath {return shift->paramGet(HOST_PARAM_LOG_PATH);}
 sub repoPath {return shift->paramGet(HOST_PARAM_REPO_PATH);}
 sub stanza {return shift->paramGet(HOST_PARAM_STANZA);}
-sub threadMax {return shift->paramGet(HOST_PARAM_THREAD_MAX);}
+sub processMax {return shift->paramGet(HOST_PARAM_PROCESS_MAX);}
 sub synthetic {return shift->{bSynthetic};}
 
 1;
