@@ -15,6 +15,7 @@ use DBI;
 use Exporter qw(import);
     our @EXPORT = qw();
 use Fcntl ':mode';
+use Storable qw(dclone);
 
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
@@ -22,7 +23,7 @@ use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Common::Wait;
 use pgBackRest::Config::Config;
-use pgBackRest::Db;
+use pgBackRest::DbVersion;
 use pgBackRest::File;
 use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
@@ -36,10 +37,8 @@ use pgBackRestTest::Common::HostGroupTest;
 ####################################################################################################################################
 # Host constants
 ####################################################################################################################################
-use constant HOST_DB_MASTER                                         => 'db-master';
-    push @EXPORT, qw(HOST_DB_MASTER);
-use constant HOST_DB_MASTER_USER                                    => 'db-master-user';
-    push @EXPORT, qw(HOST_DB_MASTER_USER);
+use constant HOST_DB_USER                                           => 'db-user';
+    push @EXPORT, qw(HOST_DB_USER);
 
 ####################################################################################################################################
 # Host parameters
@@ -82,28 +81,35 @@ sub new
             {name => 'oParam', required => false, trace => true},
         );
 
+    # Get host group
+    my $oHostGroup = hostGroupGet();
+
+    # Is standby?
+    my $bStandby = defined($$oParam{bStandby}) && $$oParam{bStandby} ? true : false;
+
     my $self = $class->SUPER::new(
         {
-            strName => $$oParam{strName},
+            strName => $bStandby ? HOST_DB_STANDBY : HOST_DB_MASTER,
             strImage => $$oParam{strImage},
-            strUser => $$oParam{strUser},
-            strVm => $$oParam{strVm},
+            strUser => $oHostGroup->paramGet(HOST_DB_USER),
+            strVm => $oHostGroup->paramGet(HOST_PARAM_VM),
+            strBackupDestination => $$oParam{strBackupDestination},
             oLogTest => $$oParam{oLogTest},
             bSynthetic => $$oParam{bSynthetic},
-            oHostBackup => $$oParam{oHostBackup},
         });
     bless $self, $class;
 
     # Set parameters
+    $self->{bStandby} = $bStandby;
+
     $self->paramSet(HOST_PARAM_DB_PATH, $self->testPath() . '/' . HOST_PATH_DB);
     $self->paramSet(HOST_PARAM_DB_BASE_PATH, $self->dbPath() . '/' . HOST_PATH_DB_BASE);
     $self->paramSet(HOST_PARAM_TABLESPACE_PATH, $self->dbPath() . '/tablespace');
 
     filePathCreate($self->dbBasePath(), undef, undef, true);
 
-    if (defined($$oParam{oHostBackup}))
+    if ($$oParam{strBackupDestination} ne $self->nameGet())
     {
-        $self->paramSet(HOST_PARAM_REPO_PATH, $$oParam{oHostBackup}->repoPath());
         $self->paramSet(HOST_PARAM_SPOOL_PATH, $self->testPath() . '/' . HOST_PATH_SPOOL);
         $self->paramSet(HOST_PARAM_LOG_PATH, $self->spoolPath() . '/' . HOST_PATH_LOG);
         $self->paramSet(HOST_PARAM_LOCK_PATH, $self->spoolPath() . '/' . HOST_PATH_LOCK);
@@ -115,6 +121,9 @@ sub new
         $self->paramSet(HOST_PARAM_SPOOL_PATH, $self->repoPath());
     }
 
+    # Initialize linkRemap Hashes
+    $self->{hLinkRemap} = {};
+
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -138,33 +147,41 @@ sub archivePush
         $strArchiveTestFile,
         $iArchiveNo,
         $iExpectedError,
+        $bAsync,
     ) =
         logDebugParam
         (
             __PACKAGE__ . '->archivePush', \@_,
             {name => 'strXlogPath'},
-            {name => 'strArchiveTestFile'},
-            {name => 'iArchiveNo'},
+            {name => 'strArchiveTestFile', required => false},
+            {name => 'iArchiveNo', required => false},
             {name => 'iExpectedError', required => false},
+            {name => 'bAsync', default => true},
         );
 
-    my $strSourceFile = "${strXlogPath}/" . uc(sprintf('0000000100000001%08x', $iArchiveNo));
+    my $strSourceFile;
 
-    $self->{oFile}->copy(
-        PATH_DB_ABSOLUTE, $strArchiveTestFile,                      # Source file
-        PATH_DB_ABSOLUTE, $strSourceFile,                           # Destination file
-        false,                                                      # Source is not compressed
-        false,                                                      # Destination is not compressed
-        undef, undef, undef,                                        # Unused params
-        true);                                                      # Create path if it does not exist
+    if (defined($strArchiveTestFile))
+    {
+        $strSourceFile = "${strXlogPath}/" . uc(sprintf('0000000100000001%08x', $iArchiveNo));
+
+        $self->{oFile}->copy(
+            PATH_DB_ABSOLUTE, $strArchiveTestFile,                      # Source file
+            PATH_DB_ABSOLUTE, $strSourceFile,                           # Destination file
+            false,                                                      # Source is not compressed
+            false,                                                      # Destination is not compressed
+            undef, undef, undef,                                        # Unused params
+            true);                                                      # Create path if it does not exist
+    }
 
     $self->executeSimple(
         $self->backrestExe() .
         ' --config=' . $self->backrestConfig() .
         ' --archive-max-mb=24 --no-fork --stanza=' . $self->stanza() .
         (defined($iExpectedError) && $iExpectedError == ERROR_HOST_CONNECT ? ' --backup-host=bogus' : '') .
-        " archive-push ${strSourceFile}",
-        {iExpectedExitStatus => $iExpectedError, oLogTest => $self->{oLogTest}});
+        ($bAsync ? '' : ' --no-archive-async') .
+        " archive-push" . (defined($strSourceFile) ? " ${strSourceFile}" : ''),
+        {iExpectedExitStatus => $iExpectedError, oLogTest => $self->{oLogTest}, bLogOutput => $self->synthetic()});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
@@ -223,7 +240,6 @@ sub configRecovery
 sub configRemap
 {
     my $self = shift;
-    my $oHostBackup = shift;
     my $oRemapHashRef = shift;
     my $oManifestRef = shift;
 
@@ -236,8 +252,11 @@ sub configRemap
 
     # Load backup config file
     my %oRemoteConfig;
+    my $oHostBackup =
+        !$self->standby() && !$self->nameTest($self->backupDestination()) ?
+            hostGroupGet()->hostGet($self->backupDestination()) : undef;
 
-    if ($oHostBackup->nameTest(HOST_BACKUP))
+    if (defined($oHostBackup))
     {
         iniLoad($oHostBackup->backrestConfig(), \%oRemoteConfig, true);
     }
@@ -252,12 +271,13 @@ sub configRemap
 
         if ($strRemap eq MANIFEST_TARGET_PGDATA)
         {
-            $oConfig{$strStanza}{'db-path'} = $strRemapPath;
+            $oConfig{$strStanza}{&OPTION_DB_PATH} = $strRemapPath;
             ${$oManifestRef}{&MANIFEST_SECTION_BACKUP_TARGET}{&MANIFEST_TARGET_PGDATA}{&MANIFEST_SUBKEY_PATH} = $strRemapPath;
 
-            if ($oHostBackup->nameTest(HOST_BACKUP))
+            if (defined($oHostBackup))
             {
-                $oRemoteConfig{$strStanza}{'db-path'} = $strRemapPath;
+                my $bForce = $oHostBackup->nameTest(HOST_BACKUP) && defined(hostGroupGet()->hostGet(HOST_DB_STANDBY, true));
+                $oRemoteConfig{$strStanza}{optionIndex(OPTION_DB_PATH, 1, $bForce)} = $strRemapPath;
             }
         }
         else
@@ -278,13 +298,40 @@ sub configRemap
     # Save db config file
     iniSave($self->backrestConfig(), \%oConfig, true);
 
-    # Save backup config file
-    if ($oHostBackup->nameTest(HOST_BACKUP))
+    # Save backup config file (but not is this is the standby which is not the source of backups)
+    if (defined($oHostBackup))
     {
         iniSave($oHostBackup->backrestConfig(), \%oRemoteConfig, true);
     }
 }
 
+####################################################################################################################################
+# linkRemap
+####################################################################################################################################
+sub linkRemap
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $strTarget,
+        $strDestination
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->linkRemap', \@_,
+            {name => 'strTarget'},
+            {name => 'strDestination'},
+        );
+
+    ${$self->{hLinkRemap}}{$strTarget} = $strDestination;
+
+    # Return from function and log return values if any
+    return logDebugReturn($strOperation);
+}
+
 ####################################################################################################################################
 # restore
 ####################################################################################################################################
@@ -312,6 +359,14 @@ sub restore
     $bDelta = defined($bDelta) ? $bDelta : false;
     $bForce = defined($bForce) ? $bForce : false;
 
+    # Build link map options
+    my $strLinkMap;
+
+    foreach my $strTarget (sort(keys(%{$self->{hLinkRemap}})))
+    {
+        $strLinkMap .= " --link-map=\"${strTarget}=${$self->{hLinkRemap}}{$strTarget}\"";
+    }
+
     $strComment = 'restore' .
                   ($bDelta ? ' delta' : '') .
                   ($bForce ? ', force' : '') .
@@ -332,22 +387,61 @@ sub restore
     my $oHostGroup = hostGroupGet();
     my $oHostBackup = defined($oHostGroup->hostGet(HOST_BACKUP, true)) ? $oHostGroup->hostGet(HOST_BACKUP) : $self;
 
+    # Load the expected manifest if it was not defined
+    my $oExpectedManifest = undef;
+
     if (!defined($oExpectedManifestRef))
     {
-        # Change mode on the backup path so it can be read
+        # Load the manifest
         my $oExpectedManifest = new pgBackRest::Manifest(
             $self->{oFile}->pathGet(
-                PATH_BACKUP_CLUSTER,
-                ($strBackup eq 'latest' ? $oHostBackup->backupLast() : $strBackup) . '/' . FILE_MANIFEST),
-                true);
+                PATH_BACKUP_CLUSTER, ($strBackup eq 'latest' ? $oHostBackup->backupLast() : $strBackup) . '/' . FILE_MANIFEST),
+            true);
 
         $oExpectedManifestRef = $oExpectedManifest->{oContent};
+
+        # Remap links in the expected manifest
+        foreach my $strTarget (sort(keys(%{$self->{hLinkRemap}})))
+        {
+            my $strDestination = ${$self->{hLinkRemap}}{$strTarget};
+            my $strTarget = 'pg_data/' . $strTarget;
+            my $strTargetPath = $strDestination;
+
+            # If this link is to a file then the specified path must be split into file and path parts
+            if ($oExpectedManifest->isTargetFile($strTarget))
+            {
+                $strTargetPath = dirname($strTargetPath);
+
+                # Error when the path is not deep enough to be valid
+                if (!defined($strTargetPath))
+                {
+                    confess &log(ERROR, "${strDestination} is not long enough to be target for ${strTarget}");
+                }
+
+                # Set the file part
+                $oExpectedManifest->set(
+                    MANIFEST_SECTION_BACKUP_TARGET, $strTarget, MANIFEST_SUBKEY_FILE,
+                    substr($strDestination, length($strTargetPath) + 1));
+
+                # Set the link target
+                $oExpectedManifest->set(
+                    MANIFEST_SECTION_TARGET_LINK, $strTarget, MANIFEST_SUBKEY_DESTINATION, $strDestination);
+            }
+            else
+            {
+                # Set the link target
+                $oExpectedManifest->set(MANIFEST_SECTION_TARGET_LINK, $strTarget, MANIFEST_SUBKEY_DESTINATION, $strTargetPath);
+            }
+
+            # Set the target path
+            $oExpectedManifest->set(MANIFEST_SECTION_BACKUP_TARGET, $strTarget, MANIFEST_SUBKEY_PATH, $strTargetPath);
+        }
     }
 
     # Get the backup host
     if (defined($oRemapHashRef))
     {
-        $self->configRemap($oHostBackup, $oRemapHashRef, $oExpectedManifestRef);
+        $self->configRemap($oRemapHashRef, $oExpectedManifestRef);
     }
 
     if (defined($oRecoveryHashRef))
@@ -367,16 +461,18 @@ sub restore
         (defined($strTarget) ? " --target=\"${strTarget}\"" : '') .
         (defined($strTargetTimeline) ? " --target-timeline=\"${strTargetTimeline}\"" : '') .
         (defined($bTargetExclusive) && $bTargetExclusive ? ' --target-exclusive' : '') .
+        (defined($strLinkMap) ? $strLinkMap : '') .
         ($self->synthetic() ? '' : ' --link-all') .
         (defined($strTargetAction) && $strTargetAction ne OPTION_DEFAULT_RESTORE_TARGET_ACTION
             ? ' --' . OPTION_TARGET_ACTION . "=${strTargetAction}" : '') .
         ' --stanza=' . $self->stanza() . ' restore',
-        {strComment => $strComment, iExpectedExitStatus => $iExpectedExitStatus, oLogTest => $self->{oLogTest}},
+        {strComment => $strComment, iExpectedExitStatus => $iExpectedExitStatus, oLogTest => $self->{oLogTest},
+         bLogOutput => $self->synthetic()},
         $strUser);
 
     if (!defined($iExpectedExitStatus))
     {
-        $self->restoreCompare($strBackup, $oExpectedManifestRef, $bTablespace);
+        $self->restoreCompare($strBackup, dclone($oExpectedManifestRef), $bTablespace);
 
         if (defined($self->{oLogTest}))
         {
@@ -445,7 +541,7 @@ sub restoreCompare
                 my $iTablespaceId =
                     ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_TARGET}{$strTarget}{&MANIFEST_SUBKEY_TABLESPACE_ID};
 
-                $$oTablespaceMap{oid}{$iTablespaceId}{name} =
+                $$oTablespaceMap{$iTablespaceId} =
                     ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_TARGET}{$strTarget}{&MANIFEST_SUBKEY_TABLESPACE_NAME};
             }
         }
@@ -485,13 +581,55 @@ sub restoreCompare
         MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_CATALOG, undef,
         $$oExpectedManifestRef{&MANIFEST_SECTION_BACKUP_DB}{&MANIFEST_KEY_CATALOG});
 
-    my $oTablespaceMapRef = undef;
-    $oActualManifest->build($self->{oFile}, $strDbClusterPath, $oLastManifest, false, $oTablespaceMap);
+    $oActualManifest->build(
+        $self->{oFile}, ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_DB}{&MANIFEST_KEY_DB_VERSION}, $strDbClusterPath,
+        $oLastManifest, false, $oTablespaceMap);
 
     my $strSectionPath = $oActualManifest->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH);
 
     foreach my $strName ($oActualManifest->keys(MANIFEST_SECTION_TARGET_FILE))
     {
+        # If synthetic match checksum errors since they can't be verified here
+        if ($self->synthetic)
+        {
+            my $bChecksumPage = $oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_CHECKSUM_PAGE};
+
+            if (defined($bChecksumPage))
+            {
+                $oActualManifest->boolSet(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE, $bChecksumPage);
+
+                if (!$bChecksumPage &&
+                    defined($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR}))
+                {
+                    $oActualManifest->set(
+                        MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR,
+                        $oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_CHECKSUM_PAGE_ERROR});
+                }
+            }
+        }
+        # Else if page checksums are enabled make sure the correct files are being checksummed
+        else
+        {
+            if ($oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_CHECKSUM_PAGE})
+            {
+                if (defined($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_CHECKSUM_PAGE}) !=
+                    isChecksumPage($strName))
+                {
+                    confess
+                        "check-page actual for ${strName} is " .
+                        ($oActualManifest->test(MANIFEST_SECTION_TARGET_FILE, $strName,
+                            MANIFEST_SUBKEY_CHECKSUM_PAGE) ? 'set' : '[undef]') .
+                        ' but isChecksumPage() says it should be ' .
+                        (isChecksumPage($strName) ? 'set' : '[undef]') . '.';
+                }
+
+                # Because the page checksum flag is copied to incr and diff from the previous backup but further processing is not
+                # done, they can't be expected to match so delete them.
+                delete($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_CHECKSUM_PAGE});
+                $oActualManifest->remove(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_CHECKSUM_PAGE);
+            }
+        }
+
         if (!$self->synthetic())
         {
             $oActualManifest->set(
@@ -499,12 +637,9 @@ sub restoreCompare
                 ${$oExpectedManifestRef}{&MANIFEST_SECTION_TARGET_FILE}{$strName}{size});
         }
 
-        if (defined(${$oExpectedManifestRef}{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_REPO_SIZE}))
-        {
-            $oActualManifest->numericSet(
-                MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_REPO_SIZE,
-                ${$oExpectedManifestRef}{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_REPO_SIZE});
-        }
+        # Remove repo-size from the manifest.  ??? This could be improved to get actual sizes from the backup.
+        $oActualManifest->remove(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_REPO_SIZE);
+        delete($oExpectedManifestRef->{&MANIFEST_SECTION_TARGET_FILE}{$strName}{&MANIFEST_SUBKEY_REPO_SIZE});
 
         if ($oActualManifest->get(MANIFEST_SECTION_TARGET_FILE, $strName, MANIFEST_SUBKEY_SIZE) != 0)
         {
@@ -536,6 +671,8 @@ sub restoreCompare
                           ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_CHECK});
     $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_ARCHIVE_COPY, undef,
                           ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_COPY});
+    $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BACKUP_STANDBY, undef,
+                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BACKUP_STANDBY});
     $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS, undef,
                           ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS});
     $oActualManifest->set(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_HARDLINK, undef,
@@ -556,21 +693,36 @@ sub restoreCompare
 
     $oActualManifest->set(INI_SECTION_BACKREST, INI_KEY_VERSION, undef,
                           ${$oExpectedManifestRef}{&INI_SECTION_BACKREST}{&INI_KEY_VERSION});
-    $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_COPY_START, undef,
-                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_COPY_START});
-    $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_START, undef,
-                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_START});
-    $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_STOP, undef,
-                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_STOP});
-    $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LABEL, undef,
-                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_LABEL});
-    $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TYPE, undef,
-                          ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TYPE});
-    $oActualManifest->set(INI_SECTION_BACKREST, INI_KEY_CHECKSUM, undef,
-                          ${$oExpectedManifestRef}{&INI_SECTION_BACKREST}{&INI_KEY_CHECKSUM});
 
-    if (!$self->synthetic())
+    # This option won't be set in the actual manifest
+    delete($oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_CHECKSUM_PAGE});
+
+    if ($self->synthetic())
     {
+        $oActualManifest->remove(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_COPY_START);
+        $oActualManifest->remove(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LABEL);
+        $oActualManifest->remove(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TYPE);
+    }
+    else
+    {
+        $oActualManifest->set(
+            INI_SECTION_BACKREST, INI_KEY_CHECKSUM, undef, $oExpectedManifestRef->{&INI_SECTION_BACKREST}{&INI_KEY_CHECKSUM});
+        $oActualManifest->set(
+            MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LABEL, undef,
+            $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_LABEL});
+        $oActualManifest->set(
+            MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_COPY_START, undef,
+            $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_COPY_START});
+        $oActualManifest->set(
+            MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_START, undef,
+            $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_START});
+        $oActualManifest->set(
+            MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_STOP, undef,
+            $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TIMESTAMP_STOP});
+        $oActualManifest->set(
+            MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TYPE, undef,
+            $oExpectedManifestRef->{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_TYPE});
+
         $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LSN_START, undef,
                               ${$oExpectedManifestRef}{&MANIFEST_SECTION_BACKUP}{&MANIFEST_KEY_LSN_START});
         $oActualManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LSN_STOP, undef,
@@ -599,6 +751,8 @@ sub restoreCompare
     # Delete the list of DBs
     delete($$oExpectedManifestRef{&MANIFEST_SECTION_DB});
 
+    $self->manifestDefault($oExpectedManifestRef);
+
     iniSave("${strTestPath}/actual.manifest", $oActualManifest->{oContent});
     iniSave("${strTestPath}/expected.manifest", $oExpectedManifestRef);
 
@@ -622,6 +776,7 @@ sub dbBasePath
 }
 
 sub spoolPath {return shift->paramGet(HOST_PARAM_SPOOL_PATH);}
+sub standby {return shift->{bStandby}}
 
 sub tablespacePath
 {
