@@ -14,12 +14,13 @@ use Carp qw(confess);
 use DBI;
 use Exporter qw(import);
     our @EXPORT = qw();
+use File::Basename qw(basename);
 
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Common::Wait;
-use pgBackRest::Db;
+use pgBackRest::DbVersion;
 use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
 use pgBackRest::Version;
@@ -27,6 +28,7 @@ use pgBackRest::Version;
 use pgBackRestTest::Backup::Common::HostBackupTest;
 use pgBackRestTest::Backup::Common::HostBaseTest;
 use pgBackRestTest::Backup::Common::HostDbCommonTest;
+use pgBackRestTest::Common::ContainerTest;
 use pgBackRestTest::Common::HostGroupTest;
 
 ####################################################################################################################################
@@ -80,11 +82,10 @@ sub new
 
     my $self = $class->SUPER::new(
         {
-            strName => HOST_DB_MASTER,
-            strImage => 'backrest/' . $oHostGroup->paramGet(HOST_PARAM_VM) . "-db-${strDbVersion}-test-pre",
-            strUser => $oHostGroup->paramGet(HOST_DB_MASTER_USER),
-            strVm => $oHostGroup->paramGet(HOST_PARAM_VM),
-            oHostBackup => $$oParam{oHostBackup},
+            strImage => containerNamespace() . '/' . $oHostGroup->paramGet(HOST_PARAM_VM) . "-db-${strDbVersion}-test-pre",
+            strBackupDestination => $$oParam{strBackupDestination},
+            oLogTest => $$oParam{oLogTest},
+            bStandby => $$oParam{bStandby},
         });
     bless $self, $class;
 
@@ -92,7 +93,7 @@ sub new
     $self->paramSet(HOST_PARAM_DB_BIN_PATH, $oHostGroup->paramGet(HOST_PARAM_DB_BIN_PATH));
     $self->paramSet(HOST_PARAM_DB_VERSION, $strDbVersion);
     $self->paramSet(HOST_PARAM_DB_SOCKET_PATH, $self->dbPath());
-    $self->paramSet(HOST_PARAM_DB_PORT, HOST_DB_PORT);
+    $self->paramSet(HOST_PARAM_DB_PORT, defined($$oParam{bStandby}) && $$oParam{bStandby} ? 6544 : 6543);
 
     $self->paramSet(HOST_PARAM_DB_LOG_PATH, $self->testPath());
     $self->paramSet(HOST_PARAM_DB_LOG_FILE, $self->dbLogPath() . '/postgresql.log');
@@ -115,7 +116,10 @@ sub new
             $strDbVersionActual =~ s/$strVersionRegExp//;
             $strDevVersion = substr($strDevVersion, length($strDbVersionActual));
 
-            &log(WARN, "Testing against ${strDbVersionActual} ${strDevVersion} version");
+            if (!defined($$oParam{bStandby}) || !$$oParam{bStandby})
+            {
+                &log(WARN, "Testing against ${strDbVersionActual} ${strDevVersion} version");
+            }
         }
 
         # Don't run unit tests for unsupported versions
@@ -132,6 +136,9 @@ sub new
         }
     }
 
+    # Create pg_xlog directory
+    filePathCreate($self->dbPath() . '/pg_xlog');
+
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -351,12 +358,25 @@ sub clusterCreate
 
     # Don't link pg_xlog for versions < 9.2 because some recovery scenarios won't work.
     $self->executeSimple(
-        $self->dbBinPath() . '/initdb' . ($self->dbVersion() >= PG_VERSION_92 ? " --xlogdir=${strXlogPath}" : '') .
+        $self->dbBinPath() . '/initdb ' .
+        ($self->dbVersion() >= PG_VERSION_93 ? ' -k' : '') .
+        ($self->dbVersion() >= PG_VERSION_92 ? " --xlogdir=${strXlogPath}" : '') .
         ' --pgdata=' . $self->dbBasePath() . ' --auth=trust');
 
+    if (!$self->standby() && $self->dbVersion() >= PG_VERSION_HOT_STANDBY)
+    {
+        $self->executeSimple(
+            "echo 'host replication replicator db-standby trust' >> " . $self->dbBasePath() . '/pg_hba.conf');
+    }
+
     $self->clusterStart(
         {bHotStandby => $$hParam{bHotStandby}, bArchive => $$hParam{bArchive}, bArchiveAlways => $$hParam{bArchiveAlways},
          bArchiveInvalid => $$hParam{bArchiveInvalid}});
+
+    if (!$self->standby() && $self->dbVersion() >= PG_VERSION_HOT_STANDBY)
+    {
+        $self->sqlExecute("create user replicator replication", {bCommit =>true});
+    }
 }
 
 ####################################################################################################################################
@@ -374,6 +394,7 @@ sub clusterStart
     my $bArchive = defined($$hParam{bArchive}) ? $$hParam{bArchive} : true;
     my $bArchiveAlways = defined($$hParam{bArchiveAlways}) ? $$hParam{bArchiveAlways} : false;
     my $bArchiveInvalid = defined($$hParam{bArchiveInvalid}) ? $$hParam{bArchiveInvalid} : false;
+    my $bArchiveEnabled = defined($$hParam{bArchiveEnabled}) ? $$hParam{bArchiveEnabled} : true;
 
     # Make sure postgres is not running
     if (-e $self->dbBasePath() . '/postmaster.pid')
@@ -391,7 +412,7 @@ sub clusterStart
         $self->dbBinPath() . '/pg_ctl start -o "-c port=' . $self->dbPort() .
         ($self->dbVersion() < PG_VERSION_95 ? ' -c checkpoint_segments=1' : '');
 
-    if ($self->dbVersion() >= PG_VERSION_83)
+    if ($bArchiveEnabled)
     {
         if ($self->dbVersion() >= PG_VERSION_95 && $bArchiveAlways)
         {
@@ -402,6 +423,10 @@ sub clusterStart
             $strCommand .= " -c archive_mode=on";
         }
     }
+    else
+    {
+        $strCommand .= " -c archive_mode=off";
+    }
 
     if ($bArchive)
     {
@@ -423,8 +448,14 @@ sub clusterStart
     }
 
     $strCommand .=
-        " -c log_error_verbosity=verbose" .
-        " -c unix_socket_director" . ($self->dbVersion() < PG_VERSION_93 ? 'y=\'' : 'ies=\'') . $self->dbPath() . '\'"' .
+        ($self->dbVersion() >= PG_VERSION_HOT_STANDBY ? ' -c max_wal_senders=3' : '') .
+        ' -c listen_addresses=\'*\'' .
+        ' -c log_directory=\'' . $self->dbLogPath() . "'" .
+        ' -c log_filename=\'' . basename($self->dbLogFile()) . "'" .
+        ' -c log_rotation_age=0' .
+        ' -c log_rotation_size=0' .
+        ' -c log_error_verbosity=verbose' .
+        ' -c unix_socket_director' . ($self->dbVersion() < PG_VERSION_93 ? 'y=\'' : 'ies=\'') . $self->dbPath() . '\'"' .
         ' -D ' . $self->dbBasePath() . ' -l ' . $self->dbLogFile() . ' -s';
 
     $self->executeSimple($strCommand);
