@@ -9,18 +9,19 @@ package pgBackRestTest::Common::ContainerTest;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess longmess);
+use English '-no_match_vars';
 
 use Cwd qw(abs_path);
 use Exporter qw(import);
     our @EXPORT = qw();
 use File::Basename qw(dirname);
 use Getopt::Long qw(GetOptions);
-use Scalar::Util qw(blessed);
 
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::FileCommon;
+use pgBackRest::Version;
 
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::VmTest;
@@ -29,19 +30,32 @@ use pgBackRestTest::Common::VmTest;
 # User/group definitions
 ####################################################################################################################################
 use constant POSTGRES_GROUP                                         => 'postgres';
-use constant POSTGRES_GROUP_ID                                      => 5000;
+    push @EXPORT, qw(POSTGRES_GROUP);
+use constant POSTGRES_GROUP_ID                                      => getgrnam(POSTGRES_GROUP) . '';
 use constant POSTGRES_USER                                          => POSTGRES_GROUP;
 use constant POSTGRES_USER_ID                                       => POSTGRES_GROUP_ID;
 
 use constant TEST_GROUP                                             => POSTGRES_GROUP;
+    push @EXPORT, qw(TEST_GROUP);
 use constant TEST_GROUP_ID                                          => POSTGRES_GROUP_ID;
-use constant TEST_USER                                              => 'vagrant';
-use constant TEST_USER_ID                                           => 1000;
+use constant TEST_USER                                              => getpwuid($UID) . '';
+    push @EXPORT, qw(TEST_USER);
+use constant TEST_USER_ID                                           => $UID;
 
 use constant BACKREST_GROUP                                         => POSTGRES_GROUP;
 use constant BACKREST_GROUP_ID                                      => POSTGRES_GROUP_ID;
 use constant BACKREST_USER                                          => 'backrest';
-use constant BACKREST_USER_ID                                       => 5001;
+use constant BACKREST_USER_ID                                       => getpwnam(BACKREST_USER) . '';
+
+####################################################################################################################################
+# Container namespace
+####################################################################################################################################
+sub containerNamespace
+{
+    return BACKREST_EXE . qw(/) . TEST_USER;
+}
+
+push @EXPORT, qw(containerNamespace);
 
 ####################################################################################################################################
 # User/group creation
@@ -156,7 +170,7 @@ sub containerWrite
     # Write the image
     fileStringWrite("${strTempPath}/${strImage}", trim($strScript) . "\n", false);
     executeTest('docker build' . (defined($bForce) && $bForce ? ' --no-cache' : '') .
-                " -f ${strTempPath}/${strImage} -t backrest/${strImage} ${strTempPath}",
+                " -f ${strTempPath}/${strImage} -t " . containerNamespace() . "/${strImage} ${strTempPath}",
                 {bSuppressStdErr => true});
 }
 
@@ -260,7 +274,7 @@ sub perlInstall
     elsif ($strOS eq VM_CO7)
     {
         return $strImage .
-            "RUN yum install -y perl perl-Thread-Queue perl-JSON-PP perl-Digest-SHA perl-DBD-Pg";
+            "RUN yum install -y perl perl-JSON-PP perl-Digest-SHA perl-DBD-Pg";
     }
     elsif ($strOS eq VM_U12 || $strOS eq VM_U14)
     {
@@ -292,7 +306,7 @@ sub containerBuild
     # Remove old images on force
     if ($bVmForce)
     {
-        my $strRegExp = '^backrest\/';
+        my $strRegExp = '^' . containerNamespace() . '/';
 
         if ($strVm ne 'all')
         {
@@ -353,26 +367,26 @@ sub containerBuild
         {
             $strScript .=
                 "RUN rpm -ivh http://yum.postgresql.org/9.0/redhat/rhel-6-x86_64/pgdg-centos90-9.0-5.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.1/redhat/rhel-6-x86_64/pgdg-centos91-9.1-4.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.2/redhat/rhel-6-x86_64/pgdg-centos92-9.2-6.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.3/redhat/rhel-6-x86_64/pgdg-centos93-9.3-1.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.4/redhat/rhel-6-x86_64/pgdg-centos94-9.4-1.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.5/redhat/rhel-6-x86_64/pgdg-centos95-9.5-2.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.6/redhat/rhel-6-x86_64/pgdg-centos96-9.6-1.noarch.rpm";
+                "RUN rpm -ivh http://yum.postgresql.org/9.1/redhat/rhel-6-x86_64/pgdg-centos91-9.1-6.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.2/redhat/rhel-6-x86_64/pgdg-centos92-9.2-8.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.3/redhat/rhel-6-x86_64/pgdg-centos93-9.3-3.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.4/redhat/rhel-6-x86_64/pgdg-centos94-9.4-3.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.5/redhat/rhel-6-x86_64/pgdg-centos95-9.5-3.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.6/redhat/rhel-6-x86_64/pgdg-centos96-9.6-3.noarch.rpm";
         }
         elsif ($strOS eq VM_CO7)
         {
             $strScript .=
-                "RUN rpm -ivh http://yum.postgresql.org/9.3/redhat/rhel-7-x86_64/pgdg-centos93-9.3-1.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.4/redhat/rhel-7-x86_64/pgdg-centos94-9.4-1.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.5/redhat/rhel-7-x86_64/pgdg-centos95-9.5-1.noarch.rpm\n" .
-                "RUN rpm -ivh http://yum.postgresql.org/9.6/redhat/rhel-7-x86_64/pgdg-centos96-9.6-1.noarch.rpm";
+                "RUN rpm -ivh http://yum.postgresql.org/9.3/redhat/rhel-7-x86_64/pgdg-centos93-9.3-3.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.4/redhat/rhel-7-x86_64/pgdg-centos94-9.4-3.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.5/redhat/rhel-7-x86_64/pgdg-centos95-9.5-3.noarch.rpm\n" .
+                "RUN rpm -ivh http://yum.postgresql.org/9.6/redhat/rhel-7-x86_64/pgdg-centos96-9.6-3.noarch.rpm";
         }
         elsif ($$oVm{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_DEBIAN)
         {
             $strScript .=
                 "RUN echo 'deb http://apt.postgresql.org/pub/repos/apt/ " .
-                $$oVm{$strOS}{&VM_OS_REPO} . "-pgdg main 9.6' >> /etc/apt/sources.list.d/pgdg.list\n" .
+                $$oVm{$strOS}{&VM_OS_REPO} . "-pgdg main' >> /etc/apt/sources.list.d/pgdg.list\n" .
                 "RUN wget --quiet -O - https://www.postgresql.org/media/keys/ACCC4CF8.asc | apt-key add -\n" .
                 "RUN apt-get update";
         }
@@ -434,7 +448,7 @@ sub containerBuild
 
         # Base pre image
         ###########################################################################################################################
-        $strImageParent = "backrest/${strOS}-base";
+        $strImageParent = containerNamespace() . "/${strOS}-base";
         $strImage = "${strOS}-base-pre";
 
         # Install Perl packages
@@ -444,6 +458,26 @@ sub containerBuild
         # Write the image
         containerWrite($strTempPath, $strOS, 'Base (Pre-Installed)', $strImageParent, $strImage, $strScript, $bVmForce, false);
 
+        # Build image
+        ###########################################################################################################################
+        $strImageParent = containerNamespace() . "/${strOS}-base-pre";
+        $strImage = "${strOS}-build";
+
+        # Install Perl packages
+        if ($$oVm{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_DEBIAN)
+        {
+            $strScript =
+                "RUN apt-get install -y gcc make\n";
+        }
+        else
+        {
+            $strScript =
+                "RUN yum install -y gcc make perl-ExtUtils-MakeMaker perl-Test-Simple\n";
+        }
+
+        # Write the image
+        containerWrite($strTempPath, $strOS, 'Build', $strImageParent, $strImage, $strScript, $bVmForce, false);
+
         # Db image
         ###########################################################################################################################
         my @stryDbBuild;
@@ -478,7 +512,7 @@ sub containerBuild
 
             my $bDocBuildVersion = ($bDocBuild && grep(/^$strDbVersion$/, @{$$oOS{&VM_DB_DOC}}));
 
-            $strImageParent = "backrest/${strOS}-base";
+            $strImageParent = containerNamespace() . "/${strOS}-base";
             $strImage = "${strOS}-db-${strDbVersion}";
 
             # Create PostgreSQL User
@@ -508,7 +542,7 @@ sub containerBuild
             ########################################################################################################################
             if ($bDocBuildVersion)
             {
-                $strImageParent = "backrest/${strOS}-db-${strDbVersion}";
+                $strImageParent = containerNamespace() . "/${strOS}-db-${strDbVersion}";
                 $strImage = "${strOS}-db-${strDbVersion}-doc";
 
                 # Install SSH key
@@ -523,7 +557,7 @@ sub containerBuild
 
             # Db test image
             ########################################################################################################################
-            $strImageParent = "backrest/${strOS}-db-${strDbVersion}";
+            $strImageParent = containerNamespace() . "/${strOS}-db-${strDbVersion}";
             $strImage = "${strOS}-db-${strDbVersion}-test";
 
             # Install SSH key
@@ -535,7 +569,7 @@ sub containerBuild
 
         # Db test image (for sythetic tests)
         ########################################################################################################################
-        $strImageParent = "backrest/${strOS}-base";
+        $strImageParent = containerNamespace() . "/${strOS}-base";
         $strImage = "${strOS}-db-test";
 
         # Install SSH key
@@ -546,7 +580,7 @@ sub containerBuild
 
         # Loop test image
         ########################################################################################################################
-        $strImageParent = "backrest/${strOS}-base";
+        $strImageParent = containerNamespace() . "/${strOS}-base";
         $strImage = "${strOS}-loop-test";
 
         # Create BackRest User
@@ -570,7 +604,7 @@ sub containerBuild
 
         # Backup image
         ###########################################################################################################################
-        $strImageParent = "backrest/${strOS}-base";
+        $strImageParent = containerNamespace() . "/${strOS}-base";
         $strImage = "${strOS}-backup";
         my $strTitle = "Backup";
 
@@ -589,7 +623,7 @@ sub containerBuild
         ###########################################################################################################################
         if ($bDocBuild)
         {
-            $strImageParent = "backrest/${strOS}-backup";
+            $strImageParent = containerNamespace() . "/${strOS}-backup";
             $strImage = "${strOS}-backup-doc";
 
             # Create configuration file
@@ -608,7 +642,7 @@ sub containerBuild
 
         # Backup Test image
         ###########################################################################################################################
-        $strImageParent = "backrest/${strOS}-backup";
+        $strImageParent = containerNamespace() . "/${strOS}-backup";
         $strImage = "${strOS}-backup-test";
 
         # Make test user home readable
@@ -641,6 +675,8 @@ sub containerRemove
     }
 }
 
+push @EXPORT, qw(containerRemove);
+
 ####################################################################################################################################
 # imageRemove
 #
@@ -660,6 +696,4 @@ sub imageRemove
     }
 }
 
-push @EXPORT, qw(containerRemove);
-
 1;
