@@ -21,16 +21,6 @@ use pgBackRest::Common::Log;
 use pgBackRest::Common::Wait;
 use pgBackRest::Protocol::IO;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_EXECUTE_TEST                                        => 'ExecuteTest';
-
-use constant OP_EXECUTE_TEST_BEGIN                                  => OP_EXECUTE_TEST . "->begin";
-use constant OP_EXECUTE_TEST_END                                    => OP_EXECUTE_TEST . "->end";
-use constant OP_EXECUTE_TEST_END_RETRY                              => OP_EXECUTE_TEST . "->endRetry";
-use constant OP_EXECUTE_TEST_NEW                                    => OP_EXECUTE_TEST . "->new";
-
 ####################################################################################################################################
 # new
 ####################################################################################################################################
@@ -50,7 +40,7 @@ sub new
     ) =
         logDebugParam
         (
-            OP_EXECUTE_TEST_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'strCommand'},
             {name => 'oParam', required => false}
         );
@@ -68,6 +58,7 @@ sub new
     $self->{bShowOutputAsync} = defined($self->{bShowOutputAsync}) ? $self->{bShowOutputAsync} : false;
     $self->{iExpectedExitStatus} = defined($self->{iExpectedExitStatus}) ? $self->{iExpectedExitStatus} : 0;
     $self->{iRetrySeconds} = defined($self->{iRetrySeconds}) ? $self->{iRetrySeconds} : undef;
+    $self->{bLogOutput} = defined($self->{bLogOutput}) ? $self->{bLogOutput} : true;
 
     # Return from function and log return values if any
     return logDebugReturn
@@ -85,7 +76,7 @@ sub begin
     my $self = shift;
 
     # Assign function parameters, defaults, and log debug info
-    logDebugParam(OP_EXECUTE_TEST_BEGIN);
+    my ($strOperation) = logDebugParam(__PACKAGE__ . '->begin');
 
     $self->{strErrorLog} = '';
     $self->{strOutLog} = '';
@@ -114,6 +105,8 @@ sub begin
     {
         confess 'STDERR handle is undefined';
     }
+
+    return logDebugReturn($strOperation);
 }
 
 ####################################################################################################################################
@@ -132,7 +125,7 @@ sub endRetry
     ) =
         logDebugParam
         (
-            OP_EXECUTE_TEST_END_RETRY, \@_,
+            __PACKAGE__ . '->endRetry', \@_,
             {name => 'strTest', required => false, trace => true},
             {name => 'bWait', required => false, default => true, trace => true}
         );
@@ -202,7 +195,12 @@ sub endRetry
     # Pass the log to the LogTest object
     if (defined($self->{oLogTest}))
     {
-        $self->{oLogTest}->logAdd($self->{strCommand}, $self->{strComment}, $self->{strOutLog});
+        if (defined($self->{strErrorLog}) && $self->{strErrorLog} ne '')
+        {
+            $self->{strOutLog} .= "STDERR:\n" . $self->{strErrorLog};
+        }
+
+        $self->{oLogTest}->logAdd($self->{strCommand}, $self->{strComment}, $self->{bLogOutput} ? $self->{strOutLog} : undef);
     }
 
     # If an error was expected then return success if that error occurred
@@ -211,17 +209,6 @@ sub endRetry
         return $iExitStatus;
     }
 
-    # This is a hack to make regression tests pass even when threaded backup/restore sometimes return 255
-    if ($self->{iExpectedExitStatus} == -1)
-    {
-        if ($iExitStatus == 0 || $iExitStatus == 255)
-        {
-            return 0;
-        }
-
-        $self->{iExpectedExitStatus}  = 0;
-    }
-
     if ($iExitStatus != 0 || ($self->{iExpectedExitStatus} != 0 && $iExitStatus != $self->{iExpectedExitStatus}))
     {
         if ($self->{bSuppressError})
@@ -240,8 +227,8 @@ sub endRetry
             {
                 confess &log(ERROR, "command '$self->{strCommand}' returned " . $iExitStatus .
                              ($self->{iExpectedExitStatus} != 0 ? ", but $self->{iExpectedExitStatus} was expected" : '') . "\n" .
-                             ($self->{strOutLog} ne '' ? "STDOUT (last 10,000 characters):\n" . substr($self->{strOutLog},
-                                 length($self->{strOutLog}) - 10000) : '') .
+                             ($self->{strOutLog} ne '' ? "STDOUT (last 20,000 characters):\n" . substr($self->{strOutLog},
+                                 length($self->{strOutLog}) - 20000) : '') .
                              ($self->{strErrorLog} ne '' ? "STDERR:\n$self->{strErrorLog}" : ''));
             }
         }
@@ -262,7 +249,12 @@ sub endRetry
         confess &log(ASSERT, "test point ${strTest} was not found");
     }
 
-    return $iExitStatus;
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'iExitStatus', value => $iExitStatus, trace => true}
+    );
 }
 
 ####################################################################################################################################
@@ -281,7 +273,7 @@ sub end
     ) =
         logDebugParam
         (
-            OP_EXECUTE_TEST_END, \@_,
+            __PACKAGE__ . '->end', \@_,
             {name => 'strTest', required => false, trace => true},
             {name => 'bWait', required => false, default => true, trace => true}
         );
@@ -318,7 +310,12 @@ sub end
         }
     }
 
-    return $iExitStatus;
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'iExitStatus', value => $iExitStatus, trace => true}
+    );
 }
 
 ####################################################################################################################################
