@@ -49,7 +49,7 @@ sub new
             {name => 'strImage', trace => true},
             {name => 'strUser', trace => true},
             {name => 'strOS', trace => true},
-            {name => 'stryMount', trace => true}
+            {name => 'stryMount', required => false, trace => true}
         );
 
     executeTest("docker rm -f $self->{strContainer}", {bSuppressError => true});
@@ -58,6 +58,7 @@ sub new
                 (defined($self->{stryMount}) ? ' -v ' . join(' -v ', @{$self->{stryMount}}) : '') .
                 " $self->{strImage}");
 
+    # Get IP Address
     $self->{strIP} = trim(executeTest("docker inspect --format '\{\{ .NetworkSettings.IPAddress \}\}' $self->{strContainer}"));
     $self->{bActive} = true;
 
@@ -233,6 +234,16 @@ sub copyFrom
     return logDebugReturn($strOperation);
 }
 
+####################################################################################################################################
+# ipGet
+####################################################################################################################################
+sub ipGet
+{
+    my $self = shift;
+
+    return $self->{strIP};
+}
+
 ####################################################################################################################################
 # nameGet
 ####################################################################################################################################
