@@ -19,17 +19,9 @@ use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Version;
 
+use pgBackRestTest::Common::ContainerTest;
 use pgBackRestTest::Common::ExecuteTest;
 
-####################################################################################################################################
-# Operation constants
-####################################################################################################################################
-use constant OP_LOG_TEST                                            => 'LogTest';
-
-use constant OP_LOG_TEST_LOG_ADD                                    => OP_LOG_TEST . "->logAdd";
-use constant OP_LOG_TEST_LOG_WRITE                                  => OP_LOG_TEST . "->logWrite";
-use constant OP_LOG_TEST_NEW                                        => OP_LOG_TEST . "->new";
-
 ####################################################################################################################################
 # new
 ####################################################################################################################################
@@ -56,7 +48,7 @@ sub new
     ) =
         logDebugParam
         (
-            OP_LOG_TEST_NEW, \@_,
+            __PACKAGE__ . '->new', \@_,
             {name => 'strModule', trace => true},
             {name => 'strTest', trace => true},
             {name => 'iRun', trace => true},
@@ -100,7 +92,7 @@ sub logAdd
     ) =
         logDebugParam
         (
-            OP_LOG_TEST_LOG_ADD, \@_,
+            __PACKAGE__ . '->logAdd', \@_,
             {name => 'strCommand', trace => true},
             {name => 'strComment', required => false, trace => true},
             {name => 'strLog', required => false, trace => true}
@@ -121,7 +113,7 @@ sub logAdd
         # Do replacements on each line of the log
         foreach my $strLine (split("\n", $strLog))
         {
-            $strLine =~ s/^[0-9]{4}-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-6][0-9]:[0-6][0-9]\.[0-9]{3} T[0-9]{2} //;
+            $strLine =~ s/^[0-9]{4}-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-6][0-9]:[0-6][0-9]\.[0-9]{3} //;
 
             if ($strLine !~ /^  TEST/)
             {
@@ -185,7 +177,7 @@ sub logWrite
     ) =
         logDebugParam
         (
-            OP_LOG_TEST_LOG_WRITE, \@_,
+            __PACKAGE__ . '->logWrite', \@_,
             {name => 'strBasePath', trace => true},
             {name => 'strTestPath', trace => true},
             {name => 'strFileName',
@@ -257,8 +249,17 @@ sub regExpReplace
 
                 if (@stryReplacement != 1)
                 {
-                    confess &log(ASSERT, "'${strToken}' is not a sub-regexp of '${strExpression}' or" .
-                                         " matches multiple times on '${strReplace}'");
+                    my $strError = "'${strToken}'";
+
+                    if (@stryReplacement == 0)
+                    {
+                        confess &log(ASSERT, $strError . " is not a sub-regexp of '${strExpression}' or" .
+                                             " matches " . @stryReplacement . " times on {[${strReplace}]}");
+                    }
+
+                    confess &log(
+                        ASSERT, $strError . " matches '${strExpression}' " . @stryReplacement . " times on '${strReplace}': " .
+                        join(',', @stryReplacement));
                 }
 
                 $strReplacement = $stryReplacement[0];
@@ -329,16 +330,15 @@ sub regExpReplaceAll
         $strLine =~ s/$self->{strPgSqlBin}/[PGSQL_BIN_PATH]/g;
     }
 
-    $strLine = $self->regExpReplace($strLine, 'BACKREST_NAME_VERSION', '^' . BACKREST_NAME . ' ' . BACKREST_VERSION,
+    $strLine = $self->regExpReplace($strLine, 'BACKREST-NAME-VERSION', '^' . BACKREST_NAME . ' ' . BACKREST_VERSION,
                                                 undef, false);
+    $strLine = $self->regExpReplace(
+        $strLine, 'BACKREST-VERSION', ' begin ' . BACKREST_VERSION . '\:', BACKREST_VERSION, false);
 
     $strLine = $self->regExpReplace($strLine, undef, '^docker exec -u [a-z]* test-[0-9]+\-', 'test-[0-9]+\-', false);
     $strLine = $self->regExpReplace($strLine, 'CONTAINER-EXEC', '^docker exec -u [a-z]*', '^docker exec -u [a-z]*', false);
 
-    $strLine = $self->regExpReplace($strLine, 'PROCESS-ID', 'process [0-9]+', '[0-9]+$', false);
-    $strLine = $self->regExpReplace($strLine, 'MODIFICATION-TIME', 'lModificationTime = [0-9]+', '[0-9]+$');
-    $strLine = $self->regExpReplace($strLine, 'MODIFICATION-TIME', 'and modification time [0-9]+', '[0-9]+$');
-    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', 'timestamp"[ ]{0,1}:[ ]{0,1}[0-9]+','[0-9]+$');
+    $strLine = $self->regExpReplace($strLine, 'PROCESS-ID', 'sent term signal to process [0-9]+', '[0-9]+$', false);
 
     $strLine = $self->regExpReplace($strLine, 'BACKUP-INCR', '[0-9]{8}\-[0-9]{6}F\_[0-9]{8}\-[0-9]{6}I');
     $strLine = $self->regExpReplace($strLine, 'BACKUP-DIFF', '[0-9]{8}\-[0-9]{6}F\_[0-9]{8}\-[0-9]{6}D');
@@ -346,11 +346,19 @@ sub regExpReplaceAll
 
     $strLine = $self->regExpReplace($strLine, 'GROUP', 'strGroup = [^ \n,\[\]]+', '[^ \n,\[\]]+$');
     $strLine = $self->regExpReplace($strLine, 'GROUP', 'group"[ ]{0,1}:[ ]{0,1}"[^"]+', '[^"]+$');
+
     $strLine = $self->regExpReplace($strLine, 'USER', 'strUser = [^ \n,\[\]]+', '[^ \n,\[\]]+$');
     $strLine = $self->regExpReplace($strLine, 'USER', 'user"[ ]{0,1}:[ ]{0,1}"[^"]+', '[^"]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', 'user=\"[^"]+', '[^"]+$');
     $strLine = $self->regExpReplace($strLine, 'USER', '^db-user=.+$', '[^=]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', 'was not mapped to a name\, set to .+$', '[^ ]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', 'set ownership [^\:]+', '[^ ]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', 'cannot be used for restore\, set to .+$', '[^ ]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', '-user=[a-z0-9_]+', '[^=]+$');
+    $strLine = $self->regExpReplace($strLine, 'USER', '[^ ]+\@db\-master', '^[^\@]+');
+    $strLine = $self->regExpReplace($strLine, 'USER', '[\( ]{1}' . TEST_USER . '[\,\)]{1}', TEST_USER);
 
-    $strLine = $self->regExpReplace($strLine, 'PORT', 'db-port=[0-9]+', '[0-9]+$');
+    $strLine = $self->regExpReplace($strLine, 'PORT', 'db[1-9]{0,1}-port=[0-9]+', '[0-9]+$');
 
     # Replace year when it falls on a single line when executing ls -1R
     $strLine = $self->regExpReplace($strLine, 'YEAR', '^20[0-9]{2}$');
@@ -363,24 +371,39 @@ sub regExpReplaceAll
     $strLine = $self->regExpReplace($strLine, 'TS_PATH', "PG\\_[0-9]\\.[0-9]\\_[0-9]{9}");
     $strLine = $self->regExpReplace($strLine, 'VERSION',
         "version[\"]{0,1}[ ]{0,1}[\:\=)]{1}[ ]{0,1}[\"]{0,1}" . BACKREST_VERSION, BACKREST_VERSION . '$');
+
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', 'timestamp"[ ]{0,1}:[ ]{0,1}[0-9]+','[0-9]{10}$');
+
     $strLine = $self->regExpReplace($strLine, 'TIMESTAMP',
-        "timestamp-[a-z-]+[\"]{0,1}[ ]{0,1}[\:\=)]{1}[ ]{0,1}[\"]{0,1}[0-9]+", '[0-9]+$', false);
-    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP',
-        "start\" : [0-9]{10}", '[0-9]{10}$', false);
-    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP',
-        "stop\" : [0-9]{10}", '[0-9]{10}$', false);
-    $strLine = $self->regExpReplace($strLine, 'SIZE',
-        "size\"[ ]{0,1}:[ ]{0,1}[0-9]+", '[0-9]+$', false);
-    $strLine = $self->regExpReplace($strLine, 'DELTA',
-        "delta\"[ ]{0,1}:[ ]{0,1}[0-9]+", '[0-9]+$', false);
-    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP-STR', " timestamp: $strTimestampRegExp / $strTimestampRegExp",
-                                                "${strTimestampRegExp} / ${strTimestampRegExp}\$", false);
+        "timestamp-[a-z-]+[\"]{0,1}[ ]{0,1}[\:\=)]{1}[ ]{0,1}[\"]{0,1}[0-9]+", '[0-9]{10}$', false);
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', "start\" : [0-9]{10}", '[0-9]{10}$', false);
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', "stop\" : [0-9]{10}", '[0-9]{10}$', false);
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP', TEST_GROUP . '\, [0-9]{10}', '[0-9]{10}$', false);
+    $strLine = $self->regExpReplace($strLine, 'SIZE', "size\"[ ]{0,1}:[ ]{0,1}[0-9]+", '[0-9]+$', false);
+    $strLine = $self->regExpReplace($strLine, 'DELTA', "delta\"[ ]{0,1}:[ ]{0,1}[0-9]+", '[0-9]+$', false);
+    $strLine = $self->regExpReplace(
+        $strLine, 'TIMESTAMP-STR', " timestamp: $strTimestampRegExp / $strTimestampRegExp",
+        "${strTimestampRegExp} / ${strTimestampRegExp}\$", false);
     $strLine = $self->regExpReplace($strLine, 'CHECKSUM', 'checksum=[\"]{0,1}[0-f]{40}', '[0-f]{40}$', false);
 
     $strLine = $self->regExpReplace($strLine, 'REMOTE-PROCESS-TERMINATED-MESSAGE',
         'remote process terminated.*: (ssh.*|no output from terminated process)$',
         '(ssh.*|no output from terminated process)$', false);
 
+    # Full test time-based recovery
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP-TARGET', "\\, target \\'.*UTC", "[^\\']+UTC\$");
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP-TARGET', " \\-\\-target\\=\\\".*UTC", "[^\\\"]+UTC\$");
+    $strLine = $self->regExpReplace($strLine, 'TIMESTAMP-TARGET', "^recovery_target_time \\= \\'.*UTC", "[^\\']+UTC\$");
+
+    # Full test xid-based recovery (this expressions only work when time-based expressions above have already been applied
+    $strLine = $self->regExpReplace($strLine, 'XID-TARGET', "\\, target \\'[0-9]+", "[0-9]+\$");
+    $strLine = $self->regExpReplace($strLine, 'XID-TARGET', " \\-\\-target\\=\\\"[0-9]+", "[0-9]+\$");
+    $strLine = $self->regExpReplace($strLine, 'XID-TARGET', "^recovery_target_xid \\= \\'[0-9]+", "[0-9]+\$");
+
+    $strLine = $self->regExpReplace(
+        $strLine, 'MODIFICATION-TIME', '(' . (substr(time(), 0, 5) - 1) . '[0-9]{5}|' . substr(time(), 0, 5) . '[0-9]{5})',
+        '^[0-9]{10}$');
+
     return $strLine;
 }
 
