@@ -9,10 +9,10 @@ package pgBackRestTest::Config::ConfigTest;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use Cwd qw(abs_path);
 use Exporter qw(import);
-use Scalar::Util qw(blessed);
 
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
@@ -108,26 +108,30 @@ sub configLoadExpect
     commandSetTest($oOption, $strCommand);
     argvWriteTest($oOption);
 
+    my $bErrorFound = false;
+
     eval
     {
-        configLoad();
-    };
-
-    if ($@)
+        configLoad(false);
+        return true;
+    }
+    or do
     {
+        my $oException = $EVAL_ERROR;
+
         if (!defined($iExpectedError))
         {
-            confess $@;
+            confess $oException;
         }
 
-        my $oMessage = $@;
+        $bErrorFound = true;
 
-        if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
+        if (isException($oException))
         {
-            if ($oMessage->code() != $iExpectedError)
+            if ($oException->code() != $iExpectedError)
             {
-                confess "expected error ${iExpectedError} from configLoad but got " . $oMessage->code() .
-                        " '" . $oMessage->message() . "'";
+                confess "expected error ${iExpectedError} from configLoad but got [" . $oException->code() .
+                        "] '" . $oException->message() . "'";
             }
 
             my $strError;
@@ -151,11 +155,19 @@ sub configLoadExpect
                                  " in ('" . join("', '",@{ $strErrorParam3}) . "')";
                 }
             }
+            elsif ($iExpectedError == ERROR_OPTION_COMMAND)
+            {
+                $strError = "option '${strErrorParam1}' not valid for command '${strErrorParam2}'";
+            }
             elsif ($iExpectedError == ERROR_OPTION_INVALID_VALUE)
             {
                 $strError = "'${strErrorParam1}' is not valid for '${strErrorParam2}' option" .
                             (defined($strErrorParam3) ? "\nHINT: ${strErrorParam3}." : '');
             }
+            elsif ($iExpectedError == ERROR_OPTION_MULTIPLE_VALUE)
+            {
+                $strError = "option '${strErrorParam1}' cannot be specified multiple times";
+            }
             elsif ($iExpectedError == ERROR_OPTION_INVALID_RANGE)
             {
                 $strError = "'${strErrorParam1}' is not valid for '${strErrorParam2}' option";
@@ -174,29 +186,25 @@ sub configLoadExpect
             }
             else
             {
-                confess "must construct message for error ${iExpectedError}, use this as an example: '" . $oMessage->message() . "'";
+                confess
+                    "must construct message for error ${iExpectedError}, use this as an example: '" . $oException->message() . "'";
             }
 
-            if ($oMessage->message() ne $strError)
+            if ($oException->message() ne $strError)
             {
-                confess "expected error message \"${strError}\" from configLoad but got \"" . $oMessage->message() . "\"";
+                confess "expected error message \"${strError}\" from configLoad but got \"" . $oException->message() . "\"";
             }
         }
         else
         {
-            confess "configLoad should throw pgBackRest::Common::Exception:\n$oMessage";
+            confess "configLoad should throw pgBackRest::Common::Exception:\n$oException";
         }
-    }
-    else
-    {
-        if (defined($iExpectedError))
-        {
-            confess "expected error ${iExpectedError} from configLoad but got success";
-        }
-    }
+    };
 
-    # cmp_deeply(OPTION_rule_get(), $oOptionRuleExpected, 'compare original and new rule hashes')
-    #     or die 'comparison failed';
+    if (!$bErrorFound && defined($iExpectedError))
+    {
+        confess "expected error ${iExpectedError} from configLoad but got success";
+    }
 }
 
 sub optionTestExpect
@@ -293,6 +301,15 @@ sub configTestRun
             optionTestExpect(OPTION_TYPE, BACKUP_TYPE_INCR);
         }
 
+        if (testRun(++$iRun, CMD_BACKUP . ' invalid option ' . OPTION_ARCHIVE_ASYNC))
+        {
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionBoolSetTest($oOption, OPTION_ARCHIVE_ASYNC);
+
+            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_COMMAND, OPTION_ARCHIVE_ASYNC, CMD_BACKUP);
+        }
+
         if (testRun(++$iRun, 'backup type set to ' . BACKUP_TYPE_FULL))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -386,33 +403,43 @@ sub configTestRun
             optionTestExpect(OPTION_TARGET_TIMELINE);
         }
 
-        if (testRun(++$iRun, 'invalid string ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'invalid string ' . OPTION_PROCESS_MAX))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
             optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_THREAD_MAX, BOGUS);
+            optionSetTest($oOption, OPTION_PROCESS_MAX, BOGUS);
 
-            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_INVALID_VALUE, BOGUS, OPTION_THREAD_MAX);
+            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_INVALID_VALUE, BOGUS, OPTION_PROCESS_MAX);
         }
 
-        if (testRun(++$iRun, 'invalid float ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'invalid float ' . OPTION_PROCESS_MAX))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
             optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_THREAD_MAX, '0.0');
+            optionSetTest($oOption, OPTION_PROCESS_MAX, '0.0');
 
-            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_INVALID_VALUE, '0.0', OPTION_THREAD_MAX);
+            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_INVALID_VALUE, '0.0', OPTION_PROCESS_MAX);
         }
 
-        if (testRun(++$iRun, 'valid ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'valid ' . OPTION_PROCESS_MAX))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
             optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_THREAD_MAX, '2');
+            optionSetTest($oOption, OPTION_PROCESS_MAX, '2');
 
             configLoadExpect($oOption, CMD_BACKUP);
         }
 
+        if (testRun(++$iRun, 'valid thread-max'))
+        {
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionSetTest($oOption, 'thread-max', '2');
+
+            configLoadExpect($oOption, CMD_BACKUP);
+            optionTestExpect(OPTION_PROCESS_MAX, 2);
+        }
+
         if (testRun(++$iRun, 'valid float ' . OPTION_TEST_DELAY))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -450,25 +477,59 @@ sub configTestRun
             configLoadExpect($oOption, CMD_RESTORE, ERROR_OPTION_INVALID_RANGE, '512', OPTION_BUFFER_SIZE);
         }
 
-        if (testRun(++$iRun, CMD_BACKUP . ' invalid option ' . OPTION_RETENTION_ARCHIVE_TYPE))
-        {
-            optionSetTest($oOption, OPTION_STANZA, $strStanza);
-            optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_RETENTION_ARCHIVE_TYPE, BOGUS);
-
-            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_INVALID, OPTION_RETENTION_ARCHIVE_TYPE, OPTION_RETENTION_ARCHIVE);
-        }
-
         if (testRun(++$iRun, CMD_BACKUP . ' invalid value ' . OPTION_RETENTION_ARCHIVE_TYPE))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
             optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_RETENTION_ARCHIVE, 3);
             optionSetTest($oOption, OPTION_RETENTION_ARCHIVE_TYPE, BOGUS);
 
             configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_INVALID_VALUE, BOGUS, OPTION_RETENTION_ARCHIVE_TYPE);
         }
 
+        if (testRun(++$iRun, CMD_BACKUP . ' default value ' . OPTION_DEFAULT_RETENTION_ARCHIVE_TYPE . ' for ' .
+                    OPTION_RETENTION_ARCHIVE_TYPE . ' with valid ' . OPTION_RETENTION_FULL))
+        {
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionSetTest($oOption, OPTION_RETENTION_FULL, 1);
+
+            configLoadExpect($oOption, CMD_BACKUP);
+            optionTestExpect(OPTION_RETENTION_ARCHIVE, 1);
+            optionTestExpect(OPTION_RETENTION_FULL, 1);
+            optionTestExpect(OPTION_RETENTION_DIFF, undef);
+            # Default is FULL so this test will fail if the default is changed, alerting to the need to update configLoad.
+            optionTestExpect(OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_FULL);
+        }
+
+        if (testRun(++$iRun, CMD_BACKUP . ' valid value ' . OPTION_RETENTION_ARCHIVE . ' for ' . OPTION_RETENTION_ARCHIVE_TYPE .
+            ' ' . BACKUP_TYPE_DIFF))
+        {
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionSetTest($oOption, OPTION_RETENTION_DIFF, 1);
+            optionSetTest($oOption, OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_DIFF);
+
+            configLoadExpect($oOption, CMD_BACKUP);
+            optionTestExpect(OPTION_RETENTION_ARCHIVE, 1);
+            optionTestExpect(OPTION_RETENTION_DIFF, 1);
+            optionTestExpect(OPTION_RETENTION_FULL, undef);
+            optionTestExpect(OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_DIFF);
+        }
+
+        if (testRun(++$iRun, CMD_BACKUP . ' warn no valid value ' . OPTION_RETENTION_ARCHIVE . ' for ' . OPTION_RETENTION_ARCHIVE_TYPE .
+                    ' ' . BACKUP_TYPE_INCR))
+        {
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionSetTest($oOption, OPTION_RETENTION_FULL, 2);
+            optionSetTest($oOption, OPTION_RETENTION_DIFF, 1);
+            optionSetTest($oOption, OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_INCR);
+
+            configLoadExpect($oOption, CMD_BACKUP);
+            optionTestExpect(OPTION_RETENTION_ARCHIVE, undef);
+            optionTestExpect(OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_INCR);
+        }
+
         if (testRun(++$iRun, CMD_BACKUP . ' invalid value ' . OPTION_PROTOCOL_TIMEOUT))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -481,18 +542,6 @@ sub configTestRun
                 "'protocol-timeout' option should be greater than 'db-timeout' option");
         }
 
-        if (testRun(++$iRun, CMD_BACKUP . ' valid value ' . OPTION_RETENTION_ARCHIVE_TYPE))
-        {
-            optionSetTest($oOption, OPTION_STANZA, $strStanza);
-            optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_RETENTION_ARCHIVE, 1);
-            optionSetTest($oOption, OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_FULL);
-
-            configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_RETENTION_ARCHIVE, 1);
-            optionTestExpect(OPTION_RETENTION_ARCHIVE_TYPE, BACKUP_TYPE_FULL);
-        }
-
         if (testRun(++$iRun, CMD_RESTORE . ' invalid value ' . OPTION_RESTORE_RECOVERY_OPTION))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -549,13 +598,14 @@ sub configTestRun
             }
         }
 
-        if (testRun(++$iRun, CMD_BACKUP . ' default value ' . OPTION_COMMAND_REMOTE))
+        if (testRun(++$iRun, CMD_BACKUP . ' default value ' . OPTION_DB_CMD))
         {
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_HOST, 'db');
             optionSetTest($oOption, OPTION_DB_PATH, '/db');
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_COMMAND_REMOTE, abs_path($0));
+            optionTestExpect(OPTION_DB_CMD, abs_path($0));
         }
 
         if (testRun(++$iRun, CMD_BACKUP . ' missing option ' . OPTION_DB_PATH))
@@ -564,6 +614,16 @@ sub configTestRun
 
             configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_REQUIRED, OPTION_DB_PATH, 'does this stanza exist?');
         }
+
+        if (testRun(++$iRun, CMD_BACKUP . ' automatically increase ' . OPTION_PROTOCOL_TIMEOUT))
+        {
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionSetTest($oOption, OPTION_DB_TIMEOUT, OPTION_DEFAULT_PROTOCOL_TIMEOUT + 30);
+
+            configLoadExpect($oOption, CMD_BACKUP);
+            optionTestExpect(OPTION_PROTOCOL_TIMEOUT, OPTION_DEFAULT_PROTOCOL_TIMEOUT + 60);
+        }
     }
 
     #-------------------------------------------------------------------------------------------------------------------------------
@@ -618,10 +678,10 @@ sub configTestRun
             configLoadExpect($oOption, CMD_BACKUP, ERROR_FILE_INVALID, $strTestPath);
         }
 
-        if (testRun(++$iRun, 'load from config stanza command section - option ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'load from config stanza command section - option ' . OPTION_PROCESS_MAX))
         {
             $oConfig = {};
-            $$oConfig{"${strStanza}:" . &CMD_BACKUP}{&OPTION_THREAD_MAX} = 2;
+            $$oConfig{"${strStanza}:" . &CMD_BACKUP}{&OPTION_PROCESS_MAX} = 2;
             iniSave($strConfigFile, $oConfig, true);
 
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -629,13 +689,13 @@ sub configTestRun
             optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_THREAD_MAX, 2);
+            optionTestExpect(OPTION_PROCESS_MAX, 2);
         }
 
-        if (testRun(++$iRun, 'load from config stanza section - option ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'load from config stanza section - option ' . OPTION_PROCESS_MAX))
         {
             $oConfig = {};
-            $$oConfig{$strStanza}{&OPTION_THREAD_MAX} = 3;
+            $$oConfig{$strStanza}{&OPTION_PROCESS_MAX} = 3;
             iniSave($strConfigFile, $oConfig, true);
 
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -643,14 +703,13 @@ sub configTestRun
             optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_THREAD_MAX, 3);
+            optionTestExpect(OPTION_PROCESS_MAX, 3);
         }
 
-
-        if (testRun(++$iRun, 'load from config global command section - option ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'load from config global command section - option thread-max'))
         {
             $oConfig = {};
-            $$oConfig{&CONFIG_SECTION_GLOBAL . ':' . &CMD_BACKUP}{&OPTION_THREAD_MAX} = 2;
+            $$oConfig{&CONFIG_SECTION_GLOBAL . ':' . &CMD_BACKUP}{'thread-max'} = 2;
             iniSave($strConfigFile, $oConfig, true);
 
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -658,13 +717,13 @@ sub configTestRun
             optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_THREAD_MAX, 2);
+            optionTestExpect(OPTION_PROCESS_MAX, 2);
         }
 
-        if (testRun(++$iRun, 'load from config global section - option ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'load from config global section - option ' . OPTION_PROCESS_MAX))
         {
             $oConfig = {};
-            $$oConfig{&CONFIG_SECTION_GLOBAL}{&OPTION_THREAD_MAX} = 5;
+            $$oConfig{&CONFIG_SECTION_GLOBAL}{&OPTION_PROCESS_MAX} = 5;
             iniSave($strConfigFile, $oConfig, true);
 
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
@@ -672,10 +731,10 @@ sub configTestRun
             optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_THREAD_MAX, 5);
+            optionTestExpect(OPTION_PROCESS_MAX, 5);
         }
 
-        if (testRun(++$iRun, 'default - option ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'default - option ' . OPTION_PROCESS_MAX))
         {
             $oConfig = {};
             iniSave($strConfigFile, $oConfig, true);
@@ -685,22 +744,22 @@ sub configTestRun
             optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_THREAD_MAX, 1);
+            optionTestExpect(OPTION_PROCESS_MAX, 1);
         }
 
-        if (testRun(++$iRun, 'command-line override - option ' . OPTION_THREAD_MAX))
+        if (testRun(++$iRun, 'command-line override - option ' . OPTION_PROCESS_MAX))
         {
             $oConfig = {};
-            $$oConfig{&CONFIG_SECTION_GLOBAL}{&OPTION_THREAD_MAX} = 9;
+            $$oConfig{&CONFIG_SECTION_GLOBAL}{&OPTION_PROCESS_MAX} = 9;
             iniSave($strConfigFile, $oConfig, true);
 
             optionSetTest($oOption, OPTION_STANZA, $strStanza);
             optionSetTest($oOption, OPTION_DB_PATH, '/db');
-            optionSetTest($oOption, OPTION_THREAD_MAX, 7);
+            optionSetTest($oOption, OPTION_PROCESS_MAX, 7);
             optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
 
             configLoadExpect($oOption, CMD_BACKUP);
-            optionTestExpect(OPTION_THREAD_MAX, 7);
+            optionTestExpect(OPTION_PROCESS_MAX, 7);
         }
 
         if (testRun(++$iRun, 'invalid boolean - option ' . OPTION_HARDLINK))
@@ -888,6 +947,19 @@ sub configTestRun
             optionTestExpect(OPTION_REPO_PATH, '/repo');
         }
 
+        if (testRun(++$iRun, CMD_BACKUP . ' option ' . OPTION_REPO_PATH . ' multiple times'))
+        {
+            $oConfig = {};
+            $$oConfig{&CONFIG_SECTION_GLOBAL}{&OPTION_REPO_PATH} = ['/repo', '/repo2'];
+            iniSave($strConfigFile, $oConfig, true);
+
+            optionSetTest($oOption, OPTION_STANZA, $strStanza);
+            optionSetTest($oOption, OPTION_DB_PATH, '/db');
+            optionSetTest($oOption, OPTION_CONFIG, $strConfigFile);
+
+            configLoadExpect($oOption, CMD_BACKUP, ERROR_OPTION_MULTIPLE_VALUE, OPTION_REPO_PATH);
+        }
+
         # Cleanup
         testCleanup();
     }
