@@ -9,15 +9,16 @@ package pgBackRestTest::File::FileTest;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use Cwd qw(abs_path cwd);
 use Exporter qw(import);
 use Fcntl qw(:mode);
 use File::stat;
 use POSIX qw(ceil);
-use Scalar::Util qw(blessed);
 use Time::HiRes qw(gettimeofday usleep);
 
+use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Config::Config;
 use pgBackRest::File;
@@ -103,8 +104,10 @@ sub fileTestRun
         or confess "Unable to create repo directory: ${strRepoPath}";
 
     my $oRemote = new pgBackRest::Protocol::RemoteMaster(
+        BACKUP,
+        OPTION_DEFAULT_CMD_SSH,
         $oHostGroup->paramGet(HOST_PARAM_BACKREST_EXE) . ' --stanza=' . HOST_STANZA .
-            " --repo-path=${strRepoPath} --no-config --command=test remote",
+            " --type=backup --repo-path=${strRepoPath} --no-config --command=test remote",
         OPTION_DEFAULT_BUFFER_SIZE,
         OPTION_DEFAULT_COMPRESS_LEVEL,
         OPTION_DEFAULT_COMPRESS_LEVEL_NETWORK,
@@ -138,7 +141,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -183,10 +185,10 @@ sub fileTestRun
                 eval
                 {
                     $oFile->pathCreate($strPathType, $strPath, $strMode);
-                };
-
+                    return true;
+                }
                 # Check for errors
-                if ($@)
+                or do
                 {
                     # Ignore errors if the path did not exist
                     if ($bErrorExpected)
@@ -194,8 +196,8 @@ sub fileTestRun
                         next;
                     }
 
-                    confess "error raised: " . $@ . "\n";
-                }
+                    confess $EVAL_ERROR;
+                };
 
                 if ($bErrorExpected)
                 {
@@ -245,13 +247,12 @@ sub fileTestRun
         for (my $bRemote = 0; $bRemote <= 0; $bRemote++)
         {
             # Create the file object
-            my $oFile = (new pgBackRest::File
+            my $oFile = new pgBackRest::File
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
-            ))->clone(1);
+            );
 
             # Loop through source exists
             for (my $bSourceExists = 0; $bSourceExists <= 1; $bSourceExists++)
@@ -302,17 +303,17 @@ sub fileTestRun
                 eval
                 {
                     $oFile->move(PATH_BACKUP_ABSOLUTE, $strSourceFile, PATH_BACKUP_ABSOLUTE, $strDestinationFile, $bCreate);
-                };
-
-                if ($@)
+                    return true;
+                }
+                or do
                 {
                     if (!$bSourceExists || (!$bDestinationExists && !$bCreate) || $bSourceError || $bDestinationError)
                     {
                         next;
                     }
 
-                    confess 'error raised: ' . $@ . "\n";
-                }
+                    confess $EVAL_ERROR;
+                };
 
                 if (!$bSourceExists || (!$bDestinationExists && !$bCreate) || $bSourceError || $bDestinationError)
                 {
@@ -350,7 +351,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -383,17 +383,17 @@ sub fileTestRun
                 eval
                 {
                     $oFile->compress(PATH_BACKUP_ABSOLUTE, $strFile);
-                };
-
-                if ($@)
+                    return true;
+                }
+                or do
                 {
                     if (!$bExists || $bError)
                     {
                         next;
                     }
 
-                    confess 'error raised: ' . $@ . "\n";
-                }
+                    confess $EVAL_ERROR;
+                };
 
                 if (!$bExists || $bError)
                 {
@@ -444,7 +444,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'db' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -515,7 +514,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -568,24 +566,24 @@ sub fileTestRun
                 }
 
                 # Execute in eval in case of error
-                my %oManifestHash;
+                my $hManifest;
                 my $bErrorExpected = !$bExists || $bError;
 
                 eval
                 {
-                    $oFile->manifest(PATH_BACKUP_ABSOLUTE, $strPath, \%oManifestHash);
-                };
-
+                    $hManifest = $oFile->manifest(PATH_BACKUP_ABSOLUTE, $strPath);
+                    return true;
+                }
                 # Check for an error
-                if ($@)
+                or do
                 {
                     if ($bErrorExpected)
                     {
                         next;
                     }
 
-                    confess 'error raised: ' . $@ . "\n";
-                }
+                    confess $EVAL_ERROR;
+                };
 
                 # Check for an expected error
                 if ($bErrorExpected)
@@ -596,7 +594,7 @@ sub fileTestRun
                 my $strManifest;
 
                 # Validate the manifest
-                foreach my $strName (sort(keys(%{$oManifestHash{name}})))
+                foreach my $strName (sort(keys(%{$hManifest})))
                 {
                     if (!defined($strManifest))
                     {
@@ -607,28 +605,28 @@ sub fileTestRun
                         $strManifest .= "\n";
                     }
 
-                    if (defined($oManifestHash{name}{"${strName}"}{inode}))
+                    if (defined($hManifest->{$strName}{inode}))
                     {
-                        $oManifestHash{name}{"${strName}"}{inode} = 0;
+                        $hManifest->{$strName}{inode} = 0;
                     }
 
                     $strManifest .=
                         "${strName}," .
-                        $oManifestHash{name}{"${strName}"}{type} . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{user}) ?
-                            $oManifestHash{name}{"${strName}"}{user} : '') . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{group}) ?
-                            $oManifestHash{name}{"${strName}"}{group} : '') . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{mode}) ?
-                            $oManifestHash{name}{"${strName}"}{mode} : '') . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{modification_time}) ?
-                            $oManifestHash{name}{"${strName}"}{modification_time} : '') . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{inode}) ?
-                            $oManifestHash{name}{"${strName}"}{inode} : '') . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{size}) ?
-                            $oManifestHash{name}{"${strName}"}{size} : '') . ',' .
-                        (defined($oManifestHash{name}{"${strName}"}{link_destination}) ?
-                            $oManifestHash{name}{"${strName}"}{link_destination} : '');
+                        $hManifest->{$strName}{type} . ',' .
+                        (defined($hManifest->{$strName}{user}) ?
+                            $hManifest->{$strName}{user} : '') . ',' .
+                        (defined($hManifest->{$strName}{group}) ?
+                            $hManifest->{$strName}{group} : '') . ',' .
+                        (defined($hManifest->{$strName}{mode}) ?
+                            $hManifest->{$strName}{mode} : '') . ',' .
+                        (defined($hManifest->{$strName}{modification_time}) ?
+                            $hManifest->{$strName}{modification_time} : '') . ',' .
+                        (defined($hManifest->{$strName}{inode}) ?
+                            $hManifest->{$strName}{inode} : '') . ',' .
+                        (defined($hManifest->{$strName}{size}) ?
+                            $hManifest->{$strName}{size} : '') . ',' .
+                        (defined($hManifest->{$strName}{link_destination}) ?
+                            $hManifest->{$strName}{link_destination} : '');
                 }
 
                 if ($strManifest ne $strManifestCompare)
@@ -659,7 +657,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -725,17 +722,17 @@ sub fileTestRun
                         eval
                         {
                             @stryFileList = $oFile->list(PATH_BACKUP_ABSOLUTE, $strPath, $strExpression, $strSort, $bIgnoreMissing);
-                        };
-
-                        if ($@)
+                            return true;
+                        }
+                        or do
                         {
                             if ($bErrorExpected)
                             {
                                 next;
                             }
 
-                            confess 'error raised: ' . $@ . "\n";
-                        }
+                            confess $EVAL_ERROR;
+                        };
 
                         if ($bErrorExpected)
                         {
@@ -787,7 +784,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -822,7 +818,7 @@ sub fileTestRun
                     }
                     else
                     {
-                        system("echo 'TESTDATA' > ${strFile}" . ($bTemp ? '.backrest.tmp' : ''));
+                        system("echo 'TESTDATA' > ${strFile}" . ($bTemp ? '.pgbackrest.tmp' : ''));
                     }
 
                     # Execute in eval in case of error
@@ -831,9 +827,9 @@ sub fileTestRun
                     eval
                     {
                         $bRemoved = $oFile->remove(PATH_BACKUP_ABSOLUTE, $strFile, $bTemp, $bIgnoreMissing);
-                    };
-
-                    if ($@)
+                        return true;
+                    }
+                    or do
                     {
                         if ($bError || $bRemote)
                         {
@@ -845,8 +841,8 @@ sub fileTestRun
                             next;
                         }
 
-                        confess 'unexpected error raised: ' . $@;
-                    }
+                        confess $EVAL_ERROR;
+                    };
 
                     if ($bError || $bRemote)
                     {
@@ -863,7 +859,7 @@ sub fileTestRun
                         confess 'remove returned false, but something should have been removed';
                     }
 
-                    if (-e ($strFile . ($bTemp ? '.backrest.tmp' : '')))
+                    if (-e ($strFile . ($bTemp ? '.pgbackrest.tmp' : '')))
                     {
                         confess 'file still exists';
                     }
@@ -892,7 +888,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -939,18 +934,18 @@ sub fileTestRun
 
                 eval
                 {
-                    ($strHash, $iSize) = $oFile->hashSize(PATH_BACKUP_ABSOLUTE, $strFile, $bCompressed)
-                };
-
-                if ($@)
+                    ($strHash, $iSize) = $oFile->hashSize(PATH_BACKUP_ABSOLUTE, $strFile, $bCompressed);
+                    return true;
+                }
+                or do
                 {
                     if ($bErrorExpected)
                     {
                         next;
                     }
 
-                    confess 'unexpected error raised: ' . $@;
-                }
+                    confess $EVAL_ERROR;
+                };
 
                 if ($bErrorExpected)
                 {
@@ -985,7 +980,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $bRemote ? 'backup' : undef,
                 $bRemote ? $oRemote : $oLocal
             );
 
@@ -1019,29 +1013,23 @@ sub fileTestRun
                         {
                             confess "bExists is set to ${bExists}, but exists() returned " . !$bExists;
                         }
-                    };
 
-                    if ($@)
+                        return true;
+                    }
+                    or do
                     {
-                        my $oMessage = $@;
+                        my $oException = $@;
                         my $iCode;
                         my $strMessage;
 
-                        if (blessed($oMessage))
+                        if (isException($oException))
                         {
-                            if ($oMessage->isa('pgBackRest::Common::Exception'))
-                            {
-                                $iCode = $oMessage->code();
-                                $strMessage = $oMessage->message();
-                            }
-                            else
-                            {
-                                confess 'unknown error object';
-                            }
+                            $iCode = $oException->code();
+                            $strMessage = $oException->message();
                         }
                         else
                         {
-                            $strMessage = $oMessage;
+                            $strMessage = $oException;
                         }
 
                         if ($bError)
@@ -1050,7 +1038,7 @@ sub fileTestRun
                         }
 
                         confess 'error raised: ' . $strMessage . "\n";
-                    }
+                    };
                 }
             }
         }
@@ -1086,7 +1074,6 @@ sub fileTestRun
             (
                 $strStanza,
                 $strTestPath,
-                $strRemote,
                 defined($strRemote) ? $oRemote : $oLocal
             );
 
@@ -1153,13 +1140,13 @@ sub fileTestRun
 
                         if ($iLarge < 3)
                         {
-                            executeTest('cp ' . testDataPath() . "/test.archive${iLarge}.bin ${strSourceFile}");
+                            executeTest('cp ' . testDataPath() . "/filecopy.archive${iLarge}.bin ${strSourceFile}");
                         }
                         else
                         {
                             for (my $iTableSizeIdx = 0; $iTableSizeIdx < 100; $iTableSizeIdx++)
                             {
-                                executeTest('cat ' . testDataPath() . "/test.table.bin >> ${strSourceFile}");
+                                executeTest('cat ' . testDataPath() . "/filecopy.table.bin >> ${strSourceFile}");
                             }
                         }
                     }
@@ -1206,8 +1193,6 @@ sub fileTestRun
 
                 # Run file copy in an eval block because some errors are expected
                 my $bReturn;
-                #
-                # exit;
 
                 eval
                 {
@@ -1217,32 +1202,26 @@ sub fileTestRun
                                      $bSourceCompressed, $bDestinationCompress,
                                      $bSourceIgnoreMissing, undef, '0770', false, undef, undef,
                                      $bChecksumAppend);
-                };
 
+                    return true;
+                }
                 # Check for errors after copy
-                if ($@)
+                or do
                 {
-                    my $oMessage = $@;
+                    my $oException = $EVAL_ERROR;
 
-                    if (blessed($oMessage))
+                    if (isException($oException))
                     {
-                        if ($oMessage->isa('pgBackRest::Common::Exception'))
+                        if ($bSourceMissing && !$bSourceIgnoreMissing)
                         {
-                            if ($bSourceMissing && !$bSourceIgnoreMissing)
-                            {
-                                next;
-                            }
+                            next;
+                        }
 
-                            confess $oMessage->message() . "\n" . $oMessage->trace();
-                        }
-                        else
-                        {
-                            confess 'unknown error object: ' . $oMessage;
-                        }
+                        confess $oException;
                     }
 
-                    confess $oMessage;
-                }
+                    confess $oException;
+                };
 
                 if ($bSourceMissing)
                 {
