@@ -9,6 +9,7 @@
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess longmess);
+use English '-no_match_vars';
 
 # Convert die to confess to capture the stack trace
 $SIG{__DIE__} = sub { Carp::confess @_ };
@@ -19,21 +20,23 @@ use Cwd qw(abs_path cwd);
 use Pod::Usage qw(pod2usage);
 use POSIX qw(ceil);
 use Time::HiRes qw(gettimeofday);
-use Scalar::Util qw(blessed);
 
+use lib dirname($0) . '/lib';
 use lib dirname($0) . '/../lib';
+use lib dirname($0) . '/../doc/lib';
+
+use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Common::Wait;
-use pgBackRest::Db;
+use pgBackRest::Config::Config;
+use pgBackRest::DbVersion;
 use pgBackRest::FileCommon;
 use pgBackRest::Version;
 
-use lib dirname($0) . '/../doc/lib';
 use BackRestDoc::Custom::DocCustomRelease;
 
-use lib dirname($0) . '/lib';
 use pgBackRestTest::Backup::BackupTest;
 use pgBackRestTest::Backup::Common::HostBackupTest;
 use pgBackRestTest::Backup::Common::HostBaseTest;
@@ -43,6 +46,7 @@ use pgBackRestTest::Common::ContainerTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::HostGroupTest;
 use pgBackRestTest::Common::ListTest;
+use pgBackRestTest::Common::VmTest;
 use pgBackRestTest::CommonTest;
 use pgBackRestTest::Config::ConfigTest;
 use pgBackRestTest::File::FileTest;
@@ -64,12 +68,13 @@ test.pl [options]
    --module             test module to execute
    --test               execute the specified test in a module
    --run                execute only the specified test run
-   --thread-max         max threads to run for backup/restore (default 4)
+   --process-max        max processes to run for compression/transfer (default 1)
    --dry-run            show only the tests that would be executed but don't execute them
    --no-cleanup         don't cleaup after the last test is complete - useful for debugging
    --db-version         version of postgres to test (all, defaults to minimal)
    --log-force          force overwrite of current test log files
    --no-lint            Disable static source code analysis
+   --libc-only          Compile the C library and run tests only
 
  Configuration Options:
    --psql-bin           path to the psql executables (e.g. /usr/lib/postgresql/9.3/bin/)
@@ -82,7 +87,7 @@ test.pl [options]
    --vm-build           build Docker containers
    --vm-force           force a rebuild of Docker containers
    --vm-out             Show VM output (default false)
-   --process-max        max VMs to run in parallel (default 1)
+   --vm-max             max VMs to run in parallel (default 1)
 
  General Options:
    --version            display version and exit
@@ -97,9 +102,9 @@ my $bVmOut = false;
 my $strModule = 'all';
 my $strModuleTest = 'all';
 my $iModuleTestRun = undef;
-my $iThreadMax = undef;
-my $iProcessMax = 1;
-my $iProcessId = undef;
+my $iProcessMax = undef;
+my $iVmMax = 1;
+my $iVmId = undef;
 my $bDryRun = false;
 my $bNoCleanup = false;
 my $strPgSqlBin;
@@ -113,6 +118,7 @@ my $strVm = 'all';
 my $bVmBuild = false;
 my $bVmForce = false;
 my $bNoLint = false;
+my $bLibCOnly = false;
 
 GetOptions ('q|quiet' => \$bQuiet,
             'version' => \$bVersion,
@@ -127,93 +133,96 @@ GetOptions ('q|quiet' => \$bQuiet,
             'module=s' => \$strModule,
             'test=s' => \$strModuleTest,
             'run=s' => \$iModuleTestRun,
-            'thread-max=s' => \$iThreadMax,
-            'process-id=s' => \$iProcessId,
             'process-max=s' => \$iProcessMax,
+            'vm-id=s' => \$iVmId,
+            'vm-max=s' => \$iVmMax,
             'dry-run' => \$bDryRun,
             'no-cleanup' => \$bNoCleanup,
             'db-version=s' => \$strDbVersion,
             'log-force' => \$bLogForce,
-            'no-lint' => \$bNoLint)
+            'no-lint' => \$bNoLint,
+            'libc-only' => \$bLibCOnly)
     or pod2usage(2);
 
-# Display version and exit if requested
-if ($bVersion || $bHelp)
+####################################################################################################################################
+# Run in eval block to catch errors
+####################################################################################################################################
+eval
 {
-    syswrite(*STDOUT, BACKREST_NAME . ' ' . BACKREST_VERSION . " Test Engine\n");
-
-    if ($bHelp)
+    # Display version and exit if requested
+    if ($bVersion || $bHelp)
     {
-        syswrite(*STDOUT, "\n");
+        syswrite(*STDOUT, BACKREST_NAME . ' ' . BACKREST_VERSION . " Test Engine\n");
+
+        if ($bHelp)
+        {
+            syswrite(*STDOUT, "\n");
+            pod2usage();
+        }
+
+        exit 0;
+    }
+
+    if (@ARGV > 0)
+    {
+        syswrite(*STDOUT, "invalid parameter\n\n");
         pod2usage();
     }
 
-    exit 0;
-}
+    ################################################################################################################################
+    # Setup
+    ################################################################################################################################
+    # Set a neutral umask so tests work as expected
+    umask(0);
 
-if (@ARGV > 0)
-{
-    syswrite(*STDOUT, "invalid parameter\n\n");
-    pod2usage();
-}
+    # Set console log level
+    if ($bQuiet)
+    {
+        $strLogLevel = 'off';
+    }
 
-####################################################################################################################################
-# Setup
-####################################################################################################################################
-# Set a neutral umask so tests work as expected
-umask(0);
+    logLevelSet(uc($strLogLevel), uc($strLogLevel), OFF);
 
-# Set console log level
-if ($bQuiet)
-{
-    $strLogLevel = 'off';
-}
+    if ($strModuleTest ne 'all' && $strModule eq 'all')
+    {
+        confess "--module must be provided for --test=\"${strModuleTest}\"";
+    }
 
-logLevelSet(uc($strLogLevel), uc($strLogLevel));
+    if (defined($iModuleTestRun) && $strModuleTest eq 'all')
+    {
+        confess "--test must be provided for --run=\"${iModuleTestRun}\"";
+    }
 
-if ($strModuleTest ne 'all' && $strModule eq 'all')
-{
-    confess "--module must be provided for --test=\"${strModuleTest}\"";
-}
+    # Check process total
+    if (defined($iProcessMax) && ($iProcessMax < 1 || $iProcessMax > OPTION_DEFAULT_PROCESS_MAX_MAX))
+    {
+        confess 'process-max must be between 1 and ' . OPTION_DEFAULT_PROCESS_MAX_MAX;
+    }
 
-if (defined($iModuleTestRun) && $strModuleTest eq 'all')
-{
-    confess "--test must be provided for --run=\"${iModuleTestRun}\"";
-}
+    # Set test path if not expicitly set
+    if (!defined($strTestPath))
+    {
+        $strTestPath = cwd() . '/test';
+    }
 
-# Check thread total
-if (defined($iThreadMax) && ($iThreadMax < 1 || $iThreadMax > 32))
-{
-    confess 'thread-max must be between 1 and 32';
-}
+    # Get the base backrest path
+    my $strBackRestBase = dirname(dirname(abs_path($0)));
 
-# Set test path if not expicitly set
-if (!defined($strTestPath))
-{
-    $strTestPath = cwd() . '/test';
-}
+    ################################################################################################################################
+    # Build Docker containers
+    ################################################################################################################################
+    if ($bVmBuild)
+    {
+        containerBuild($strVm, $bVmForce, $strDbVersion);
+        exit 0;
+    }
 
-# Get the base backrest path
-my $strBackRestBase = dirname(dirname(abs_path($0)));
-
-####################################################################################################################################
-# Build Docker containers
-####################################################################################################################################
-if ($bVmBuild)
-{
-    containerBuild($strVm, $bVmForce, $strDbVersion);
-    exit 0;
-}
-
-eval
-{
     ################################################################################################################################
     # Start VM and run
     ################################################################################################################################
-    if (!defined($iProcessId))
+    if (!defined($iVmId))
     {
         # Load the doc module dynamically since it is not supported on all systems
-        use lib dirname(abs_path($0)) . '/../doc/lib';
         require BackRestDoc::Common::Doc;
         BackRestDoc::Common::Doc->import();
 
@@ -237,7 +246,7 @@ eval
         if (!$bDryRun)
         {
             # Run Perl critic
-            if (!$bNoLint)
+            if (!$bNoLint && !$bLibCOnly)
             {
                 my $strBasePath = dirname(dirname(abs_path($0)));
 
@@ -264,9 +273,51 @@ eval
             logFileSet(cwd() . "/test");
         }
 
+        # Build the C Library in host
+        #-----------------------------------------------------------------------------------------------------------------------
+        {
+            my $bLogDetail = $strLogLevel eq 'detail';
+            my $strBuildPath = "${strBackRestBase}/test/.vagrant/libc/host";
+
+            &log(INFO, "Build/test/install C library for host (${strBuildPath})");
+
+            filePathCreate($strBuildPath, undef, true, true);
+            executeTest("cp -rp ${strBackRestBase}/libc/* ${strBuildPath}");
+
+            executeTest(
+                "cd ${strBuildPath} && perl Makefile.PL INSTALLMAN1DIR=none INSTALLMAN3DIR=none",
+                {bShowOutputAsync => $bLogDetail});
+            executeTest("make -C ${strBuildPath}", {bSuppressStdErr => true, bShowOutputAsync => $bLogDetail});
+            executeTest("sudo make -C ${strBuildPath} install", {bShowOutputAsync => $bLogDetail});
+            executeTest("cd ${strBuildPath} && perl t/pgBackRest-LibC.t", {bShowOutputAsync => $bLogDetail});
+
+            # Load the module dynamically
+            require pgBackRest::LibC;
+            pgBackRest::LibC->import(qw(:debug));
+
+            # Do a basic test to make sure it installed correctly
+            if (&UVSIZE != 8)
+            {
+                confess &log(ERROR, 'UVSIZE in C library does not equal 8');
+            }
+
+            # Also check the version number
+            my $strLibCVersion =
+                BACKREST_VERSION =~ /dev$/ ?
+                    substr(BACKREST_VERSION, 0, length(BACKREST_VERSION) - 3) . '.999' : BACKREST_VERSION;
+
+            if (libCVersion() ne $strLibCVersion)
+            {
+                confess &log(ERROR, $strLibCVersion . ' was expected for LibC version but found ' . libCVersion());
+            }
+
+            # Exit if only testing the C library
+            exit 0 if $bLibCOnly;
+        }
+
         # Determine which tests to run
         #-----------------------------------------------------------------------------------------------------------------------
-        my $oyTestRun = testListGet($strVm, $strModule, $strModuleTest, $iModuleTestRun, $strDbVersion, $iThreadMax);
+        my $oyTestRun = testListGet($strVm, $strModule, $strModuleTest, $iModuleTestRun, $strDbVersion, $iProcessMax);
 
         if (@{$oyTestRun} == 0)
         {
@@ -287,43 +338,78 @@ eval
 
         if (!$bDryRun || $bVmOut)
         {
-            containerRemove('test-[0-9]+');
+            containerRemove('test-([0-9]+|build)');
 
-            for (my $iProcessIdx = 0; $iProcessIdx < 8; $iProcessIdx++)
+            for (my $iVmIdx = 0; $iVmIdx < 8; $iVmIdx++)
             {
                 push(@{$oyProcess}, undef);
             }
 
             executeTest("sudo rm -rf ${strTestPath}/*");
-            filePathCreate($strTestPath);
+            filePathCreate($strTestPath, undef, true);
+        }
+
+        # Build the C Library in container
+        #-----------------------------------------------------------------------------------------------------------------------
+        {
+            my $bLogDetail = $strLogLevel eq 'detail';
+            my @stryBuildVm = $strVm eq 'all' ? (VM_CO6, VM_U16, VM_D8, VM_CO7, VM_U14, VM_U12) : ($strVm);
+
+            foreach my $strBuildVM (sort(@stryBuildVm))
+            {
+                executeTest(
+                    "docker run -itd -h test-build --name=test-build" .
+                    " -v ${strBackRestBase}:${strBackRestBase} " . containerNamespace() . "/${strBuildVM}-build");
+
+                my $strBuildPath = "${strBackRestBase}/test/.vagrant/libc/${strBuildVM}";
+                &log(INFO, "Build/test C library for ${strBuildVM} (${strBuildPath})");
+
+                filePathCreate($strBuildPath, undef, true, true);
+                executeTest("cp -rp ${strBackRestBase}/libc/* ${strBuildPath}");
+
+                executeTest(
+                    "docker exec -i test-build " .
+                    "bash -c 'cd ${strBuildPath} && perl Makefile.PL INSTALLMAN1DIR=none INSTALLMAN3DIR=none'");
+                executeTest(
+                    "docker exec -i test-build " .
+                    "make -C ${strBuildPath}", {bSuppressStdErr => true});
+                executeTest(
+                    "docker exec -i test-build " .
+                    "make -C ${strBuildPath} test");
+                executeTest(
+                    "docker exec -i test-build " .
+                    "make -C ${strBuildPath} install", {bShowOutputAsync => $bLogDetail});
+
+                executeTest("docker rm -f test-build");
+            }
         }
 
         if ($bDryRun)
         {
-            $iProcessMax = 1;
+            $iVmMax = 1;
         }
 
         my $iTestIdx = 0;
-        my $iProcessTotal;
+        my $iVmTotal;
         my $iTestMax = @{$oyTestRun};
         my $lStartTime = time();
-        my $bShowOutputAsync = $bVmOut && (@{$oyTestRun} == 1 || $iProcessMax == 1) && ! $bDryRun ? true : false;
+        my $bShowOutputAsync = $bVmOut && (@{$oyTestRun} == 1 || $iVmMax == 1) && ! $bDryRun ? true : false;
 
         do
         {
             do
             {
-                $iProcessTotal = 0;
+                $iVmTotal = 0;
 
-                for (my $iProcessIdx = 0; $iProcessIdx < $iProcessMax; $iProcessIdx++)
+                for (my $iVmIdx = 0; $iVmIdx < $iVmMax; $iVmIdx++)
                 {
-                    if (defined($$oyProcess[$iProcessIdx]))
+                    if (defined($$oyProcess[$iVmIdx]))
                     {
-                        my $oExecDone = $$oyProcess[$iProcessIdx]{exec};
-                        my $strTestDone = $$oyProcess[$iProcessIdx]{test};
-                        my $iTestDoneIdx = $$oyProcess[$iProcessIdx]{idx};
+                        my $oExecDone = $$oyProcess[$iVmIdx]{exec};
+                        my $strTestDone = $$oyProcess[$iVmIdx]{test};
+                        my $iTestDoneIdx = $$oyProcess[$iVmIdx]{idx};
 
-                        my $iExitStatus = $oExecDone->end(undef, $iProcessMax == 1);
+                        my $iExitStatus = $oExecDone->end(undef, $iVmMax == 1);
 
                         if (defined($iExitStatus))
                         {
@@ -332,7 +418,7 @@ eval
                                 syswrite(*STDOUT, "\n");
                             }
 
-                            my $fTestElapsedTime = ceil((gettimeofday() - $$oyProcess[$iProcessIdx]{start_time}) * 100) / 100;
+                            my $fTestElapsedTime = ceil((gettimeofday() - $$oyProcess[$iVmIdx]{start_time}) * 100) / 100;
 
                             if (!($iExitStatus == 0 || $iExitStatus == 255))
                             {
@@ -350,53 +436,53 @@ eval
 
                             if (!$bNoCleanup)
                             {
-                                my $strImage = 'test-' . $iProcessIdx;
+                                my $strImage = 'test-' . $iVmIdx;
                                 my $strHostTestPath = "${strTestPath}/${strImage}";
 
-                                containerRemove("test-${iProcessIdx}");
+                                containerRemove("test-${iVmIdx}");
                                 executeTest("sudo rm -rf ${strHostTestPath}");
                             }
 
-                            $$oyProcess[$iProcessIdx] = undef;
+                            $$oyProcess[$iVmIdx] = undef;
                         }
                         else
                         {
-                            $iProcessTotal++;
+                            $iVmTotal++;
                         }
                     }
                 }
 
-                if ($iProcessTotal == $iProcessMax)
+                if ($iVmTotal == $iVmMax)
                 {
                     waitHiRes(.1);
                 }
             }
-            while ($iProcessTotal == $iProcessMax);
+            while ($iVmTotal == $iVmMax);
 
-            for (my $iProcessIdx = 0; $iProcessIdx < $iProcessMax; $iProcessIdx++)
+            for (my $iVmIdx = 0; $iVmIdx < $iVmMax; $iVmIdx++)
             {
-                if (!defined($$oyProcess[$iProcessIdx]) && $iTestIdx < @{$oyTestRun})
+                if (!defined($$oyProcess[$iVmIdx]) && $iTestIdx < @{$oyTestRun})
                 {
                     my $oTest = $$oyTestRun[$iTestIdx];
                     $iTestIdx++;
 
-                    my $strTest = sprintf('P%0' . length($iProcessMax) . 'd-T%0' . length($iTestMax) . 'd/%0' .
-                                          length($iTestMax) . "d - ", $iProcessIdx, $iTestIdx, $iTestMax) .
+                    my $strTest = sprintf('P%0' . length($iVmMax) . 'd-T%0' . length($iTestMax) . 'd/%0' .
+                                          length($iTestMax) . "d - ", $iVmIdx, $iTestIdx, $iTestMax) .
                                           'vm=' . $$oTest{&TEST_VM} .
                                           ', module=' . $$oTest{&TEST_MODULE} .
                                           ', test=' . $$oTest{&TEST_NAME} .
                                           (defined($$oTest{&TEST_RUN}) ? ', run=' . $$oTest{&TEST_RUN} : '') .
-                                          (defined($$oTest{&TEST_THREAD}) ? ', thread-max=' . $$oTest{&TEST_THREAD} : '') .
+                                          (defined($$oTest{&TEST_PROCESS}) ? ', process-max=' . $$oTest{&TEST_PROCESS} : '') .
                                           (defined($$oTest{&TEST_DB}) ? ', db=' . $$oTest{&TEST_DB} : '');
 
-                    my $strImage = 'test-' . $iProcessIdx;
+                    my $strImage = 'test-' . $iVmIdx;
                     my $strDbVersion = (defined($$oTest{&TEST_DB}) ? $$oTest{&TEST_DB} : PG_VERSION_94);
                     $strDbVersion =~ s/\.//;
 
                     &log($bDryRun && !$bVmOut || $bShowOutputAsync ? INFO : DETAIL, "${strTest}" .
                          ($bVmOut || $bShowOutputAsync ? "\n" : ''));
 
-                    my $strVmTestPath = "/home/vagrant/test/${strImage}";
+                    my $strVmTestPath = '/home/' . TEST_USER . "/test/${strImage}";
                     my $strHostTestPath = "${strTestPath}/${strImage}";
 
                     # Don't create the container if this is a dry run unless output from the VM is required.  Ouput can be requested
@@ -411,20 +497,21 @@ eval
                             executeTest(
                                 'docker run -itd -h ' . $$oTest{&TEST_VM} . "-test --name=${strImage}" .
                                 " -v ${strHostTestPath}:${strVmTestPath}" .
-                                " -v ${strBackRestBase}:${strBackRestBase} backrest/" . $$oTest{&TEST_VM} . "-loop-test-pre");
+                                " -v ${strBackRestBase}:${strBackRestBase} " . containerNamespace() . '/' . $$oTest{&TEST_VM} .
+                                "-loop-test-pre");
                         }
                     }
 
                     my $strCommand =
-                        ($$oTest{&TEST_CONTAINER} ? "docker exec -i -u vagrant ${strImage} " : '') . abs_path($0) .
+                        ($$oTest{&TEST_CONTAINER} ? 'docker exec -i -u ' . TEST_USER . " ${strImage} " : '') . abs_path($0) .
                         " --test-path=${strVmTestPath}" .
                         " --vm=$$oTest{&TEST_VM}" .
-                        " --process-id=${iProcessIdx}" .
+                        " --vm-id=${iVmIdx}" .
                         " --module=" . $$oTest{&TEST_MODULE} .
                         ' --test=' . $$oTest{&TEST_NAME} .
                         (defined($$oTest{&TEST_RUN}) ? ' --run=' . $$oTest{&TEST_RUN} : '') .
                         (defined($$oTest{&TEST_DB}) ? ' --db-version=' . $$oTest{&TEST_DB} : '') .
-                        (defined($$oTest{&TEST_THREAD}) ? ' --thread-max=' . $$oTest{&TEST_THREAD} : '') .
+                        (defined($$oTest{&TEST_PROCESS}) ? ' --process-max=' . $$oTest{&TEST_PROCESS} : '') .
                         ($strLogLevel ne lc(INFO) ? " --log-level=${strLogLevel}" : '') .
                         ' --pgsql-bin=' . $$oTest{&TEST_PGSQL_BIN} .
                         ($bLogForce ? ' --log-force' : '') .
@@ -438,13 +525,11 @@ eval
                     {
                         my $fTestStartTime = gettimeofday();
 
-                        filePathCreate($strVmTestPath, '0777', undef, true);
-
                         # Set permissions on the Docker test directory.  This can be removed once users/groups are sync'd between
                         # Docker and the host VM.
                         if ($$oTest{&TEST_CONTAINER})
                         {
-                            executeTest("docker exec ${strImage} chown vagrant:postgres -R ${strVmTestPath}");
+                            executeTest("docker exec ${strImage} chown " . TEST_USER . ":postgres -R ${strVmTestPath}");
                         }
 
                         my $oExec = new pgBackRestTest::Common::ExecuteTest(
@@ -462,14 +547,14 @@ eval
                             start_time => $fTestStartTime
                         };
 
-                        $$oyProcess[$iProcessIdx] = $oProcess;
+                        $$oyProcess[$iVmIdx] = $oProcess;
                     }
 
-                    $iProcessTotal++;
+                    $iVmTotal++;
                 }
             }
         }
-        while ($iProcessTotal > 0);
+        while ($iVmTotal > 0);
 
         # Print test info and exit
         #-----------------------------------------------------------------------------------------------------------------------
@@ -494,11 +579,11 @@ eval
     # Set parameters in host group
     my $oHostGroup = hostGroupGet();
     $oHostGroup->paramSet(HOST_PARAM_VM, $strVm);
-    $oHostGroup->paramSet(HOST_PARAM_PROCESS_ID, $iProcessId);
+    $oHostGroup->paramSet(HOST_PARAM_VM_ID, $iVmId);
     $oHostGroup->paramSet(HOST_PARAM_TEST_PATH, $strTestPath);
     $oHostGroup->paramSet(HOST_PARAM_BACKREST_EXE, "${strBackRestBase}/bin/pgbackrest");
-    $oHostGroup->paramSet(HOST_PARAM_THREAD_MAX, $iThreadMax);
-    $oHostGroup->paramSet(HOST_DB_MASTER_USER, 'vagrant');
+    $oHostGroup->paramSet(HOST_PARAM_PROCESS_MAX, $iProcessMax);
+    $oHostGroup->paramSet(HOST_DB_USER, TEST_USER);
     $oHostGroup->paramSet(HOST_BACKUP_USER, 'backrest');
 
     if ($strDbVersion ne 'minimal')
@@ -546,24 +631,29 @@ eval
             executeTest("sudo rm -rf ${strTestPath}");
         }
     }
-};
 
-if ($@)
-{
-    my $oMessage = $@;
-
-    # If a backrest exception then return the code - don't confess
-    if (blessed($oMessage) && $oMessage->isa('pgBackRest::Common::Exception'))
+    if (!$bDryRun && !$bVmOut)
     {
-        syswrite(*STDOUT, $oMessage->trace());
-        exit $oMessage->code();
+        &log(INFO, 'TESTS COMPLETED SUCCESSFULLY (DESPITE ANY ERROR MESSAGES YOU SAW)');
     }
 
-    syswrite(*STDOUT, $oMessage);
-    exit 250;
+    # Exit with success
+    exit 0;
 }
 
-if (!$bDryRun && !$bVmOut)
+####################################################################################################################################
+# Check for errors
+####################################################################################################################################
+or do
 {
-    &log(INFO, 'TESTS COMPLETED SUCCESSFULLY (DESPITE ANY ERROR MESSAGES YOU SAW)');
-}
+    # If a backrest exception then return the code
+    exit $EVAL_ERROR->code() if (isException($EVAL_ERROR));
+
+    # Else output the unhandled error
+    print $EVAL_ERROR;
+    exit ERROR_UNHANDLED;
+};
+
+# It shouldn't be possible to get here
+&log(ASSERT, 'execution reached invalid location in ' . __FILE__ . ', line ' . __LINE__);
+exit ERROR_ASSERT;
